/**
 * \file AngleTrk.c
 * \brief Angle-tracking observer for sin/cos analog position sensor
 *
 * \license
 * You can use this file under the terms of the IFX License.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * IFX License for more details (IFX_License.txt).
 *
 * This file may be used, copied, and distributed, with or without
 * modification, provided that all copyright notices are retained;
 * that all modifications to this file are prominently noted in the
 * modified file; and that this paragraph is not modified.
 *
 * \copyright Copyright (C) 2012 Infineon Technologies AG
 *
 *
 * $Revision: $1381.3
 * $Date: $2013-06-24 18:38:08 GMT
 *
 * FIXME this is not an interface, it should be moved to the SysSe library
 */

//________________________________________________________________________________________
// INCLUDES

#include "AngleTrk.h"
#include <math.h>

//________________________________________________________________________________________
// LOCAL DEFINITIONS

#define TO_TICK ((float32)(ANGLETRK_RESOLUTION / 2) * PI)
#define TO_RAD  ((float32)PI / (float32)(ANGLETRK_RESOLUTION / 2))
#define ATAN2F  AngleTrk_atan2

//________________________________________________________________________________________
// LOCAL FUNCTION PROTOTYPES

IFX_EXTERN float32 AngleTrk_atan2(float32 y, float32 x);

//________________________________________________________________________________________
// GLOBAL VARIABLES

//________________________________________________________________________________________
// LOCAL VARIABLES

const PosIf_Calls AngleTrk_If_functions = {
    (PosIf_ResetOffset) & AngleTrk_resetOffset,
    (PosIf_SetOffset) & AngleTrk_setOffset,
    (PosIf_SetupElAngleConst) & AngleTrk_setupElAngleConst,
    (PosIf_SetupSpeedConst) & PosIf_setupSpeedConst_base,
    (PosIf_UpdatePosition) & AngleTrk_update,
    (PosIf_UpdateSpeed) & AngleTrk_getSpeed,
};

//________________________________________________________________________________________
// LOCAL FUNCTION IMPLEMENTATIONS

IFX_INLINE float32 AngleTrk_bound(float32 angle, float32 periodPerRotation)
{
    float32 fullPeriod = 2 * PI;    //((ANGLETRK_RESOLUTION/2)*periodPerRotation)

    if (angle >= (fullPeriod / 2))
    {
        angle = angle - fullPeriod;
    }
    else if (angle < (-fullPeriod / 2))
    {
        angle = angle + fullPeriod;
    }
    else
    {}

    return angle;
}


IFX_INLINE float32 AngleTrk_boundInput(float32 input)
{
    return AngleTrk_bound(input, 1);
}


IFX_INLINE float32 round_f(float32 val)
{
    /*lint -e632 -e633 -e524 */
    sint32 val_i = (sint32)val;

    /*lint +e632 +e633 +e524 */
    if ((val - (float32)val_i) > 0.5)
    {
        val = (float32)(val_i + 1);
    }
    else
    {
        val = (float32)val_i;
    }

    return val;
}


//________________________________________________________________________________________
// PUBLIC FUNCTION IMPLEMENTATIONS
#define ATO_T   (15e-3)
#define ATO_K   (39.05)
#define ATO_PSI (1.5 * PI)

void AngleTrk_setControlGains(AngleTrk *aObsv, float32 K, float32 T, float32 psi)
{
    aObsv->cfg.Kp = ((2 * K) + (psi * psi) + 1) / (T * T);
    aObsv->cfg.Ki = K * ((psi * psi) + 1) / (T * T * T);
    aObsv->cfg.Kd = (K + 2) / T;
}


/** \brief Fill config with default values */
void AngleTrk_initConfig(AngleTrk_Config *config, sint16 *sinIn, sint16 *cosIn)
{
    AngleTrk aObsv;

    AngleTrk_setControlGains(&aObsv, ATO_K, ATO_T, ATO_PSI);
    config->Kp                = aObsv.cfg.Kp;
    config->Ki                = aObsv.cfg.Ki;
    config->Kd                = aObsv.cfg.Kd;
    config->sinIn             = sinIn;
    config->cosIn             = cosIn;
    config->errorThreshold    = 5.0 / 180 * PI;
    config->sqrAmplMax        = 1.01 * 1.01;
    config->sqrAmplMin        = 0.99 * 0.99;
    config->speedLpfFc        = 100;
    config->periodPerRotation = 1;
    config->reversed          = FALSE;
    config->offset            = 0;
}


/** \brief Initialize the Angle Tracking object
 * \param aObsv Pointer to the AngleTrk object
 * \param config Pointer to the configuration data
 * \param Ts sampling period in seconds
 */
void AngleTrk_init(AngleTrk *aObsv, const AngleTrk_Config *config, float32 Ts)
{
    PosIf_Config baseCfg;

    baseCfg.offset            = config->offset;
    baseCfg.resolution        = ANGLETRK_RESOLUTION;
    baseCfg.periodPerRotation = config->periodPerRotation;
    baseCfg.reversed          = config->reversed;
    PosIf_init(&aObsv->base, &baseCfg);
    aObsv->base.functions     = &AngleTrk_If_functions;

    if (!__neqf(config->Kp, 0) && !__neqf(config->Ki, 0) && !__neqf(config->Kp, 0))
    {                           /* all gains are zero, use default */
        AngleTrk_setControlGains(aObsv, ATO_K, ATO_T, ATO_PSI);
    }

    aObsv->cfg      = *config;
    aObsv->Ts       = Ts;
    aObsv->halfTs   = Ts / (float32)2.0;
    aObsv->angleEst = (float32)0;
    aObsv->accelEst = (float32)0;
    aObsv->speedEst = (float32)0;
    {
        LowPass_PT1_Config lpfConfig;
        lpfConfig.K         = 1.0;
        lpfConfig.T         = 1 / (2 * PI * config->speedLpfFc);
        lpfConfig.shiftCoef = 0;
        LowPass_PT1_init(&aObsv->speedLpf, &lpfConfig, Ts);
    }

#if ANGLETRK_TEST_MODE
    AngleGen_init(&aObsv->stimuli.gen, Ts);
#endif
}


/** \brief Step function of AngleTrk should be called at every sampling period provided
 * during initialisation (call to AngleTrk_init()).
 *
 * \note This function is automatically called by AngleTrk_update()
 * \param sinIn sine input signal. The offset shall be zero.
 * \param cosIn cosine input signal. The offset shall be zero.
 * \return the angular position in radians
 * \note The amplitude of both sinIn and cosIn signal shall be the same.
 */
float32 AngleTrk_step(AngleTrk *aObsv, sint32 sinIn, sint32 cosIn, float32 phase)
{
    float32 angleEst, dAccel, dSpeed, dAngle;

#if ANGLETRK_TEST_MODE
    sint32  angle = AngleGen_step(&aObsv->stimuli.gen, aObsv->stimuli.speed);
    sinIn = (2048.0 * LookUp_sin(angle));
    cosIn = (2048.0 * LookUp_cos(angle));
#endif

    if (aObsv->base.reversed != FALSE)
    {
        aObsv->angleRef = ATAN2F((float32)cosIn, (float32)sinIn) + phase;
    }
    else
    {
        aObsv->angleRef = ATAN2F((float32)sinIn, (float32)cosIn) + phase;
    }

    // Error:
    aObsv->error = AngleTrk_boundInput(aObsv->angleRef - aObsv->angleEst);

    // Acceleration, zero-order-hold integrator:
    dAccel          = aObsv->cfg.Ki * aObsv->error;
    aObsv->accelEst = aObsv->accelEst + (dAccel * aObsv->Ts);

    // Speed, zero-order-hold integrator:
    dSpeed          = ((aObsv->cfg.Kp * aObsv->error) + aObsv->accelEst);
    aObsv->speedEst = aObsv->speedEst + (dSpeed * aObsv->Ts);

    // Angle, first-order-hold integrator:
    dAngle          = ((aObsv->cfg.Kd * aObsv->error) + aObsv->speedEst);
    angleEst        = aObsv->angleEst + ((dAngle + aObsv->dAngle) * aObsv->halfTs);
    aObsv->dAngle   = dAngle;

    aObsv->angleEst = AngleTrk_boundInput(angleEst);

    // Filter speed:
    aObsv->base.speed = LowPass_PT1_do(&aObsv->speedLpf, aObsv->speedEst) * aObsv->base.speedConst;

    return aObsv->angleEst;
}


/** \brief Set the position offset (in ticks)
 * \param aObsv Pointer to the AngleTrk object
 * \param offset Position offset in ticks */
void AngleTrk_setOffset(AngleTrk *aObsv, PosIf_Raw offset)
{
    PosIf *base = &aObsv->base;

    base->offset = offset;
}


/** \brief Update the status flags.
 * \param aObsv Pointer to the AngleTrk object
 * \note This function is automatically called by AngleTrk_update()
 */
void AngleTrk_updateStatus(AngleTrk *aObsv, sint32 sinIn, sint32 cosIn)
{
    PosIf *base    = &aObsv->base;
    sint32 sqrAmpl = (sinIn * sinIn) + (cosIn * cosIn);

    base->status.B.signalLoss        = (sqrAmpl < aObsv->cfg.sqrAmplMin);
    base->status.B.signalDegradation = (sqrAmpl > aObsv->cfg.sqrAmplMax);
    base->status.B.trackingLoss      = __absf(aObsv->error) > aObsv->cfg.errorThreshold;
}


/** \brief Update the object directly from its analog input
 * \param aObsv Pointer to the AngleTrk object
 * \return the angular position in ticks [0 .. ANGLETRK_RESOLUTION-1] (represents
 * 0..360 degree) */
sint32 AngleTrk_update(AngleTrk *aObsv)
{
    PosIf    *base        = &aObsv->base;
    float32   angleEst    = AngleTrk_step(aObsv, *aObsv->cfg.sinIn, *aObsv->cfg.cosIn, 0);

    PosIf_Raw newPosition = (PosIf_Raw)(angleEst * (ANGLETRK_RESOLUTION / 2) / PI);

    newPosition    = newPosition & (ANGLETRK_RESOLUTION - 1);
    base->position = newPosition;

    AngleTrk_updateStatus(aObsv, *aObsv->cfg.sinIn, *aObsv->cfg.cosIn);

    return newPosition;
}


/** \brief Reset the position offset into actual value
 * \param aObsv Pointer to the AngleTrk object
 * \returns the new obtained offset (in ticks) */
sint32 AngleTrk_resetOffset(AngleTrk *aObsv)
{
    PosIf    *base = &aObsv->base;
    PosIf_Raw offset;

    offset = (base->position - base->offset);
    /* -e639 Allow operation for fast rounding operation */
    offset = -(PosIf_Raw)(offset & (ANGLETRK_RESOLUTION - 1));
    /* +e639 */
    AngleTrk_setOffset(aObsv, offset);
    offset = base->offset;

    return offset;
}


/** \brief Given the motor pole pairs, the constant for calculating electrical angle is
 * computed by this function
 * \param aObsv Pointer to the AngleTrk object
 * \param motorPolePairs Number of motor pole-pairs
 */
void AngleTrk_setupElAngleConst(AngleTrk *aObsv, sint32 motorPolePairs)
{
    PosIf *base = &aObsv->base;

    base->elAngleConst = (float32)motorPolePairs *
                         ((float32)POSIF_ANGLE_RANGE / (base->periodPerRotation * ANGLETRK_RESOLUTION));
}


/** \brief Returns the actual position
 * \param aObsv Pointer to the AngleTrk object
 */
PosIf_Raw AngleTrk_getPosition(AngleTrk *aObsv)
{
    PosIf *base = &aObsv->base;

    return base->position;
}


/** \brief Returns the actual speed
 * \param aObsv Pointer to the AngleTrk object
 * \param update (ignored)
 */
StdReal AngleTrk_getSpeed(AngleTrk *aObsv, boolean update)
{
    PosIf *base = &aObsv->base;

    return base->speed;
}


#define ANGLETRK_ATAN_SIZE 1024
#ifndef PI
#define PI                 (3.1415926535897932384626433832795)
#endif

const float32 AngleTrk_atan2Table[ANGLETRK_ATAN_SIZE + 1] =
{0,                   0.000976562189559319, 0.00195312251647882, 0.00292967911813999, 0.00390623013196697, 0.00488277369544783,
 0.00585930794615589, 0.00683583102177106,  0.00781234106010111, 0.008788836199103,
 0.00976531457690415, 0.0107417743318238,   0.0117182136023941,  0.0126946305273818,  0.0136710232458091,
 0.014647389896975,   0.0156237286204768,   0.0166000375562313,  0.0175763148444956,
 0.018552558625889,   0.0195287670414137,   0.0205049382324764,  0.0214810703409091,  0.0224571615089906,
 0.0234332098794676,  0.0244092135955758,   0.0253851708010611,  0.0263610796402008,
 0.0273369382578244,  0.0283127447993352,   0.029288497410731,   0.0302641942386252,  0.0312398334302683,
 0.0322154131335681,  0.0331909314971116,   0.0341663866701854,  0.0351417768027968,
 0.036117100045695,   0.0370923545503918,   0.0380675384691825,  0.039042649955167,   0.0400176871622703,
 0.0409926482452638,  0.0419675313597857,   0.0429423346623622,  0.0439170563104278,
 0.0448916944623465,  0.0458662472774322,   0.0468407129159697,  0.0478150895392348,  0.0487893753095156,
 0.0497635683901328,  0.0507376669454602,   0.0517116691409454,  0.05268557314313,
 0.0536593771196708,  0.0546330792393595,   0.0556066776721433,  0.0565801705891457,  0.0575535561626864,
 0.0585268325663018,  0.0594999979747652,   0.0604730505641073,  0.0614459885116361,
 0.0624188099959574,  0.0633915131969944,   0.0643640962960086,  0.0653365574756192,  0.0663088949198235,
 0.0672811068140167,  0.068253191345012,    0.0692251467010603,  0.0701969710718705,
 0.0711686626486288,  0.0721402196240188,   0.0731116401922413,  0.0740829225490337,  0.0750540648916902,
 0.0760250654190807,  0.0769959223316711,   0.0779666338315423,  0.07893719812241,
 0.0799076134096439,  0.0808778779002873,   0.0818479898030765,  0.0828179473284599,  0.0837877486886171,
 0.0847573920974787,  0.0857268757707448,   0.0866961979259047,  0.0876653567822554,
 0.0886343505609211,  0.0896031774848717,   0.0905718357789422,  0.0915403236698511,  0.0925086393862194,
 0.0934767811585895,  0.0944447472194436,   0.0954125358032227,  0.0963801451463448,
 0.0973475734872237,  0.0983148190662873,   0.0992818801259964,  0.100248754910863,   0.101215441667467,   0.102181938644477,
 0.103148244092669,   0.10411435626494,     0.10508027341633,    0.106045993804039,
 0.107011515687447,   0.107976837328126,    0.108941956989866,   0.109906872938685,   0.110871583442852,   0.111836086772904,
 0.112800381201659,   0.113764465004242,    0.114728336458092,   0.115691993842991,
 0.116655435441069,   0.117618659536834,    0.118581664417177,   0.1195444483714,     0.120507009691225,   0.121469346670814,
 0.122431457606789,   0.123393340798243,    0.124354994546761,   0.125316417156437,
 0.126277606933887,   0.127238562188269,    0.128199281231298,   0.129159762377265,   0.130120003943049,   0.131080004248137,
 0.132039761614639,   0.132999274367304,    0.133958540833537,   0.134917559343415,
 0.135876328229701,   0.136834845827863,    0.137793110476088,   0.138751120515297,   0.139708874289164,   0.140666370144127,
 0.14162360642941,    0.14258058149703,     0.143537293701821,   0.144493741401443,
 0.145449922956401,   0.146405836730058,    0.147361481088652,   0.148316854401309,   0.14927195504006,    0.150226781379856,
 0.15118133179858,    0.152135604677064,    0.153089598399105,   0.154043311351475,
 0.154996741923941,   0.155949888509276,    0.156902749503275,   0.157855323304766,   0.158807608315631,   0.159759602940813,
 0.160711305588332,   0.161662714669305,    0.162613828597949,   0.163564645791604,
 0.164515164670743,   0.165465383658987,    0.166415301183115,   0.167364915673083,   0.168314225562034,   0.169263229286312,
 0.170211925285474,   0.171160312002307,    0.172108387882836,   0.17305615137634,
 0.174003600935368,   0.174950735015743,    0.175897552076585,   0.176844050580316,   0.177790228992676,   0.178736085782736,
 0.179681619422909,   0.180626828388963,    0.181571711160032,   0.182516266218631,
 0.183460492050666,   0.184404387145446,    0.185347949995695,   0.186291179097566,   0.187234072950649,   0.188176630057987,
 0.189118848926084,   0.190060728064918,    0.191002265987953,   0.191943461212149,
 0.192884312257975,   0.193824817649417,    0.194764975913995,   0.195704785582767,   0.196644245190345,   0.197583353274903,
 0.19852210837819,    0.199460509045539,    0.200398553825879,   0.201336241271742,
 0.202273569939279,   0.203210538388266,    0.204147145182117,   0.205083388887891,   0.206019268076305,   0.206954781321742,
 0.207889927202263,   0.208824704299615,    0.209759111199241,   0.210693146490291,
 0.21162680876563,    0.212560096621847,    0.213493008659266,   0.214425543481956,   0.215357699697738,   0.216289475918194,
 0.217220870758679,   0.218151882838326,    0.219082510780058,   0.220012753210596,
 0.220942608760466,   0.22187207606401,     0.222801153759395,   0.223729840488615,   0.22465813489751,    0.225586035635764,
 0.22651354135692,    0.227440650718385,    0.228367362381439,   0.229293675011242,
 0.230219587276844,   0.231145097851188,    0.232070205411125,   0.232994908637414,   0.233919206214733,   0.234843096831689,
 0.23576657918082,    0.236689651958605,    0.237612313865471,   0.238534563605801,
 0.239456399887938,   0.240377821424194,    0.241298826930859,   0.242219415128201,   0.24313958474048,    0.244059334495949,
 0.244978663126864,   0.24589756936949,     0.246816051964103,   0.247734109655002,
 0.248651741190513,   0.249568945322993,    0.250485720808836,   0.251402066408485,   0.252317980886427,   0.25323346301121,
 0.254148511555439,   0.25506312529579,     0.255977303013006,   0.25689104349191,
 0.25780434552141,    0.258717207894497,    0.259629629408258,   0.260541608863876,   0.261453145066638,   0.262364236825938,
 0.263274882955282,   0.264185082272293,    0.265094833598715,   0.266004135760417,
 0.2669129875874,     0.267821387913799,    0.268729335577886,   0.269636829422078,   0.270543868292937,   0.271450451041176,
 0.272356576521665,   0.27326224359343,     0.274167451119659,   0.275072197967707,
 0.275976483009098,   0.27688030511953,     0.277783663178873,   0.278686556071182,   0.27958898268469,    0.280490941911819,
 0.281392432649178,   0.282293453797569,    0.283194004261987,   0.284094082951627,
 0.284993688779881,   0.285892820664347,    0.286791477526827,   0.28768965829333,    0.288587361894077,   0.289484587263501,
 0.29038133334025,    0.291277599067188,    0.292173383391399,   0.293068685264188,
 0.293963503641084,   0.294857837481838,    0.295751685750432,   0.296645047415071,   0.297537921448196,   0.298430306826474,
 0.299322202530807,   0.300213607546333,    0.301104520862424,   0.301994941472688,
 0.302884868374971,   0.303774300571361,    0.304663237068183,   0.305551676876003,   0.30643961900963,    0.307327062488116,
 0.308214006334753,   0.309100449577082,    0.309986391246883,   0.310871830380185,
 0.31175676601726,    0.312641197202625,    0.313525122985044,   0.314408542417527,   0.31529145455733,    0.316173858465954,
 0.317055753209147,   0.317937137856902,    0.318818011483458,   0.3196983731673,
 0.320578221991157,   0.321457557042003,    0.322336377411056,   0.323214682193777,   0.324092470489872,   0.324969741403286,
 0.325846494042208,   0.326722727519067,    0.327598440950531,   0.328473633457506,
 0.329348304165139,   0.330222452202808,    0.331096076704132,   0.33196917680696,    0.332841751653376,   0.333713800389694,
 0.334585322166459,   0.335456316138443,    0.336326781464645,   0.33719671730829,
 0.338066122836825,   0.338934997221921,    0.339803339639465,   0.340671149269565,   0.341538425296542,   0.342405166908932,
 0.343271373299484,   0.344137043665154,    0.345002177207105,   0.345866773130707,
 0.34673083064553,    0.347594348965346,    0.348457327308122,   0.349319764896022,   0.350181660955402,   0.351043014716805,
 0.351903825414965,   0.352764092288795,    0.353623814581394,   0.354482991540035,
 0.355341622416168,   0.356199706465416,    0.357057242947567,   0.35791423112658,    0.358770670270572,   0.359626559651822,
 0.360481898546763,   0.361336686235982,    0.362190922004212,   0.363044605140335,
 0.363897734937373,   0.364750310692485,    0.365602331706967,   0.366453797286243,   0.367304706739864,   0.368155059381507,
 0.369004854528964,   0.369854091504145,    0.370702769633069,   0.371550888245862,
 0.372398446676754,   0.373245444264073,    0.374091880350239,   0.374937754281765,   0.375783065409249,   0.376627813087368,
 0.377471996674877,   0.378315615534604,    0.379158669033442,   0.380001156542349,
 0.380843077436342,   0.381684431094488,    0.382525216899905,   0.383365434239755,   0.384205082505239,   0.385044161091591,
 0.385882669398074,   0.386720606827975,    0.387557972788602,   0.388394766691274,
 0.389230987951321,   0.390066635988073,    0.390901710224862,   0.39173621008901,    0.392570135011829,   0.393403484428609,
 0.394236257778621,   0.395068454505103,    0.395900074055263,   0.396731115880264,
 0.397561579435227,   0.39839146417922,     0.399220769575253,   0.400049495090274,   0.400877640195163,   0.401705204364725,
 0.402532187077683,   0.403358587816674,    0.404184406068245,   0.405009641322841,
 0.405834293074804,   0.406658360822366,    0.407481844067642,   0.408304742316622,   0.409127055079168,   0.409948781869008,
 0.410769922203726,   0.411590475604759,    0.412410441597387,   0.413229819710733,
 0.414048609477749,   0.414866810435215,    0.415684422123729,   0.416501444087703,   0.417317875875355,   0.418133717038701,
 0.418948967133553,   0.419763625719506,    0.420577692359935,   0.42139116662199,
 0.422204048076584,   0.423016336298389,    0.423828030865831,   0.42463913136108,    0.425449637370042,   0.426259548482358,
 0.427068864291389,   0.427877584394215,    0.428685708391626,   0.429493235888114,
 0.430300166491866,   0.431106499814758,    0.431912235472348,   0.432717373083866,   0.433521912272209,   0.434325852663933,
 0.435129193889247,   0.435931935582003,    0.43673407737969,    0.437535618923428,
 0.438336559857958,   0.439136899831635,    0.439936638496422,   0.440735775507881,   0.441534310525167,   0.442332243211017,
 0.443129573231746,   0.443926300257239,    0.444722423960939,   0.445517944019847,
 0.446312860114506,   0.447107171928999,    0.447900879150937,   0.448693981471457,   0.449486478585208,   0.450278370190345,
 0.451069655988523,   0.451860335684889,    0.452650408988071,   0.453439875610172,
 0.454228735266762,   0.455016987676872,    0.45580463256298,    0.456591669651011,   0.457378098670321,   0.458163919353695,
 0.458949131437335,   0.459733734660856,    0.460517728767271,   0.461301113502991,
 0.46208388861781,    0.462866053864901,    0.463647609000806,   0.464428553785428,   0.465208887982023,   0.46598861135719,
 0.466767723680866,   0.467546224726316,    0.468324114270123,   0.46910139209218,
 0.469878057975687,   0.470654111707133,    0.471429553076297,   0.472204381876234,   0.472978597903266,   0.473752200956977,
 0.474525190840205,   0.475297567359028,    0.476069330322761,   0.476840479543945,
 0.477611014838337,   0.478380936024907,    0.479150242925823,   0.479918935366444,   0.480687013175316,   0.481454476184158,
 0.482221324227854,   0.482987557144447,    0.48375317477513,    0.484518176964233,
 0.485282563559221,   0.48604633441068,     0.48680948937231,    0.487572028300918,   0.488333951056406,   0.489095257501764,
 0.489855947503062,   0.490616020929441,    0.491375477653102,   0.4921343175493,
 0.492892540496335,   0.49365014637554,     0.494407135071275,   0.49516350647092,    0.495919260464861,   0.496674396946486,
 0.497428915812172,   0.498182816961281,    0.498936100296146,   0.499688765722065,
 0.500440813147294,   0.501192242483033,    0.501943053643421,   0.502693246545526,   0.503442821109336,   0.504191777257751,
 0.504940114916572,   0.505687834014494,    0.506434934483097,   0.507181416256835,
 0.50792727927303,    0.508672523471861,    0.509417148796356,   0.510161155192383,   0.51090454260864,    0.511647310996647,
 0.512389460310738,   0.513130990508049,    0.513871901548512,   0.514612193394845,
 0.515351866012543,   0.51609091936987,     0.516829353437846,   0.517567168190245,   0.518304363603578,   0.519040939657092,
 0.519776896332754,   0.520512233615247,    0.521246951491958,   0.521981049952972,
 0.522714528991058,   0.523447388601666,    0.524179628782913,   0.524911249535579,   0.525642250863092,   0.526372632771524,
 0.52710239526958,    0.527831538368588,    0.528560062082493,   0.529287966427846,
 0.530015251423793,   0.530741917092071,    0.531467963456995,   0.53219339054545,    0.532918198386882,   0.533642387013291,
 0.534365956459219,   0.535088906761743,    0.535811237960464,   0.5365329500975,
 0.537254043217479,   0.537974517367523,    0.538694372597247,   0.539413608958744,   0.540132226506582,   0.540850225297788,
 0.541567605391845,   0.542284366850679,    0.543000509738655,   0.54371603412256,
 0.544430940071603,   0.545145227657401,    0.54585889695397,    0.546571948037719,   0.547284380987437,   0.547996195884288,
 0.548707392811801,   0.549417971855859,    0.550127933104693,   0.55083727664887,
 0.551546002581289,   0.552254110997165,    0.552961601994028,   0.553668475671709,   0.554374732132331,   0.555080371480305,
 0.555785393822314,   0.55648979926731,     0.557193587926504,   0.557896759913355,
 0.558599315343562,   0.559301254335059,    0.560002577007999,   0.560703283484751,   0.561403373889889,   0.562102848350186,
 0.562801706994599,   0.563499949954267,    0.564197577362498,   0.564894589354762,
 0.565590986068683,   0.566286767644028,    0.5669819342227,     0.56767648594873,    0.568370422968264,   0.569063745429561,
 0.569756453482978,   0.570448547280968,    0.571140026978063,   0.571830892730873,
 0.572521144698072,   0.573210783040395,    0.573899807920624,   0.57458821950358,    0.575276017956118,   0.575963203447116,
 0.576649776147467,   0.577335736230069,    0.57802108386982,    0.578705819243603,
 0.579389942530287,   0.580073453910708,    0.58075635356767,    0.58143864168593,    0.582120318452191,   0.582801384055095,
 0.583481838685215,   0.584161682535044,    0.584840915798988,   0.585519538673358,
 0.586197551356361,   0.586874954048091,    0.587551746950524,   0.588227930267504,   0.588903504204738,   0.58957846896979,
 0.590252824772067,   0.590926571822816,    0.591599710335111,   0.592272240523851,
 0.592944162605743,   0.593615476799303,    0.594286183324841,   0.594956282404456,   0.595625774262027,   0.596294659123204,
 0.596962937215402,   0.597630608767789,    0.598297674011284,   0.598964133178543,
 0.599629986503951,   0.600295234223621,    0.600959876575375,   0.601623913798746,   0.602287346134964,   0.60295017382695,
 0.603612397119307,   0.604274016258314,    0.604935031491914,   0.605595443069711,
 0.606255251242958,   0.606914456264552,    0.607573058389022,   0.608231057872528,   0.608888454972845,   0.609545249949361,
 0.610201443063065,   0.610857034576544,    0.611512024753969,   0.612166413861094,
 0.612820202165241,   0.613473389935299,    0.614125977441711,   0.614777964956469,   0.615429352753105,   0.616080141106684,
 0.616730330293797,   0.617379920592551,    0.618028912282562,   0.61867730564495,
 0.619325100962327,   0.619972298518794,    0.620618898599929,   0.621264901492783,   0.621910307485869,   0.622555116869157,
 0.623199329934066,   0.623842946973455,    0.624485968281619,   0.625128394154276,
 0.625770224888563,   0.626411460783031,    0.62705210213763,    0.627692149253711,   0.62833160243401,    0.628970461982645,
 0.62960872820511,    0.630246401408263,    0.630883481900322,   0.631519969990857,
 0.632155865990784,   0.632791170212352,    0.633425882969145,   0.634060004576065,   0.634693535349333,   0.635326475606475,
 0.635958825666321,   0.636590585848993,    0.637221756475899,   0.637852337869727,
 0.638482330354438,   0.639111734255255,    0.639740549898664,   0.640368777612397,   0.640996417725432,   0.641623470567984,
 0.642249936471496,   0.642875815768636,    0.643501108793284,   0.644125815880533,
 0.644749937366675,   0.645373473589196,    0.645996424886772,   0.646618791599258,   0.647240574067683,   0.647861772634245,
 0.648482387642301,   0.649102419436359,    0.649721868362078,   0.650340734766252,
 0.650959018996812,   0.651576721402813,    0.652193842334429,   0.652810382142948,   0.653426341180762,   0.654041719801364,
 0.65465651835934,    0.655270737210358,    0.655884376711171,   0.656497437219599,
 0.657109919094532,   0.657721822695918,    0.658333148384756,   0.658943896523094,   0.659554067474019,   0.660163661601649,
 0.660772679271133,   0.661381120848635,    0.661988986701338,   0.662596277197427,
 0.663202992706093,   0.663809133597518,    0.664414700242873,   0.665019693014312,   0.665624112284961,   0.666227958428919,
 0.666831231821246,   0.667433932837957,    0.66803606185602,    0.668637619253345,
 0.66923860540878,    0.669839020702103,    0.670438865514021,   0.671038140226157,   0.671636845221047,   0.672234980882134,
 0.672832547593763,   0.673429545741172,    0.674025975710487,   0.674621837888718,
 0.67521713266375,    0.675811860424338,    0.676406021560103,   0.676999616461522,   0.677592645519925,   0.678185109127489,
 0.678777007677231,   0.679368341563002,    0.679959111179482,   0.680549316922172,
 0.681138959187393,   0.681728038372273,    0.682316554874748,   0.682904509093552,   0.683491901428213,   0.684078732279046,
 0.684665002047149,   0.685250711134394,    0.685835859943427,   0.686420448877654,
 0.687004478341245,   0.68758794873912,     0.688170860476949,   0.688753213961141,   0.689335009598846,   0.689916247797941,
 0.69049692896703,    0.691077053515437,    0.6916566218532,     0.692235634391065,
 0.692814091540483,   0.693391993713601,    0.69396934132326,    0.694546134782985,   0.695122374506987,   0.695698060910148,
 0.696273194408024,   0.696847775416835,    0.697421804353461,   0.697995281635439,
 0.69856820768095,    0.699140582908824,    0.699712407738527,   0.70028368259016,    0.70085440788445,    0.70142458404275,
 0.701994211487028,   0.702563290639866,    0.703131821924454,   0.703699805764582,
 0.704267242584639,   0.704834132809605,    0.705400476865049,   0.705966275177119,   0.706531528172541,   0.707096236278612,
 0.707660399923198,   0.708224019534724,    0.708787095542173,   0.70934962837508,
 0.709911618463525,   0.710473066238132,    0.711033972130061,   0.711594336571003,   0.712154159993179,   0.712713442829328,
 0.713272185512711,   0.713830388477098,    0.714388052156769,   0.714945176986506,
 0.715501763401588,   0.71605781183779,     0.716613322731375,   0.717168296519088,   0.717722733638155,   0.718276634526277,
 0.718829999621625,   0.719382829362832,    0.719935124188996,   0.720486884539668,
 0.721038110854852,   0.721588803574998,    0.722138963140998,   0.722688589994184,   0.723237684576318,   0.723786247329592,
 0.724334278696622,   0.724881779120445,    0.725428749044511,   0.725975188912682,
 0.726521099169226,   0.727066480258813,    0.727611332626511,   0.72815565671778,    0.72869945297847,    0.729242721854815,
 0.729785463793429,   0.730327679241302,    0.730869368645794,   0.731410532454635,
 0.731951171115917,   0.732491285078088,    0.733030874789955,   0.733569940700672,   0.73410848325974,    0.734646502917002,
 0.73518400012264,    0.735720975327167,    0.736257428981428,   0.736793361536592,
 0.737328773444149,   0.737863665155908,    0.73839803712399,    0.738931889800823,   0.739465223639145,   0.73999803909199,
 0.740530336612693,   0.741062116654879,    0.741593379672465,   0.74212412611965,
 0.742654356450918,   0.743184071121027,    0.74371327058501,    0.74424195529817,    0.744770125716075,   0.745297782294555,
 0.745824925489697,   0.746351555757843,    0.746877673555588,   0.747403279339768,
 0.747928373567467,   0.748452956696006,    0.748977029182941,   0.749500591486061,   0.750023644063381,   0.750546187373141,
 0.751068221873802,   0.751589748024043,    0.752110766282754,   0.752631277109036,
 0.753151280962194,   0.75367077830174,     0.754189769587379,   0.754708255279016,   0.755226235836745,   0.755743711720849,
 0.756260683391796,   0.756777151310235,    0.757293115936992,   0.757808577733069,
 0.758323537159635,   0.758837994678031,    0.759351950749758,   0.75986540583648,    0.760378360400016,   0.760890814902341,
 0.761402769805578,   0.761914225571999,    0.762425182664017,   0.762935641544187,
 0.763445602675202,   0.763955066519886,    0.764464033541195,   0.764972504202212,   0.765480478966144,   0.765987958296319,
 0.766494942656179,   0.767001432509286,    0.767507428319308,   0.768012930550024,
 0.768517939665315,   0.769022456129166,    0.769526480405658,   0.77003001295897,    0.770533054253371,   0.771035604753219,
 0.77153766492296,    0.77203923522712,     0.772540316130307,   0.773040908097206,
 0.773541011592573,   0.774040627081239,    0.7745397550281,     0.775038395898116,   0.775536550156312,   0.776034218267768,
 0.776531400697624,   0.77702809791107,     0.777524310373348,   0.778020038549745,
 0.778515282905595,   0.779010043906271,    0.779504322017186,   0.77999811770379,    0.780491431431562,   0.780984263666015,
 0.781476614872688,   0.781968485517144,    0.782459876064968,   0.782950786981764,
 0.783441218733152,   0.783931171784766,    0.784420646602251,   0.784909643651259,   0.785398163397448};

IFX_INLINE float32 AngleTrk_atan2Lookup(float32 val)
{
    return AngleTrk_atan2Table[(int)(val * ANGLETRK_ATAN_SIZE)];
}


float32 AngleTrk_atan2(float32 y, float32 x)
{
    float32 angle;

    if (y < 0)
    {
        if (x < 0)
        {
            if (-y < -x)
            {                   // 180 .. 225 deg
                angle = -PI + AngleTrk_atan2Lookup(-y / -x);
            }
            else
            {                   // 225 .. 270 deg
                angle = -(PI / 2) - AngleTrk_atan2Lookup(-x / -y);
            }
        }
        else                                           // x >= 0
        {
            if (-y < x)
            {                                          // 315 .. 360 deg
                angle = -AngleTrk_atan2Lookup(-y / x); // ok
            }
            else
            {                                          // 225 .. 270 deg
                angle = -(PI / 2) + AngleTrk_atan2Lookup(x / -y);
            }
        }
    }
    else                        // y >= 0
    {
        if (x < 0)
        {
            if (y < -x)
            {                                                    // 135 .. 180 deg
                angle = PI - AngleTrk_atan2Lookup(y / -x);       // ok
            }
            else
            {                                                    // 90 .. 135 deg
                angle = (PI / 2) + AngleTrk_atan2Lookup(-x / y); // ok
            }
        }
        else                                                     // x >= 0
        {
            if (y < x)
            {                                                    // 0 .. 45 deg
                angle = AngleTrk_atan2Lookup(y / x);             // ok
            }
            else
            {                                                    // 45 .. 90 deg
                angle = (PI / 2) - AngleTrk_atan2Lookup(x / y);  // ok
            }
        }
    }

    return angle;
}
