/**
 * \file AngleTrk.h
 * \brief Angle-tracking observer for sin/cos analog position sensor
 * \ingroup library_srvsw_if_pos_angletrk
 *
 * \license
 * You can use this file under the terms of the IFX License.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * IFX License for more details (IFX_License.txt).
 *
 * This file may be used, copied, and distributed, with or without
 * modification, provided that all copyright notices are retained;
 * that all modifications to this file are prominently noted in the
 * modified file; and that aObsv paragraph is not modified.
 *
 * \copyright Copyright (C) 2011 Infineon Technologies AG
 *
 *
 * $Revision: $1381.4
 * $Date: $2013-07-12 16:20:30 GMT
 *
 * \defgroup library_srvsw_if_pos_angletrk Angle-Tracking Observer
 * This driver implements \ref library_srvsw_if_pos using 3rd order Angle-Tracking Observer algorithm.
 * \ingroup library_srvsw_if_pos
 *
 * FIXME this is not an interface, it should be moved to the SysSe library
 */

#ifndef ANGLETRK_H_
#define ANGLETRK_H_

//________________________________________________________________________________________
// INCLUDES

#include "If/PosIf.h"
#include "SysSe/Math/LowPass_PT1.h"

/** \addtogroup library_srvsw_if_pos_angletrk
 * \{ */

//________________________________________________________________________________________
// CONFIGURATION DEFINES

#ifndef ANGLETRK_RESOLUTION
#define ANGLETRK_RESOLUTION (1UL << 12)        /** \brief 12-bit default resolution */
#endif

#ifndef ANGLETRK_TEST_MODE
#define ANGLETRK_TEST_MODE  (0)
#endif

#if ANGLETRK_TEST_MODE
#include "AngleGen.h"
#endif

//________________________________________________________________________________________
// DATA STRUCTURES

/** \brief Angle Tracking Observer configuration */
typedef struct
{
    float32   Kp;                   /**< \brief Proportional gain */
    float32   Ki;                   /**< \brief Integrator gain */
    float32   Kd;                   /**< \brief Derivative gain */
    float32   speedLpfFc;           /**< \brief Cut-off frequency of speed low-pass filter. */
    float32   errorThreshold;       /**< \brief Threshold of error value in the tracking loop */
    sint32    sqrAmplMax;           /**< \brief Maximum value for square of signal amplitudes */
    sint32    sqrAmplMin;           /**< \brief Minimum value for square of signal amplitudes */
    sint32    periodPerRotation;    /**< \brief Number of electrical periods per mechanical rotation */
    boolean   reversed;             /**< \brief TRUE: reversed direction, FALSE: straight direction */
    PosIf_Raw offset;               /**< \brief Offset in ticks. [0 .. (\ref ANGLETRK_RESOLUTION - 1)] */
    sint16   *sinIn;                /**< \brief Pointer to SIN input variable */
    sint16   *cosIn;                /**< \brief Pointer to COS input variable */
} AngleTrk_Config;

/** \brief Angle Tracking Observer object */
typedef struct
{
    PosIf           base;
    AngleTrk_Config cfg;
    float32         Ts;
    float32         halfTs;
    float32         angleRef;
    float32         angleEst;
    float32         speedEst;
    float32         accelEst;
    float32         dAngle;
    float32         error;
    LowPass_PT1     speedLpf;
#if ANGLETRK_TEST_MODE
    struct
    {
        float32  speed;
        AngleGen gen;
    } stimuli;
#endif
} AngleTrk;

//________________________________________________________________________________________
// PUBLIC FUNCTION PROTOTYPES

/** \name Initialisation functions
 * Example use:
 * \code
 * extern sint16    g_SinInput, g_CosInput;
 * AngleTrk_Config  driverConfig;
 * AngleTrk         driverData;
 * AngleTrk_initConfig(&driverConfig, &g_SinInput, g_CosInput);
 * driverConfig.errorThreshold    = 5.0 / 180 * PI; // 5 degree error
 * driverConfig.sqrAmplMax        = 1.01 * 1.01;
 * driverConfig.sqrAmplMin        = 0.99 * 0.99;
 * driverConfig.speedLpfFc        = 100;
 * driverConfig.periodPerRotation = 1;
 * driverConfig.reversed          = FALSE;
 * driverConfig.offset            = 0;
 * AngleTrk_init(&driverData, &driverConfig);
 * \endcode
 * Prototypes:
 * \{ */
IFX_EXTERN void AngleTrk_init(AngleTrk *aObsv, const AngleTrk_Config *config, float32 Ts);
IFX_EXTERN void AngleTrk_initConfig(AngleTrk_Config *config, sint16 *sinIn, sint16 *cosIn);
/** \} */

/** \name Interface implementations
 * It is recommended to use the \ref mod_PosIf functions, e.g.:
 * \code
 * newPosition = PosIf_updatePosition(&driverData.base);
 * \endcode
 * Prototypes:
 * \{ */
IFX_EXTERN PosIf_Raw AngleTrk_update(AngleTrk *aObsv);
IFX_EXTERN void      AngleTrk_setupElAngleConst(AngleTrk *aObsv, sint32 motorPolePairs);
IFX_EXTERN PosIf_Raw AngleTrk_getPosition(AngleTrk *aObsv);
IFX_EXTERN StdReal   AngleTrk_getSpeed(AngleTrk *aObsv, boolean update);
IFX_EXTERN sint32    AngleTrk_resetOffset(AngleTrk *aObsv);
IFX_EXTERN void      AngleTrk_setOffset(AngleTrk *aObsv, PosIf_Raw offset);
/** \} */

/** \name Protected functions
 * These functions are not for end-user but may be used by another driver which extends
 * the functionality of mod_AngleTrk
 * \{ */
IFX_EXTERN float32 AngleTrk_step(AngleTrk *aObsv, sint32 sinIn, sint32 cosIn, float32 phase);
IFX_EXTERN void    AngleTrk_updateStatus(AngleTrk *aObsv, sint32 sinIn, sint32 cosIn);
/** \} */

/** \} */

#endif /* ANGLETRK_H_ */
