/**
 * \file CanIf.h
 * \brief CAN interface.
 *
 * \license
 * You can use this file under the terms of the IFX License.
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the IFX License for more details (IFX_License.txt).
 *
 * This file may be used, copied, and distributed, with or without modification, provided
 * that all copyright notices are retained; that all modifications to this file are
 * prominently noted in the modified file; and that this paragraph is not modified.
 *
 * \copyright Copyright (C) 2012 - 2008 Infineon Technologies A.G.
 *
 *
 * $Revision: $1381.4
 * $Date: $2013-07-12 16:20:30 GMT
 *
 * \defgroup library_srvsw_if_can CAN interface
 *
 * This module implements the CAN interface functions.
 *
 * \ingroup library_srvsw_if
 *
 * FIXME this is not an interface, it should be moved to the SysSe library
 */

#ifndef CAN_IF_H
#define CAN_IF_H
//________________________________________________________________________________________
// Includes

#include "Can/Std/IfxCan.h"

/** \addtogroup library_srvsw_if_can
 * \{ */

//________________________________________________________________________________________
// Macros

#ifndef CAN_IF_NODE_COUNT
#define CAN_IF_NODE_COUNT      (2)          /**< \brief Number of CAN nodes used */
#endif

#ifndef CAN_IF_CHANNEL_COUNT
#define CAN_IF_CHANNEL_COUNT   (16)         /**< \brief Maximum number of channels to configure */
#endif

#ifndef CAN_IF_SOFTWARE_FILTER
#define CAN_IF_SOFTWARE_FILTER (0)          /**< \brief whether software filter is enabled, default is 0=disabled */
#endif

#define CAN_IF_CHANNEL_GROUPS  (IFXCAN_MO_COUNT / 32)

//________________________________________________________________________________________
// Type definitions
typedef uint32 CanIf_ChannelId;

/** \brief CAN channel object */
typedef struct
{
    IfxCan_MsgObjId msgObjId;       /**< \brief Index of the message object used for the channel */
#if CAN_IF_SOFTWARE_FILTER
    uint32          firstId;        /**< \brief ID filter lower boundary */
    uint32          lastId;         /**< \brief ID filter upper boundary */
#endif
    IfxCan_Message *message;        /**< \brief Pointer to message storage */
    boolean         newData;        /**< \brief New data in the buffer */
    uint32          txCounter;
    uint32          rxCounter;
    uint32          lostCounter;

    /** \brief Callback function: A message has been read or received (depends on the message type) */
    void (*onChannelRTx)(CanIf_ChannelId channelId);
} CanIf_Channel;

/** \brief CAN channel configuration */
typedef struct
{
    IfxCan_NodeId   nodeId;             /**< \brief Specifies the node Id assigned to this message object*/
    IfxCan_Frame    frame;              /**< \brief Specifies the frame type */
    uint32          messageId;          /**< \brief Specifies the message Id */
    uint32          acceptanceMask;     /**< \brief Specifies the message acceptance mask */
    uint32          messageLen;         /**< \brief Specifies the length of the transmited data (number of bytes). This value is ignored for receive object */
    boolean         extendedFrame;      /**< \brief If TRUE 29-bit identifier, else 11-bit */
#if CAN_IF_SOFTWARE_FILTER
    uint32          firstId;            /**< \brief Specifies the ID filter lower boundary */
    uint32          lastId;             /**< \brief Specifies the ID filter upper boundary. If LastId >= 0 the driver check for the ID range (software filtering) (FirstID <= ID <= LastId), the hardware mask is calculated acording to FirstID and LastID */
#endif
    IfxCan_Message *message;            /**< \brief Pointer to message storage (will be initialised by CanIf)*/
    /** \brief Callback function: A message has been read or received */
    void (*onChannelRTx)(CanIf_ChannelId channelId);

    /** \brief Callback function: An error in message transfer has been occurred */
    void (*onChannelError)(CanIf_ChannelId channelId);
} CanIf_ChannelConfig;

/** \brief CAN node status */
typedef enum
{
    CanIf_NodeState_init,
    CanIf_NodeState_starting,
    CanIf_NodeState_started,
    CanIf_NodeState_stopping,
    CanIf_NodeState_stopped,
    //CanIf_NodeState_startToSleep,
    //CanIf_NodeState_wakingUp
} CanIf_NodeState;

/** \brief CAN node data */
typedef struct
{
    IfxCan_NodeId nodeId;

    uint32        baudrate;
    uint16        samplePoint;
    uint16        synchJumpWidth;

    /** \brief Node ISR counter */
    uint32 isrCounter;

    /** \brief Call back function: node error */
    void (*onError)(IfxCan_NodeId node);
} CanIf_Node;

/** \brief CAN node configuration */
typedef struct
{
    IfxCan_NodeId nodeId;               /**< \brief Node Id */
    uint32        baudrate;             /**< \brief Specifies the baudrate */
    uint16        samplePoint;          /**< \brief Specifies the sample point. Range = [0, 10000] resp. [0%, 100%] of the total bit time */
    uint16        synchJumpWidth;       /**< \brief Specifies the resynchronisation jump width. Range = [0, 10000] resp. [0%, 100%] of the total bit time*/
    const void   *rxdInput;             /**< \brief Pointer to the RX pin configuration: \ref IfxCan_Rxdcan_In */
    const void   *txdOutput;            /**< \brief Pointer to the TX pin configuration: \ref IfxCan_Txdcan_Out */
    Priority      priority;             /**< \brief Node interrupt priority. If priority > 0, user shall provide interrupt which calls the CanIf_nodeProcess()  */

    /** \brief Call back function: node error */
    void (*onError)(IfxCan_NodeId node);
} CanIf_NodeConfig;

typedef struct
{
    uint32 isrCounter;
} CanIf_Group;

/** \brief CAN interface object */
typedef struct
{
    /** \brief CAN node objects */
    CanIf_Node nodes[CAN_IF_NODE_COUNT];

    /** \brief CAN channel objects */
    CanIf_Channel channels[CAN_IF_CHANNEL_COUNT];

    /** \brief CAN channel groups */
    CanIf_Group groups[CAN_IF_CHANNEL_GROUPS];
} CanIf;

/** \brief CAN interface configuration */
typedef struct
{
    /** \brief CAN node configurations */
    CanIf_NodeConfig nodeConfig[CAN_IF_NODE_COUNT];

    /** \brief CAN message group interrupt priorities
     *  - Each group consists 32 message objects
     *  - If priority > 0, user shall provide interrupt which calls the CanIf_channelProcess()
     *  - If priority == 0, user shall call CanIf_channelProcess() periodically */
    Priority groupPriority[CAN_IF_CHANNEL_GROUPS];
} CanIf_Config;

//________________________________________________________________________________________
// Public function prototypes

IFX_EXTERN void    CanIf_initConfig(CanIf_Config *config);
IFX_EXTERN boolean CanIf_init(const CanIf_Config *config);
IFX_EXTERN void    CanIf_deinit(void);

IFX_EXTERN void    CanIf_initChannelConfig(CanIf_ChannelConfig *config);
IFX_EXTERN boolean CanIf_initChannel(CanIf_ChannelId channelId, const CanIf_ChannelConfig *config);
IFX_EXTERN void    CanIf_deinitChannel(CanIf_ChannelId channelId);
IFX_EXTERN boolean CanIf_sendMessage(CanIf_ChannelId channelId, const IfxCan_Message *msg);
IFX_EXTERN boolean CanIf_readMessage(CanIf_ChannelId channelId, IfxCan_Message *msg);

IFX_EXTERN void CanIf_channelProcess(IfxCan_MsgObjGrp groupNr);
IFX_EXTERN void CanIf_nodeProcess(IfxCan_NodeId node);

/** \} */

//________________________________________________________________________________________
#endif
