/**
 * \file PwmHl.h
 * \brief Generic complementary (dual) PWM channel object.
 *
 * \license
 * You can use this file under the terms of the IFX License.
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the IFX License for more details (IFX_License.txt).
 *
 * This file may be used, copied, and distributed, with or without modification, provided
 * that all copyright notices are retained; that all modifications to this file are
 * prominently noted in the modified file; and that this paragraph is not modified.
 *
 * \copyright Copyright (C) 2012 Infineon Technologies AG
 *
 *
 * $Revision: $1498.5
 * $Date: $2013-07-12 16:20:32 GMT
 *
 * \defgroup library_srvsw_if_pwmhl Multi-channels, dual-complementary PWM interface
 * \ingroup library_srvsw_if
 *
 * The figure below shows the standard Multi-channels, dual-complementary PWM interface.
 *
 * \image html "PwmHl-overview.png" "Standard Multi-channels, dual-complementary PWM interface"
 *
 * This PWM interface provides multiple dual-complementary PWM channels.
 * Each dual-complementary PWM channel is made of two PWM channels, a top channel CCx and a bottom channel COUTx.
 * A Timer as defined by the \ref library_srvsw_if_timer must be linked to the PWM interface.
 *
 * This interface defines the following features:
 * - Configurable duty cycle from 0% to 100%
 * - Adjustable dead-time between top and bottom channels
 * - Center aligned, left aligned, right aligned PWM
 * - Optional minimal pulse cancellation
 * - Optional emergency stop
 * - Configurable signal active state for top and bottom PWM
 * - Configurable output ports
 * - Inherit the features from library_srvsw_if_timer
 *
 * Example of signal generation in center aligned mode:
 *
 * \image html "PwmHl-centerAligned.png" "Standard Multi-channels, dual-complementary PWM interface - Center aligned mode"
 */

#ifndef PWMHLCH_H
#define PWMHLCH_H

#include "Cpu/Std/Ifx_Types.h"
#include "Port/Std/IfxPort.h"   /* FIXME interface should not have dependencies to the HAL, an If type needs to be defined */
#include "Pwm.h"
#include "Timer.h"

/** \addtogroup library_srvsw_if_pwmhl
 * \{ */

/** \brief Multi-channels PWM object configuration */
typedef struct
{
    Timer             *timer;               /**< \brief Pointer on the timer object used. The timer must be Initialised previously */
    float32            deadtime;            /**< \brief Dead time between the top and bottom channels in seconds */
    float32            minPulse;            /**< \brief Min pulse allowed as active state for the top and bottom PWM in seconds */
    uint8              channelCount;        /**< \brief Number of PWM channels, one channel is made of a top and bottom channel */
    boolean            emergencyEnabled;    /**< \brief Specifies if the emergency stop should be enabled or not */

    IfxPort_OutputMode outputMode;          /**< \brief Output mode of ccx and coutx pins */
    IfxPort_PadDriver  outputDriver;        /**< \brief Output pad driver of ccx and coutx pins *//* FIXME use generic type (No specific tricore AURIX type ) */

    ActiveState        ccxActiveState;      /**< \brief Top PWM active state */
    const sint32      *ccxFeature;          /**< \brief Pointer to an array of size channelCount containing the resource Id for output IO, e.g. Gpta_Io_x. Value IFXPORT_OUTOUTFEATURE_NONE is considered as NONE FIXME move to customize driver, see IfxGtmAtom_ToutMapP*/
    const If_IoOption *ccxOption;           /**< \brief Pointer to an array of size channelCount containing the output option FIXME move to customize driver, see IfxGtmAtom_ToutMapP*/

    ActiveState        coutxActiveState;    /**< \brief Bottom PWM active state */
    const sint32      *coutxFeature;        /**< \brief Pointer to an array of size channelCount containing the resource Id for output IO, e.g. Gpta_Io_x. Value IFXPORT_OUTOUTFEATURE_NONE is considered as NONE FIXME move to customize driver, see IfxGtmAtom_ToutMapP*/
    const If_IoOption *coutxOption;         /**< \brief Pointer to an array of size channelCount containing the output option FIXME move to customize driver, see IfxGtmAtom_ToutMapP */
} PwmHl_Config;

/** \brief Alias to the PwmHl structure */
typedef struct PwmHl_s PwmHl;

typedef boolean      (*PwmHl_SetDeadtime)(PwmHl *handle, float32 deadtime);                                 /**< \brief Prototype */
typedef float32      (*PwmHl_GetDeadtime)(PwmHl *handle);                                                   /**< \brief Prototype */
typedef boolean      (*PwmHl_SetMinPulse)(PwmHl *handle, float32 minPulse);                                 /**< \brief Prototype */
typedef float32      (*PwmHl_GetMinPulse)(PwmHl *handle);                                                   /**< \brief Prototype */
typedef Pwm_Mode     (*PwmHl_GetMode)(PwmHl *handle);                                                       /**< \brief Prototype */
typedef boolean      (*PwmHl_SetMode)(PwmHl *handle, Pwm_Mode mode);                                        /**< \brief Prototype */
typedef void         (*PwmHl_SetOnTime)(PwmHl *handle, TimerVal *tOn);                                      /**< \brief Prototype */
typedef void         (*PwmHl_SetupChannels)(PwmHl *handle, boolean *activeCh, boolean *stuckSt);            /**< \brief Prototype */

/** \brief Pointers to the PwmHl function implementations */
typedef struct
{
    PwmHl_SetDeadtime   setDeadtime;        /**< \brief Set the dead time in s */
    PwmHl_GetDeadtime   getDeadtime;        /**< \brief returns the dead time in s */
    PwmHl_SetMinPulse   setMinPulse;        /**< \brief Set the min pulse in s */
    PwmHl_GetMinPulse   getMinPulse;        /**< \brief Return the min pulse in s */
    PwmHl_GetMode       getMode;            /**< \brief  */
    PwmHl_SetMode       setMode;            /**< \brief  */
    PwmHl_SetOnTime     setOnTime;          /**< \brief  */
    PwmHl_SetupChannels setupChannels;      /**< \brief  */
} PwmHl_Funcs;

/** \brief Multi-channels PWM object definition (channels only) */
struct PwmHl_s
{
    pvoid       driver;                 /**< \brief PwmHl specific driver */
    Timer      *timer;                  /**< \brief Pointer to the linked timer object */
    TimerVal    deadtime;               /**< \brief Dead time between the top and bottom channel in ticks */
    TimerVal    minPulse;               /**< \brief minimum pulse that is output, shorter pulse time will be output as 0% duty cycle */
    TimerVal    maxPulse;               /**< \brief internal parameter */
    Pwm_Mode    mode;                   /**< \brief actual PWM mode */
    sint8       setMode;                /**< \brief a non zero flag indicates that the PWM mode is being modified */
    ActiveState ccxActiveState;         /**< \brief Top PWM active state */
    ActiveState coutxActiveState;       /**< \brief Bottom PWM active state */
    boolean     inverted;               /**< \brief Flag indicating the center aligned inverted mode (TRUE) */
    uint8       channelCount;           /**< \brief Number of PWM channels, one channel is made of a top and bottom channel */
    PwmHl_Funcs functions;
};

/** \name Base functions
 * These functions are implemented in PwmHl.c/h but only usable after a driver has been initialised.
 * \{ */
IFX_EXTERN boolean  PwmHl_setDeadtime(PwmHl *handle, float32 deadtime);
IFX_EXTERN float32  PwmHl_getDeadtime(PwmHl *handle);
IFX_EXTERN boolean  PwmHl_setMinPulse(PwmHl *handle, float32 minPulse);
IFX_EXTERN float32  PwmHl_getMinPulse(PwmHl *handle);
IFX_EXTERN Pwm_Mode PwmHl_getMode(PwmHl *handle);
IFX_INLINE Timer   *PwmHl_getTimer(PwmHl *handle)
{
    return handle->timer;
}


/** \} */

/** \name Virtual functions
 * These functions are implemented by a driver.
 * \{ */
IFX_INLINE boolean PwmHl_setMode(PwmHl *handle, Pwm_Mode mode);
IFX_INLINE void    PwmHl_setOnTime(PwmHl *handle, TimerVal *tOn);
IFX_INLINE void    PwmHl_setupChannels(PwmHl *handle, boolean *activeCh, boolean *stuckSt);
/** \} */

/** \name Protected functions
 * Only the driver may use these functions.
 * \{ */
IFX_EXTERN void PwmHl_initConfig(PwmHl_Config *config);
/** \} */

/** \} */
//________________________________________________________________________________________
//  INLINE FUNCTION IMPLEMENTATIONS

IFX_INLINE boolean PwmHl_setMode(PwmHl *handle, Pwm_Mode mode)
{
    return handle->functions.setMode(handle->driver, mode);
}


IFX_INLINE void PwmHl_setOnTime(PwmHl *handle, TimerVal *tOn)
{
    handle->functions.setOnTime(handle->driver, tOn);
}


IFX_INLINE void PwmHl_setupChannels(PwmHl *handle, boolean *activeCh, boolean *stuckSt)
{
    handle->functions.setupChannels(handle->driver, activeCh, stuckSt);
}


#endif /* PWMHLCH_H */
