/**
 * \file Timer.h
 * \brief Standard timer interface
 * \ingroup library_srvsw_if_timer
 *
 * \license
 * You can use this file under the terms of the IFX License.
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the IFX License for more details (IFX_License.txt).
 *
 * This file may be used, copied, and distributed, with or without modification, provided
 * that all copyright notices are retained; that all modifications to this file are
 * prominently noted in the modified file; and that this paragraph is not modified.
 *
 * \copyright Copyright (C) 2012 Infineon Technologies AG
 *
 *
 * $Revision: $1478.5
 * $Date: $2013-07-12 16:20:33 GMT
 *
 * \defgroup library_srvsw_if_timer Timer interface
 * \ingroup library_srvsw_if
 *
 * The figure below shows the standard timer interface.
 *
 * \image html "Timer.png" "Standard timer interface"
 *
 * This interface defines the following features:
 * - Timer with configurable frequency
 * - Configurable counting direction
 * - Optional interrupt on timer event with configurable service request provider and priority
 * - Optional trigger signal with:
 *   - Optional interrupt on trigger event with configurable service request provider and priority
 *   - Configurable trigger point
 *   - Configurable trigger edge
 *   - trigger signal can be internal only or forwarded to a port pin
 *
 */

#ifndef TIMER_H
#define TIMER_H

#include "Cpu/Std/Ifx_Types.h"
#include "Src/Std/IfxSrc.h"     /* FIXME interface should not have dependencies to the HAL, an If type needs to be defined */
#include "Generic.h"
#include "Port/Std/IfxPort.h"   /* FIXME interface should not have dependencies to the HAL, an If type needs to be defined */ /*FIXME introduce generic IF for ports. ex: IfxPort_OutputMode */

/** \addtogroup library_srvsw_if_timer
 * \{ */

/** Reset event for channel counter CN0 */
typedef enum
{
    Timer_Output_disabled,              /**< \brief Output is set to the inactive level */
    Timer_Output_enabled                /**< \brief In case the trigger is enabled,
                                         *          the output is set to active between time 0 to the trigger point,
                                         *          and is set to inactive between the trigger point and the period.
                                         *          In case the trigger is disabled, the output is toggled when the timer is reset */
} Timer_OutputEvent;

/** Timer increment direction */
typedef enum
{
    Timer_CountDir_up,                      /**< \brief Timer is counting up */
    Timer_CountDir_upAndDown,               /**< \brief Timer is counting up and down */
    Timer_CountDir_down                     /**< \brief Timer is counting down */
} Timer_CountDir;

/** \brief Trigger object definition */
typedef struct
{
    boolean            enabled;             /**< \brief If true, the trigger functionality is Initialised, else ignored */
    float32            dutyCycle;           /**< \brief Trigger duty cycle */
    Priority           isrPriority;         /**< \brief Interrupt isrPriority of the trigger interrupt, if 0 the interrupt is disable */
    IfxSrc_Tos         isrProvider;         /**< \brief Interrupt service provider for the trigger interrupt */
    sint32             outputFeature;       /**< \brief Resource Id for output IO, e.g. Gpta_Io_x. Value IFXPORT_OUTOUTFEATURE_NONE is considered as NONE */
    If_IoOption        outputOption;        /**< \brief Output option */
    IfxPort_OutputMode outputMode;          /**< \brief Output mode *//* FIXME use generic type (No specific tricore AURIX type ) */
    IfxPort_PadDriver  outputDriver;        /**< \brief Output pad driver *//* FIXME use generic type (No specific tricore AURIX type ) */
    boolean            risingEdgeAtPeriod;  /**< \brief Set the clock signal polarity, if TRUE, the rising edge is at the period, else at the trigger offset. When the timer is stopped, the output is set to high */
    boolean            outputEnabled;       /**< \brief If TRUE, the output pin is enabled, else disabled. In case the output is disabled, the output pin is not initialized. FIXME use outputOption==If_IoOption_none instated and remove this member */
} Timer_TrigConfig;

/** \brief PWM timer object definition */
typedef struct
{
    float32          frequency;         /**< \brief PWM frequency in Hz. This parameter is only used to initialise the timer structure. An additional cell is required to build the timer. */
    Priority         isrPriority;       /**< \brief Interrupt isrPriority of the timer interrupt, if 0 the interrupt is disable */
    IfxSrc_Tos       isrProvider;       /**< \brief Interrupt service provider for the timer interrupt */
    float32          minResolution;     /**< \brief Minimum resolution of the timer in seconds */
    Timer_TrigConfig trigger;           /**< \brief Trigger configuration */
    Timer_CountDir   countDir;          /**< \brief Timer counting mode */
} Timer_Config;

/** \brief Alias to the Timer structure */
typedef struct Timer_s Timer;

typedef float32      (*Timer_GetFrequency)(Timer *timer);                                           /**< \brief Prototype */
typedef TimerVal     (*Timer_GetPeriod)(Timer *timer);                                              /**< \brief Prototype */
typedef float32      (*Timer_GetResolution)(Timer *timer);                                          /**< \brief Prototype */
typedef boolean      (*Timer_SetFrequency)(Timer *timer, float32 frequency);                        /**< \brief Prototype */
typedef TimerVal     (*Timer_SToTicks)(Timer *timer, float32 seconds);                              /**< \brief Prototype */
typedef float32      (*Timer_TicksToS)(Timer *timer, TimerVal ticks);                               /**< \brief Prototype */
typedef void         (*Timer_UpdateInputFrequency)(Timer *timer);                                   /**< \brief Prototype */
typedef void         (*Timer_ApplyUpdate)(Timer *handle);                                           /**< \brief Prototype */
typedef void         (*Timer_DisableUpdate)(Timer *handle);                                         /**< \brief Prototype */
typedef float32      (*Timer_GetInputFrequency)(Timer *handle);                                     /**< \brief Prototype */
typedef void         (*Timer_Run)(Timer *handle);                                                   /**< \brief Prototype */
typedef boolean      (*Timer_SetPeriod)(Timer *handle, TimerVal period);                            /**< \brief Prototype */
typedef void         (*Timer_SetSingleMode)(Timer *handle, boolean enabled);                        /**< \brief Prototype */
typedef void         (*Timer_SetTrigger)(Timer *handle, TimerVal triggerPoint);                     /**< \brief Prototype */
typedef void         (*Timer_Stop)(Timer *handle);                                                  /**< \brief Prototype */

/** Return the timer interrupt flag and reset it
 * \param handle pointer to the Timer object
 * \return Return the timer interrupt flag
 */
typedef boolean (*Timer_AckTimerIrq)(Timer *handle);

/** Return the trigger interrupt flag and reset it
 * \param handle pointer to the Timer object
 * \return Return the trigger interrupt flag
 */
typedef boolean (*Timer_AckTriggerIrq)(Timer *handle);

/** \brief Pointers to the PwmHl function implementations */
typedef struct
{
    Timer_GetFrequency         getFrequency;            /**< \brief TODO: add documentation */
    Timer_GetPeriod            getPeriod;               /**< \brief Return the timer period in ticks  */
    Timer_GetResolution        getResolution;           /**< \brief TODO: add documentation */
    Timer_SetFrequency         setFrequency;            /**< \brief TODO: add documentation */
    Timer_SToTicks             sToTick;                 /**< \brief TODO: add documentation */
    Timer_TicksToS             tickToS;                 /**< \brief TODO: add documentation */
    Timer_UpdateInputFrequency updateInputFrequency;    /**< \brief TODO: this may be not needed */
    Timer_ApplyUpdate          applyUpdate;             /**< \brief TODO: add documentation */
    Timer_DisableUpdate        disableUpdate;           /**< \brief TODO: add documentation */
    Timer_GetInputFrequency    getInputFrequency;       /**< \brief TODO: add documentation */
    Timer_Run                  run;                     /**< \brief TODO: add documentation */
    Timer_SetPeriod            setPeriod;               /**< \brief TODO: add documentation */
    Timer_SetSingleMode        setSingleMode;           /**< \brief TODO: add documentation */
    Timer_SetTrigger           setTrigger;              /**< \brief TODO: add documentation */
    Timer_Stop                 stop;                    /**< \brief TODO: add documentation */
    Timer_AckTimerIrq          ackTimerIrq;             /**< \brief \see Timer_AckTimerIrq */
    Timer_AckTriggerIrq        ackTriggerIrq;           /**< \brief \see Timer_AckTriggerIrq */
} Timer_Funcs;

/** \brief Timer object definition */
struct Timer_s
{
    pvoid          driver;              /**< \brief Timer specific driver */
    Timer_Funcs    functions;
    TimerVal       period;              /**< \brief PWM period in ticks (cached value) */
    boolean        triggerEnabled;      /**< \brief If TRUE, the trigger functionality is Initialised */
    float32        clockFreq;           /**< \brief PWM input clock frequency (cached value) */
    Timer_CountDir countDir;            /**< \brief Timer counting mode */
};

/** \name Base functions
 * These functions are implemented in Timer.c but only usable after a driver has been initialised.
 * \{ */
IFX_EXTERN TimerVal Timer_getPeriod(Timer *handle);
IFX_EXTERN float32  Timer_getFrequency(Timer *handle);
IFX_EXTERN boolean  Timer_setFrequency(Timer *handle, float32 frequency);
IFX_EXTERN float32  Timer_getResolution(Timer *handle);
IFX_EXTERN float32  Timer_tickToS(Timer *handle, TimerVal ticks);
IFX_EXTERN TimerVal Timer_sToTick(Timer *handle, float32 seconds);
/** \} */

/** \name Virtual functions
 * These functions are implemented by a driver.
 * \{ */
IFX_INLINE void    Timer_applyUpdate(Timer *handle);
IFX_INLINE void    Timer_disableUpdate(Timer *handle);
IFX_INLINE void    Timer_run(Timer *handle);
IFX_INLINE boolean Timer_setPeriod(Timer *handle, TimerVal period);
IFX_INLINE void    Timer_setSingleMode(Timer *handle, boolean enabled);
IFX_INLINE void    Timer_setTrigger(Timer *handle, TimerVal triggerPoint);
IFX_INLINE void    Timer_stop(Timer *handle);
IFX_INLINE boolean Timer_ackTimerIrq(Timer *handle);
IFX_INLINE boolean Timer_ackTriggerIrq(Timer *handle);
/** \} */

/** \name Protected functions
 * Only the driver may use these functions.
 * \{ */
IFX_EXTERN void Timer_initConfig(Timer_Config *config);
/** \} */

/** \} */

//________________________________________________________________________________________
//  INLINE FUNCTION IMPLEMENTATIONS

IFX_INLINE boolean Timer_ackTimerIrq(Timer *handle)
{
    return handle->functions.ackTimerIrq(handle->driver);
}


IFX_INLINE boolean Timer_ackTriggerIrq(Timer *handle)
{
    return handle->functions.ackTriggerIrq(handle->driver);
}


/** \brief Apply requested updates.
 *
 * Such requested update are calls to:
 * - Timer_setSingleMode()
 * - Timer_setTrigger()
 * - Timer_setPeriod() or Timer_setFrequency()
 * as well as PWM generation related calls such as:
 * - PwmHl_setDeadtime()
 * - PwmHl_setMode()
 * - PwmHl_setOnTime()
 * - PwmHl_setupChannels()
 *
 * It is important to note, that only one call to Timer_applyUpdate() is allowed within
 * one timer period!!!
 */
IFX_INLINE void Timer_applyUpdate(Timer *handle)
{
    handle->functions.applyUpdate(handle->driver);
}


IFX_INLINE void Timer_disableUpdate(Timer *handle)
{
    handle->functions.disableUpdate(handle->driver);
}


/** \brief Set the run flag of timer resource.
 *
 * Please note that other mechanism may avoid the timer to tick, such as when the
 * global (input) clock to the timer block is inactive.
 */
IFX_INLINE void Timer_run(Timer *handle)
{
    handle->functions.run(handle->driver);
}


/** \brief Set the new period.
 *
 * Timer_applyUpdate() shall be invoked in order this to take effect. */
IFX_INLINE boolean Timer_setPeriod(Timer *handle, TimerVal period)
{
    return handle->functions.setPeriod(handle->driver, period);
}


/** \brief Set to stop timer after one period is elapsed.
 *
 * Timer_applyUpdate() shall be invoked in order this to take effect. */
IFX_INLINE void Timer_setSingleMode(Timer *handle, boolean enabled)
{
    handle->functions.setSingleMode(handle->driver, enabled);
}


/** \brief Set trigger point.
 *
 * Below trigger point, the output is active.
 * Beyond trigger point, the output is inactive.
 * Active state is defined by Timer_Config::outputActiveState.
 *
 * Timer_applyUpdate() shall be invoked in order this to take effect. */
IFX_INLINE void Timer_setTrigger(Timer *handle, TimerVal triggerPoint)
{
    handle->functions.setTrigger(handle->driver, triggerPoint);
}


/** \brief Clear the run flag of timer resource */
IFX_INLINE void Timer_stop(Timer *handle)
{
    handle->functions.stop(handle->driver);
}


#endif /* TIMER_H */
