/**
 * \file IfxDma_Dma.c
 * \brief DMA functions
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 *
 * $Revision: $0.6
 * $Date: $2013-08-05 13:05:48 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 */

/*________________________________________________________________________________________ */
/* INCLUDES */

#include "IfxDma_Dma.h"

/*________________________________________________________________________________________ */
/* LOCAL FUNCTION IMPLEMENTATIONS (MODULE BASE ADDRESS) */

void IfxDma_Dma_initModuleConfig(IfxDma_Dma_Config *config, Ifx_DMA *dma)
{
    config->dma = dma;
}


void IfxDma_Dma_initModule(IfxDma_Dma *dma, IfxDma_Dma_Config *config)
{
    dma->dma = config->dma;
}


void IfxDma_Dma_createModuleHandle(IfxDma_Dma *dmaHandle, Ifx_DMA *dma)
{
    dmaHandle->dma = dma;
}


void IfxDma_Dma_deInitModule(IfxDma_Dma *dma)
{
    //TODO
}


void IfxDma_Dma_initChannelConfig(IfxDma_Dma_ChannelConfig *config, IfxDma_Dma *dma)
{
    config->module                                 = dma;
    config->channelId                              = IfxDma_ChannelId_0;
    config->sourceAddress                          = 0;
    config->destinationAddress                     = 0;
    config->shadowAddress                          = 0;
    config->readDataCrc                            = 0;
    config->sourceDestinationAddressCrc            = 0;
    config->transferCount                          = 0;
    config->blockMode                              = IfxDma_ChannelMove_1;
    config->requestMode                            = IfxDma_ChannelRequestMode_oneTransferPerRequest;
    config->operationMode                          = IfxDma_ChannelOperationMode_single;
    config->moveSize                               = IfxDma_ChannelMoveSize_8bit;
    config->requestSource                          = IfxDma_ChannelRequestSource_peripheral;
    config->busPriority                            = IfxDma_ChannelBusPriority_medium;
    config->pattern                                = IfxDma_ChannelPattern_0_disable; //enter here depending on enum
    config->hardwareRequestEnabled                 = 0;

    config->sourceAddressIncrementStep             = IfxDma_ChannelIncrementStep_1;
    config->sourceAddressIncrementDirection        = IfxDma_ChannelIncrementDirection_positive;
    config->sourceAddressCircularRange             = IfxDma_ChannelIncrementCircular_32768;

    config->destinationAddressIncrementStep        = IfxDma_ChannelIncrementStep_1;
    config->destinationAddressIncrementDirection   = IfxDma_ChannelIncrementDirection_positive;
    config->destinationAddressCircularRange        = IfxDma_ChannelIncrementCircular_32768;

    config->shadowControl                          = IfxDma_ChannelShadow_none;

    config->sourceCircularBufferEnabled            = 0;
    config->destinationCircularBufferEnabled       = 0;
    config->timestampEnabled                       = 0;

    config->wrapSourceInterruptEnabled             = 0;
    config->wrapDestinationInterruptEnabled        = 0;
    config->channelInterruptEnabled                = 0;
    config->channelInterruptControl                = IfxDma_ChannelInterruptControl_thresholdLimitMatch;
    config->interruptRaiseThreshold                = 0;
    config->transactionRequestLostInterruptEnabled = 0;
}


/** local function to copy a transaction set into DMA channel SFRs or memory location (for linked lists) */
static void IfxDma_Dma_configureTransactionSet(Ifx_DMA_CH *channel, IfxDma_Dma_ChannelConfig *config)
{
    {
        Ifx_DMA_CH_CHCFGR chcfgr;
        chcfgr.U          = 0;
        chcfgr.B.TREL     = config->transferCount;
        chcfgr.B.BLKM     = config->blockMode;
        chcfgr.B.RROAT    = config->requestMode;
        chcfgr.B.CHMODE   = config->operationMode;
        chcfgr.B.CHDW     = config->moveSize;
        chcfgr.B.PRSEL    = config->requestSource;
        chcfgr.B.PATSEL   = config->pattern;

        channel->CHCFGR.U = chcfgr.U;
    }

    {
        Ifx_DMA_CH_ADICR adicr;
        adicr.U       = 0;
        adicr.B.SMF   = config->sourceAddressIncrementStep;
        adicr.B.INCS  = config->sourceAddressIncrementDirection;
        adicr.B.CBLS  = config->sourceAddressCircularRange;
        adicr.B.SCBE  = config->sourceCircularBufferEnabled;
        adicr.B.DMF   = config->destinationAddressIncrementStep;
        adicr.B.INCD  = config->destinationAddressIncrementDirection;
        adicr.B.CBLD  = config->destinationAddressCircularRange;
        adicr.B.DCBE  = config->destinationCircularBufferEnabled;
        adicr.B.SHCT  = config->shadowControl;
        adicr.B.STAMP = config->timestampEnabled;
        adicr.B.WRPSE = config->wrapSourceInterruptEnabled;
        adicr.B.WRPDE = config->wrapDestinationInterruptEnabled;
        adicr.B.INTCT = (config->channelInterruptEnabled ? 2 : 0) | (config->channelInterruptControl ? 1 : 0);
        adicr.B.IRDV  = config->interruptRaiseThreshold;
        adicr.B.ETRL  = config->transactionRequestLostInterruptEnabled;
        //enter also the circular buffer enable bits

        channel->ADICR.U = adicr.U;
    }

    channel->SADR.U   = config->sourceAddress;
    channel->DADR.U   = config->destinationAddress;
    channel->SDCRCR.U = config->sourceDestinationAddressCrc;
    channel->RDCRCR.U = config->readDataCrc;

    // write only allowed in certain shadow modes:
    if ((config->shadowControl == IfxDma_ChannelShadow_srcDirectWrite) ||
        (config->shadowControl == IfxDma_ChannelShadow_dstDirectWrite) ||
        (config->shadowControl == IfxDma_ChannelShadow_linkedList) ||
        (config->shadowControl == IfxDma_ChannelShadow_doubleDestinationBufferingHwSwSwitch)
        )
    {
        channel->SHADR.U = config->shadowAddress;
    }
}


void IfxDma_Dma_initChannel(IfxDma_Dma_Channel *channel, IfxDma_Dma_ChannelConfig *config)
{
    Ifx_DMA *dma = config->module->dma;

    channel->dma       = dma;
    channel->channelId = config->channelId;
    channel->channel   = &dma->CH[config->channelId];

    IfxDma_Dma_configureTransactionSet(channel->channel, config);

    {
        Ifx_DMA_TSR tsr;
        tsr.U = 0;

        if (config->hardwareRequestEnabled)
        {
            tsr.B.ECH = 1;
        }
        else
        {
            tsr.B.DCH = 1;
        }

        dma->TSR[channel->channelId].U = tsr.U;
    }
}


void IfxDma_Dma_initLinkedListEntry(void *ptrToAddress, IfxDma_Dma_ChannelConfig *config)
{
    IfxDma_Dma_configureTransactionSet((Ifx_DMA_CH *)ptrToAddress, config);
}


void IfxDma_Dma_startChannelTransaction(IfxDma_Dma_Channel *channel)
{
    channel->channel->CHCSR.U = 0;  // for linked lists: ensure that this memory location is initialized
    IfxDma_startChannelTransaction(channel->dma, channel->channelId);
}
