/**
 * \file IfxDma_Dma.h
 * \brief DMA functions
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 *
 * $Revision: $0.6
 * $Date: $2013-08-08 08:57:10 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * \defgroup IfxLld_Dma_Interface Interface Driver
 * \ingroup  IfxLld_Dma
 * \defgroup IfxLld_Dma_Interface_Channel_Initialize Channel Initialize functions
 * \ingroup  IfxLld_Dma_Interface
 * \defgroup IfxLld_Dma_Interface_Channel_Transaction_Initiate Channel Transaction Initiate functions
 * \ingroup  IfxLld_Dma_Interface
 * \defgroup IfxLld_Dma_Interface_Linked_List Linked List functions
 * \ingroup  IfxLld_Dma_Interface
 * \defgroup IfxLld_Dma_Interface_Module_Initialize Module Initialize functions
 * \ingroup  IfxLld_Dma_Interface
 *******************************************************************************/

#ifndef IFXDMA_DMA_H_
#define IFXDMA_DMA_H_

#include <_Impl/IfxDma_cfg.h>
#include <Dma/Std/IfxDma.h>

/** \addtogroup IfxLld_Dma_Interface_Channel_Initialize
 * \{ */

/** DMA base address data structure */
typedef struct IfxDma_Dma
{
    Ifx_DMA *dma;       /**< \brief Specifies the pointer to the DMA */
} IfxDma_Dma;

/** DMA channel configuration data structure */
typedef struct
{
    Ifx_DMA *dma;       /**< \brief Specifies the pointer to the DMA */
} IfxDma_Dma_Config;

typedef struct
{
    Ifx_DMA         *dma;           /**< \brief Specifies the pointer to the DMA registers */
    IfxDma_ChannelId channelId;     /**< \brief Specifies the DMA channel */
    Ifx_DMA_CH      *channel;       /**< \brief Specifies the pointer to the DMA registers */
} IfxDma_Dma_Channel;

/** DMA channel configuration data structure */
typedef struct
{
    IfxDma_Dma                      *module;                                    /**< \brief Specifies pointer to the IfxDma_Dma module handle */
    IfxDma_ChannelId                 channelId;                                 /**< \brief Specifies the channel being used */
    uint32                           sourceAddress;                             /**< \brief Source address for the DMA channel */
    uint32                           destinationAddress;                        /**< \brief Destination address for the DMA channel */
    uint32                           shadowAddress;                             /**< \brief Initial content of shadow address for the DMA channel */
    uint32                           readDataCrc;                               /**< \brief Checksum for read data of the channel */
    uint32                           sourceDestinationAddressCrc;               /**< \brief Checksum for source and destination address of channel */
    uint16                           transferCount;                             /**< \brief Number of transfers in a transaction */
    IfxDma_ChannelMove               blockMode;                                 /**< \brief Number of moves in a transfer */
    IfxDma_ChannelRequestMode        requestMode;                               /**< \brief A service request initiates a single transfer, or the complete transaction */
    IfxDma_ChannelOperationMode      operationMode;                             /**< \brief keep enable/disable the hardware channel request after a transaction */
    IfxDma_ChannelMoveSize           moveSize;                                  /**< \brief Read/write data size */
    IfxDma_ChannelPattern            pattern;                                   /**< \brief Pattern selection operation modes */
    IfxDma_ChannelRequestSource      requestSource;                             /**< \brief Request of channel transfer through hardware or daisy chain. channel transfer complete interrupt of previous channel will trigger the next channel request */
    IfxDma_ChannelBusPriority        busPriority;                               /**< \brief Bus priority select */
    boolean                          hardwareRequestEnabled;                    /**< \brief Enabling channel transaction via hardware request */

    IfxDma_ChannelIncrementStep      sourceAddressIncrementStep;                /**< \brief Describes the address offset with which the source address should be modified after each move */
    IfxDma_ChannelIncrementDirection sourceAddressIncrementDirection;           /**< \brief Decides whether the source address offset after each move should be added or decremented from the exisiting address */
    IfxDma_ChannelIncrementCircular  sourceAddressCircularRange;                /**< \brief Determines which part of the source address remains unchanged and therby not updated after each move */
    IfxDma_ChannelIncrementStep      destinationAddressIncrementStep;           /**< \brief Describes the address offset with which the destination address should be modified after each move */
    IfxDma_ChannelIncrementDirection destinationAddressIncrementDirection;      /**< \brief Decides whether the destination address offset after each move should be added or decremented from the exisiting address */
    IfxDma_ChannelIncrementCircular  destinationAddressCircularRange;           /**< \brief Determines which part of the destination address remains unchanged and therby not updated after each move */

    IfxDma_ChannelShadow             shadowControl;                             /**< \brief selects the shadow transfer mode */

    boolean                          sourceCircularBufferEnabled;               /**< \brief Enables/Disables the source circular buffering */
    boolean                          destinationCircularBufferEnabled;          /**< \brief Enables/Disables the destination circular buffering */
    boolean                          timestampEnabled;                          /**< \brief Enables/Disables the appendage of the time stamp after end of the last DMA move in a transaction */
    boolean                          wrapSourceInterruptEnabled;                /**< \brief An interrupt should be triggered whenever source address is wrapped */
    boolean                          wrapDestinationInterruptEnabled;           /**< \brief An interrupt should be triggered whenever destination address is wrapped */
    boolean                          channelInterruptEnabled;                   /**< \brief The channel interrupt should be triggered. See also channelInterruptControl */
    IfxDma_ChannelInterruptControl   channelInterruptControl;                   /**< \brief The channel interrupt can either be triggered depending on the interruptRaiseThreshold, or each time the transaction count is decremented */
    uint8                            interruptRaiseThreshold;                   /**< \brief The value of the transferCount at which the interrupt should be raised */
    boolean                          transactionRequestLostInterruptEnabled;    /**< \brief Enables/Disables the channel transaction request lost interrupt */
} IfxDma_Dma_ChannelConfig;

/*******************************************************************************
**            Global Exported function prototypes              **
*******************************************************************************/

/** \brief initialize the DMA channel
 * \param channel pointer to the DMA base address and channel ID
 * \param config pointer to the DMA default channel configuration
 * \return void
 *
 * \code
 *
 *    // Initialize the testrun with help of below code snippet
 *
 *    // create module config
 *    IfxDma_Dma_Config dmaConfig;
 *    IfxDma_Dma_initModuleConfig(&dmaConfig, &MODULE_DMA);
 *
 *    // initialize module
 *    IfxDma_Dma dma;
 *    IfxDma_Dma_initModule(&dma, &dmaConfig);
 *
 *    // initial configuration for all channels
 *    IfxDma_Dma_ChannelConfig cfg;
 *    IfxDma_Dma_initChannelConfig(&cfg, &dma);
 *
 *    // following configuration is used by all channels
 *    cfg.transferCount = BUFFER_SIZE;
 *    cfg.requestMode = IfxDma_ChannelRequestMode_completeTransactionPerRequest;
 *    cfg.moveSize = IfxDma_ChannelMoveSize_32bit;
 *    // channel specific configurations
 *    for(int i=0; i<NUM_CHANNELS; ++i) {
 *      cfg.channelId = (IfxDma_ChannelId)i;
 *      cfg.sourceAddress = ADDR_CPU_DSPR(IfxCpu_getCoreID(), &sourceBuffer[i][0]);
 *      cfg.destinationAddress = ADDR_CPU_DSPR(IfxCpu_getCoreID(), &destinationBuffer[i][0]);
 *        IfxDma_Dma_initChannel(&chn[i], &cfg);
 *    }
 *
 *    // start the channel transactions
 *    for(int i=0; i<NUM_CHANNELS; ++i) {
 *        IfxDma_Dma_startChannelTransaction(&chn[i]);
 *    }
 *
 *    // wait until all transactions are completed
 *    bool notFinished;
 *    do {
 *        notFinished = false;
 *        for(int i=0; i<NUM_CHANNELS; ++i) {
 *            if( IfxDma_getChannelTransferCount(chn[i].dma, chn[i].channelId) ) {
 *                notFinished = true;
 *                break;
 *            }
 *        }
 *    } while( notFinished );
 *
 *    // Check the data integrity with help of below code snippet
 *    for(int i=0; i<NUM_CHANNELS; ++i) {
 *        uint32 *dst = (uint32 *)&destinationBuffer[i];
 *        uint32 chnWord = i * 0x10000000;
 *
 *        for(int j=0; j<BUFFER_SIZE; ++j)
 *            result |= *dst++ != chnWord++;
 *    }
 *
 * \endcode
 */
IFX_EXTERN void IfxDma_Dma_initChannel(IfxDma_Dma_Channel *channel, IfxDma_Dma_ChannelConfig *config);

/** \brief initialize the DMA module channel configuration
 * \param dma pointer to the DMA module
 * \param config pointer to the DMA default channel configuration
 * \return void
 *
 * A coding example can be found in \ref IfxDma_Dma_initChannel
 * \endcode
 */
IFX_EXTERN void IfxDma_Dma_initChannelConfig(IfxDma_Dma_ChannelConfig *config, IfxDma_Dma *dma);

/** \} */
/** \addtogroup IfxLld_Dma_Interface_Channel_Transaction_Initiate
 * \{ */

/** \brief initiate the the DMA transaction
 * \param channel pointer to the DMA base address and channel ID
 * \return void
 *
 * \code
 * if (IfxDma_isChannelReset(chn[0].dma, chn[0].channelId) &&
 *     (!IfxDma_isChannelTransactionEnable(chn[0].dma, chn[0].channelId))) {
 *     // DMD is out of RESET and there is no hardware request enabled
 *     IfxDma_Dma_startChannelTransaction(&chn[0]);
 * }
 * \endcode
 */
IFX_EXTERN void IfxDma_Dma_startChannelTransaction(IfxDma_Dma_Channel *channel);

/** \} */
/** \addtogroup IfxLld_Dma_Interface_Linked_List
 * \{ */

/** \brief linked list functionality within the DMA module
 * \param ptrToAddress pointer to the memory location where the linked list entry should be stored
 * \param config pointer to the transaction configuration
 * \return void
 *
 * \code
 * // Before initialization please adopt the below steps
 *
 * IfxDma_Dma_Channel chn;
 *
 * // Linked List storage
 * // IMPORTANT: it has to be aligned to an 64bit address, otherwise DMA can't read it
 * #define NUM_LINKED_LIST_ITEMS 5
 * __attribute__ ((aligned(64))) Ifx_DMA_CH linkedList[NUM_LINKED_LIST_ITEMS] ;
 *
 * // transfer these values to various CAN_MODATA[LH] registers via linked lists
 * #define NUM_TRANSFERED_WORDS 2
 * uint32 sourceBuffer[NUM_LINKED_LIST_ITEMS][NUM_TRANSFERED_WORDS];
 *
 * const uint32 destinationAddresses[NUM_LINKED_LIST_ITEMS] = {
 *    (uint32)&CAN_MODATAL0,
 *    (uint32)&CAN_MODATAL1,
 *    (uint32)&CAN_MODATAL2,
 *    (uint32)&CAN_MODATAL3,
 *    (uint32)&CAN_MODATAL4,
 * };
 *
 * // Initialize the linked list testrun with help of below snippet
 *
 * // Generate the module configuration
 * IfxDma_Dma_Config dmaConfig;
 * IfxDma_Dma_initModuleConfig(&dmaConfig, &MODULE_DMA);
 *
 * // initialize the module
 * IfxDma_Dma dma;
 * IfxDma_Dma_initModule(&dma, &dmaConfig);
 *
 * // initial configuration for all channels
 * // Default configuration will be set to all channels
 * IfxDma_Dma_ChannelConfig cfg;
 * IfxDma_Dma_initChannelConfig(&cfg, &dma);
 *
 * // following configuration is used by all channels
 * // configuration can be changed from the default one
 *
 * cfg.transferCount = BUFFER_SIZE;
 *
 * // generate linked list items
 * for(int i=0; i<NUM_LINKED_LIST_ITEMS; ++i) {
 *    cfg.sourceAddress = ADDR_CPU_DSPR(IfxCpu_getCoreID(), &sourceBuffer[i]);
 *    cfg.destinationAddress = destinationAddresses[i];
 *
 *    // address to next transaction set
 *    cfg.shadowAddress = ADDR_CPU_DSPR(IfxCpu_getCoreID(), (uint32)&linkedList[(i + 1) % NUM_LINKED_LIST_ITEMS]);
 *
 *    // transfer first transaction set into DMA channel
 *    if( i == 0 ) {
 *        IfxDma_Dma_initChannel(&chn, &cfg);
 *    }
 *
 *    // transfer into linked list storage
 *    IfxDma_Dma_initLinkedListEntry((void *)&linkedList[i], &cfg);
 *
 * }
 *
 * // Use below snippet for running the test
 *
 * for(int loop=0; loop<3; ++loop) {
 *
 *    // write "house numbers" into source buffer
 *    for(int i=0; i<NUM_LINKED_LIST_ITEMS; ++i) {
 *        uint32 *src = (uint32 *)&sourceBuffer[i];
 *        for(int j=0; j<NUM_TRANSFERED_WORDS; ++j) {
 *            *src++ = loop*0x10000000 + i*0x00010000 + j + 1;
 *        }
 *    }
 *
 *    // clear service request flag
 *    MODULE_SRC.DMA.CH[chn.channelId].B.CLRR = 1;
 *
 *    // start linked list transaction
 *    IfxDma_Dma_startChannelTransaction(&chn);
 *
 *    // wait for service request which is triggered at the end of linked list transfers
 *    while( !MODULE_SRC.DMA.CH[chn.channelId].B.SRR );
 *
 *    // check transfered values against expected values
 *
 *    for(int i=0; i<NUM_LINKED_LIST_ITEMS; ++i) {
 *         uint32 *src = (uint32 *)&sourceBuffer[i];
 *         uint32 *dst = (uint32 *)destinationAddresses[i];
 *         for(int j=0; j<NUM_TRANSFERED_WORDS; ++j) {
 *             result |= *dst++ != *src++;
 *         }
 *    }
 * }
 * \endcode
 */

/** \brief Returns the SRC pointer for given DMA channel
 *
 * \param channel pointer to the DMA base address and channel ID
 */
IFX_INLINE volatile Ifx_SRC_SRCR *IfxDma_Dma_getSrcPointer(IfxDma_Dma_Channel *channel)
{
    return IfxDma_getSrcPointer(channel->dma, channel->channelId);
}


IFX_EXTERN void IfxDma_Dma_initLinkedListEntry(void *ptrToAddress, IfxDma_Dma_ChannelConfig *config);

/** \} */
/** \addtogroup IfxLld_Dma_Interface_Module_Initialize
 * \{ */

/** \brief initialize the DMA module
 * \param dma pointer to the DMA module handle
 * \param config pointer to the transaction configuration
 * \return void
 *
 * A coding example can be found in \ref IfxDma_Dma_initChannel
 */

IFX_EXTERN void IfxDma_Dma_initModule(IfxDma_Dma *dma, IfxDma_Dma_Config *config);

/** \brief initialize the DMA module configuration
 * \param config pointer to the DMA register space
 * \param dma pointer to the DMA module
 * \return void
 *
 * A coding example can be found in \ref IfxDma_Dma_initChannel
 * \endcode
 */
IFX_EXTERN void IfxDma_Dma_initModuleConfig(IfxDma_Dma_Config *config, Ifx_DMA *dma);

/** \brief de-initialize the DMA module
 * \param dma pointer to the DMA module handle
 * \return void
 */
IFX_EXTERN void IfxDma_Dma_deInitModule(IfxDma_Dma *dma);

/** \brief initializes a DMA module handle based on the current configuration.
 *  Can be used in code where it's ensure that the DMA module is already initialized, and a DMA handle isn't globally available.
 * \param dma pointer to the DMA module handle
 * \return void
 *
 * Coding example:
 * \code
 *     IfxDma_Dma dma;
 *     IfxDma_Dma_createModuleHandle(&dma, &MODULE_DMA);
 * \endcode
 */

IFX_EXTERN void IfxDma_Dma_createModuleHandle(IfxDma_Dma *dmaHandle, Ifx_DMA *dma);

/** \} */

#endif
