/**
 * \file IfxGpt12_IncrEnc.c
 * \brief GPT12-based Incremental Encoder Interface
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: 1334 $
 * $Date: 2013-01-16 12:50:25 +0100 (Wed, 16 Jan 2013) $
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 */

#include "IfxGpt12_IncrEnc.h"

typedef const struct
{
    union { Ifx_GPT12_T3CON_Bits B; uint32 U; } T3CON;
    union { Ifx_GPT12_T4CON_Bits B; uint32 U; } T4CON;
    union { Ifx_GPT12_T2CON_Bits B; uint32 U; } T2CON;
    union { Ifx_GPT12_T5CON_Bits B; uint32 U; } T5CON;
} IfxGpt12_IncrEnc_ConfigRegs;


/** Configuration of GPT12 using T3 for incremental encoder operation:
 * - T3 is used as encoder A B main processing
 * - T4 is used as Z (synchronisation) pulse
 * - T2 is used as concatenation of T3, forming 32-bit timer
 *
 * Interrupt is generated every synchronisation pulse.
 */
/*
IfxGpt12_IncrEnc_ConfigRegs IfxGpt12_IncrEnc_regsT3AsCore =
{
    .T3CON = {
        .B.T3M  = IfxGpt12_TxCON_TxM_incrementalEdgeMode,
        .B.T3I  = IfxGpt12_TxCON_TxI_incrementalBothTxEudEdges,
        .B.T3R  = IfxGpt12_TxCON_TxR_startRun,
        .B.T3UDE= IfxGpt12_TxCON_TxEUD_directionOnTxEudControl,
    },
    .T4CON = {
        .B.T4M  = IfxGpt12_TxCON_TxM_captureMode,
        .B.T4I  = IfxGpt12_TxCON_TxI_risingEdge,
        .B.T4RC = IfxGpt12_TxCON_TxRC_remoteTxR
    },
    .T2CON = {
        .B.T2M  = IfxGpt12_TxCON_TxM_counterMode,
        .B.T2I  = IfxGpt12_TxCON_TxI_bothEdgesT3Otl,
        .B.T2RC = IfxGpt12_TxCON_TxRC_remoteTxR
    },
};*/
IfxGpt12_IncrEnc_ConfigRegs IfxGpt12_IncrEnc_regsT3AsCore =
{
    .T3CON = {
        .B.T3M  = IfxGpt12_TxCON_TxM_incrementalEdgeMode,
        .B.T3I  = IfxGpt12_TxCON_TxI_incrementalBothTxEudEdges,
        .B.T3R  = IfxGpt12_TxCON_TxR_startRun,
        .B.T3UDE= IfxGpt12_TxCON_TxEUD_directionOnTxEudControl,
    },
    .T4CON = {
        .B.T4M  = IfxGpt12_TxCON_TxM_captureMode,
        .B.T4I  = IfxGpt12_TxCON_TxI_risingEdge,
        .B.CLRT3EN = 1,
        .B.T4RC = IfxGpt12_TxCON_TxRC_remoteTxR
    },
    .T2CON = {
        .B.T2M  = IfxGpt12_TxCON_TxM_timerMode,
        .B.T2I  = IfxGpt12_TxCON_TxI_ignored,
        .B.T2RC = IfxGpt12_TxCON_TxRC_ownTxR
    },
    .T5CON = {
    	.B.T5I = 2,  // Prescaler of 16 with BPS2  = 0
    	.B.T5M = IfxGpt12_TxCON_TxM_timerMode,
    	.B.CT3 = 1,
    	.B.T5R  = IfxGpt12_TxCON_TxR_startRun,
    	.B.CI = 1, //Positiv Transistion of T3IN
    	.B.T5CLR = 1, // Clear on Capture Event
    	.B.T5SC = 1	// Enable Capture
    }

};

/** Configuration of GPT12 using T2 for incremental encoder operation:
 * - T2 is used as encoder A B main processing
 * - T4 is used as Z (synchronisation) pulse and reset pulse
 */
IfxGpt12_IncrEnc_ConfigRegs IfxGpt12_IncrEnc_regsT2AsCore =
{
    .T3CON = { .U = 0
   	},
    .T4CON = {	// not usable for speed measuring
        .B.T4M  = IfxGpt12_TxCON_TxM_captureMode,
        .B.T4I  = IfxGpt12_TxCON_TxI_risingEdge,
        .B.T4RC = IfxGpt12_TxCON_TxRC_ownTxR,
        .B.T4R  = IfxGpt12_TxCON_TxR_startRun,
        .B.CLRT2EN	= 1,		// Falling edge at T4EUD Clears T2
    },
    .T2CON = {
        .B.T2M  = IfxGpt12_TxCON_TxM_incrementalEdgeMode,
        .B.T2I  = IfxGpt12_TxCON_TxI_incrementalBothTxEudEdges,
        .B.T2RC = IfxGpt12_TxCON_TxRC_ownTxR,
        .B.T2R  = IfxGpt12_TxCON_TxR_startRun,
        .B.T2UDE= IfxGpt12_TxCON_TxEUD_directionOnTxEudControl
    },
    .T5CON = {// 5 is not useable for speed measuring
    	.U = 0  // not used
    }

};

IfxGpt12_IncrEnc g_IfxGpt12_IncrEnc[GPT12_INCRENC_COUNT];
extern const PosIf_Calls g_PosIf_IncrEnc_Gpt12_T3;
extern const PosIf_Calls g_PosIf_IncrEnc_Gpt12_T2;


/** \brief Initialise the handle using given configuration */
void IfxGpt12_IncrEnc_init(IfxGpt12_IncrEnc* gptEnc, const IfxGpt12_IncrEnc_Config* config)
{
    Ifx_GPT12* gpt12 = config->module;
    gptEnc->module = gpt12;
    PosIf_init(&gptEnc->base, &config->base);
    gptEnc->timerOffset = 0;
    gptEnc->base.status.B.notSynchronised = 1;

    IfxGpt12_enableModule(gpt12);

    if (config->pinForwardTxIn->timer == 3)
    {
        gptEnc->base.functions = &g_PosIf_IncrEnc_Gpt12_T3;

        IFXGPT12(gpt12).T4CON.U = IfxGpt12_IncrEnc_regsT3AsCore.T4CON.U;
        IFXGPT12(gpt12).T2CON.U = IfxGpt12_IncrEnc_regsT3AsCore.T2CON.U;
        IFXGPT12(gpt12).T3CON.U = IfxGpt12_IncrEnc_regsT3AsCore.T3CON.U;
        IFXGPT12(gpt12).T5CON.U = IfxGpt12_IncrEnc_regsT3AsCore.T5CON.U;
        {   /* setup interrupt */
            volatile Ifx_SRC_SRCR* src = &(MODULE_SRC.GPT12.GPT12[0].T4);
            IfxSrc_init(src, config->isrProvider, config->servPriority);
            IfxSrc_enable(src);
        }
    }
    else if (config->pinForwardTxIn->timer == 2)
    {
        gptEnc->base.functions = &g_PosIf_IncrEnc_Gpt12_T2;

        IFXGPT12(gpt12).T4CON.U = IfxGpt12_IncrEnc_regsT2AsCore.T4CON.U;
        IFXGPT12(gpt12).T2CON.U = IfxGpt12_IncrEnc_regsT2AsCore.T2CON.U;
        //IFXGPT12(gpt12).T3CON.U = IfxGpt12_IncrEnc_regsT2AsCore.T3CON.U;
        {   /* setup interrupt */
            volatile Ifx_SRC_SRCR* src = &(MODULE_SRC.GPT12.GPT12[0].T4);
            IfxSrc_init(src, config->isrProvider, config->servPriority);
            IfxSrc_enable(src);
        }
    }

    IfxGpt12_configureTxInPin (config->pinForwardTxIn,   config->pinMode);
    IfxGpt12_configureTxEudPin(config->pinBackwardTxEud, config->pinMode);
    if (config->pinZeroTxIn != NULL_PTR)
    {
        IfxGpt12_configureTxInPin (config->pinZeroTxIn,  config->pinMode);
    }
}

#define TIMER_32_BIT 0

/** \brief Handle the on Zero pulse interrupt */
void IfxGpt12_IncrEnc_isrOnZeroPulse(IfxGpt12_IncrEnc* gptEnc)
{
#if TIMER_32_BIT
    Ifx_GPT12* gpt12 = gptEnc->module;
    gptEnc->timerOffset = (IFXGPT12(gpt12).T2.U << 16) | IFXGPT12(gpt12).T4.U;
#endif
    if (gptEnc->base.status.B.notSynchronised)
    {
        gptEnc->base.status.B.notSynchronised = 0;
    }
}

/** \brief Update internal data when incremental mode is using T3.
 * This function shall be periodically called */
PosIf_Raw IfxGpt12_IncrEnc_updateFromT3(IfxGpt12_IncrEnc* gptEnc)
{
  //  Ifx_GPT12* gpt12 = gptEnc->module;

#if TIMER_32_BIT
    uint32 newTimerTicks = (IFXGPT12(gpt12).T2.U << 16) | IFXGPT12(gpt12).T3.U;
    PosIf_Raw newPosition = (PosIf_Raw)(newTimerTicks - gptEnc->timerOffset);
#else
    PosIf_Raw newPosition = (IFXGPT12(gpt12).T3.U);
#endif
    newPosition = (newPosition + gptEnc->base.offset) & (gptEnc->base.resolution - 1);
    //newPosition = (newPosition + gptEnc->base.offset) % (gptEnc->base.resolution);
    gptEnc->base.position = newPosition;
    gptEnc->base.direction = IFXGPT12(gpt12).T3CON.B.T3RDIR;
    return newPosition;
}

/** \brief Update internal data when incremental mode is using T2.
 * This function shall be periodically called */
PosIf_Raw IfxGpt12_IncrEnc_updateFromT2(IfxGpt12_IncrEnc* gptEnc)
{
  //  Ifx_GPT12* gpt12 = gptEnc->module;

    PosIf_Raw newPosition = IFXGPT12(gpt12).T2.U;
    newPosition = (gptEnc->base.reversed != 0) ? -newPosition : newPosition;
    //newPosition = (newPosition + gptEnc->base.offset) & (gptEnc->base.resolution - 1);
    newPosition = (newPosition + gptEnc->base.offset) % (gptEnc->base.resolution);
    gptEnc->base.position = newPosition;
    gptEnc->base.direction = IFXGPT12(gpt12).T2CON.B.T2RDIR;
    return newPosition;
}

PosIf_Raw IfxGpt12_IncrEnc_resetOffset(PosIf* handle)
{
    PosIf_Raw offset;
    offset = (handle->offset - handle->position);
    PosIf_setOffset(handle, offset);
    return handle->offset;
}

void IfxGpt12_IncrEnc_setOffset(PosIf* handle, PosIf_Raw offset)
{
    handle->offset = offset;
}

void IfxGpt12_IncrEnc_setupElAngleConst(PosIf* handle, sint32 motorPolePairs)
{
    handle->elAngleConst = (float32)motorPolePairs *
        ((float32)POSIF_ANGLE_RANGE / (handle->periodPerRotation * handle->resolution));
}

StdReal IfxGpt12_IncrEnc_updateSpeedFromT3(IfxGpt12_IncrEnc* gptEnc, boolean update)
{
	Ifx_GPT12* gpt12 = gptEnc->module;
	float32 speed = 0.0;
   	// Check if an overflow of T5 is recognized
	volatile Ifx_SRC_SRCR* src = &(MODULE_SRC.GPT12.GPT12[0].T5);
    if(src->B.SRR != 1)
    {
    	volatile Ifx_SRC_SRCR* src = &(MODULE_SRC.GPT12.GPT12[0].CIRQ);
    	// Check if a new value is captured
    	if(src->B.SRR != 0)
    	{
    		// Get Timer Frequency
    		float32 frequency = IfxGpt12_getTimerFrequency(gpt12, IfxGpt12_Timer5);
    		uint32 captureValue = IFXGPT12(gpt12).CAPREL.B.CAPREL;
    		// Calculate Speed rad/s
    		speed = 2 * PI * frequency / captureValue / (gptEnc->base.resolution);
    		// Delete Capture Request Bit
    		src->B.CLRR = 1;
    		gptEnc->base.speed = speed;
    	}
    }else
    {
    	// Delete Overflow Request bit
    	src->B.CLRR = 1;
    	gptEnc->base.speed = 0.0;
    }
    return gptEnc->base.speed;
}

StdReal IfxGpt12_IncrEnc_updateSpeedFromT2(IfxGpt12_IncrEnc* gptEnc, boolean update)
{
//	Ifx_GPT12* gpt12 = gptEnc->module;
    // TODO: IfxGpt12_IncrEnc_updateSpeed
	// Check if an overflow of T5 is recognized
	// Delete Overflow Request bit
   	gptEnc->base.speed = 0.0;

    return gptEnc->base.speed;
}

const PosIf_Calls g_PosIf_IncrEnc_Gpt12_T3 =
{
    &IfxGpt12_IncrEnc_resetOffset,
    &IfxGpt12_IncrEnc_setOffset,
    &IfxGpt12_IncrEnc_setupElAngleConst,
    &PosIf_setupSpeedConst_base,
    (void*)&IfxGpt12_IncrEnc_updateFromT3,
    &IfxGpt12_IncrEnc_updateSpeedFromT3,
};

const PosIf_Calls g_PosIf_IncrEnc_Gpt12_T2 =
{
    &IfxGpt12_IncrEnc_resetOffset,
    &IfxGpt12_IncrEnc_setOffset,
    &IfxGpt12_IncrEnc_setupElAngleConst,
    &PosIf_setupSpeedConst_base,
    (void*)&IfxGpt12_IncrEnc_updateFromT2,
   &IfxGpt12_IncrEnc_updateSpeedFromT2,
};

