/**
 * \file IfxGtmAtom_PwmHl.c
 * \brief GTM ATOM dual top-bottom PWM channel functions functions, with timer
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1479.6
 * $Date: $2013-08-09 16:42:23 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 */

/* *INDENT-OFF* Note: this file was indented manually by the author. */
//________________________________________________________________________________________
// INCLUDES

#include "IfxGtmAtom_PwmHl.h"
#include "_Utilities/Ifx_Assert.h"
#include "stddef.h"

//________________________________________________________________________________________
// LOCAL FUNCTION PROTOTYPES

//________________________________________________________________________________________
// FUNCTION IMPLEMENTATIONS

IFX_INLINE ActiveState IfxGtmAtom_PwmHl_invertActiveState(ActiveState activeState)
{
    return activeState == ActiveState_low ? ActiveState_high : ActiveState_low;
}

/** Update the x output duty cycle in edge aligned modes (left and right aligned)
 *
 * \return None.
 * \private
 */
static void IfxGtmAtom_PwmHl_updateEdgeAligned(IfxGtmAtom_PwmHl* driver, TimerVal* tOn)
{
    uint8 channelIndex;
    TimerVal period;
    TimerVal deadtime = driver->base.deadtime;

    period = driver->base.timer->period;

    for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
    {
        TimerVal x; /* x=period*dutyCycle, x=OnTime+deadTime */
        TimerVal cm0, cm1;
        x = tOn[channelIndex];
        if (driver->base.inverted != FALSE)
        {
            x = period - x;
        } else {}
        if ((x < driver->base.minPulse) || (x <= deadtime))
        {/* For deadtime condition: avoid leading edge of top channel to occur after the trailing edge */
            x = 0;
        }
        else if (x > driver->base.maxPulse)
        {
            x = period;
        }
        else {}

        /* Special handling due to GTM issue */
        if (x == period)
        { /* 100% duty cycle */
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->ccxTemp[channelIndex]  , period + 1 /* No compare event */, 2 /* 1st compare event (issue: expected to be 1)*/ + deadtime);
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->coutxTemp[channelIndex], period + 2 /* No compare event, issues has been seen with +1 */, 2 /* 1st compare event (issue: expected to be 1)*/);
        }
        else if (x == 0)
        {
            cm0 = 1;
            cm1 = period+2;
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->ccxTemp[channelIndex]  , cm0, cm1);
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->coutxTemp[channelIndex], cm0 + deadtime, cm1);
        }
        else
        { /* x% duty cycle */
            cm1 = 2; // CM1, set to 2 due to a GTM issue. should be 1 according to spec
            cm0 = x; // CM0, set to x+2 due to a GTM issue. should be x+1 according to spec
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->ccxTemp[channelIndex]  , cm0           , cm1 + deadtime);
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->coutxTemp[channelIndex], cm0 + deadtime, cm1           );
        }
    }
}

/** Update the x output duty cycle in center aligned and center aligned inverted modes
 *
 * \return None.
 * \private
 */
static void IfxGtmAtom_PwmHl_updateCenterAligned(IfxGtmAtom_PwmHl* driver, TimerVal* tOn)
{
    uint8 channelIndex;
    TimerVal period;
    TimerVal deadtime = driver->base.deadtime;

    period = driver->base.timer->period;

    for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
    {
        TimerVal x; /* x=period*dutyCycle, x=OnTime+deadTime */
        TimerVal cm0, cm1;
        x = tOn[channelIndex];
        if (driver->base.inverted != FALSE)
        {
            x = period - x;
        } else {}
        if ((x < driver->base.minPulse) || (x <= deadtime))
        {/* For deadtime condition: avoid leading edge of top channel to occur after the trailing edge */
            x = 0;
        }
        else if (x > driver->base.maxPulse)
        {
            x = period;
        }
        else {}

        /* Special handling due to GTM issue */
        if (x == period)
        { /* 100% duty cycle */
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->ccxTemp[channelIndex]  , period + 1 /* No compare event */, 2 /* 1st compare event (issue: expected to be 1)*/ + deadtime);
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->coutxTemp[channelIndex], period + 2 /* No compare event, issues has been seen with +1 */, 2 /* 1st compare event (issue: expected to be 1)*/);
        }
        else if (x == 0)
        {
            cm0 = 1;
            cm1 = period+2;
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->ccxTemp[channelIndex]  , cm0, cm1);
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->coutxTemp[channelIndex], cm0 + deadtime, cm1);
        }
        else
        { /* x% duty cycle */
            cm1 = (period - x) / 2; // CM1
            cm0 = (period + x) / 2; // CM0
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->ccxTemp[channelIndex]  , cm0           , cm1 + deadtime);
            IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->coutxTemp[channelIndex], cm0 + deadtime, cm1           );
        }
    }
}

/** Set the outputs to inactive
 *
 * \return None.
 */
static void IfxGtmAtom_PwmHl_updateOff(IfxGtmAtom_PwmHl* driver, TimerVal* tOn)
{
    uint8 channelIndex;
    TimerVal period;
    period = driver->base.timer->period;

    for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
    {
        IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->ccxTemp[channelIndex], 2 /* 1 will keep the previous level*/, period+2);
        IfxGtmAtom_Ch_setCompareShadow(driver->atom, driver->coutxTemp[channelIndex], period+1, 2);
    }
}

void IfxGtmAtom_PwmHl_setupChannels(IfxGtmAtom_PwmHl* driver, boolean* activeCh, boolean* stuckSt)
{
    /* FIXME TODO */
}

const IfxGtmAtom_PwmHl_Mode IfxGtmAtom_PwmHl_modes[5] =
{
    {FALSE, &IfxGtmAtom_PwmHl_updateCenterAligned}, // Pwm_Mode_centerAligned
    {TRUE , &IfxGtmAtom_PwmHl_updateCenterAligned}, // Pwm_Mode_centerAlignedInverted
    {FALSE, &IfxGtmAtom_PwmHl_updateEdgeAligned},   // Pwm_Mode_leftAligned
    {TRUE , &IfxGtmAtom_PwmHl_updateEdgeAligned},   // Pwm_Mode_rightAligned
    {FALSE, &IfxGtmAtom_PwmHl_updateOff}            // Pwm_Mode_off
};

/** Initialize the timer object
 *
 * \note To ensure that the channels counter are reset by the timer and do not overflow, leading to random signal on the output, the timer must be started before the call to this function.
 */
boolean IfxGtmAtom_PwmHl_init(IfxGtmAtom_PwmHl* driver, const IfxGtmAtom_PwmHl_Config* config)
{
    boolean result = TRUE;
    uint16 channelMask;
    uint16 channelsMask = 0;
    uint32 channelIndex;

    IfxGtmAtom_Timer* timer = (IfxGtmAtom_Timer*) config->base.timer;

    /* driver.base must be at offset 0 to be compatible with the standard interface PwmHl */
    IFX_ASSERT(VERBOSE_LEVEL_ERROR, offsetof(IfxGtmAtom_PwmHl, base) == 0);

    driver->base.driver                    = driver;
    driver->base.mode                      = Pwm_Mode_init;
    driver->base.timer                     = config->base.timer;
    driver->base.setMode                   = 0;
    driver->base.inverted                  = FALSE;
    driver->base.ccxActiveState            = config->base.ccxActiveState;
    driver->base.coutxActiveState          = config->base.coutxActiveState;
    driver->base.channelCount              = config->base.channelCount;

    /* Link pwm channel functions */
    driver->base.functions.getDeadtime     = &PwmHl_getDeadtime;
    driver->base.functions.getMinPulse     = &PwmHl_getMinPulse;
    driver->base.functions.getMode         = &PwmHl_getMode;
    driver->base.functions.setDeadtime     = &PwmHl_setDeadtime;
    driver->base.functions.setMinPulse     = &PwmHl_setMinPulse;
    driver->base.functions.setMode         = (PwmHl_SetMode)&IfxGtmAtom_PwmHl_setMode;
    driver->base.functions.setOnTime       = NULL_PTR; /* Set by setMode() below */
    driver->base.functions.setupChannels   = (PwmHl_SetupChannels)&IfxGtmAtom_PwmHl_setupChannels;

    PwmHl_setDeadtime(&driver->base, config->base.deadtime);
    PwmHl_setMinPulse(&driver->base, config->base.minPulse);

    driver->atom = &(timer->gtm->ATOM[config->atom]);
    /* Only one AGC */
    driver->agc  = (Ifx_GTM_ATOM_AGC *)&driver->atom->AGC_GLB_CTRL;

    IFX_ASSERT(VERBOSE_LEVEL_ERROR, config->base.channelCount <= IFXGTMATOM_PWMHLCH_MAX_CHANNELS);

    IfxGtmCmu_Clk clock = IfxGtmAtom_Ch_getClockSource(timer->atom, timer->timerChannel);

    for (channelIndex = 0; channelIndex < config->base.channelCount; channelIndex++)
    {
        IfxGtmAtom_Ch channel;
        /* CCX */
        channel = config->ccx[channelIndex]->channel;
        driver->ccx[channelIndex] = channel;
        channelMask = 1 << channel;
        channelsMask |= channelMask;

        /* Initialize the timer part */
        IfxGtmAtom_Ch_configurePwmMode(driver->atom, channel, clock,
                driver->base.inverted ? config->base.ccxActiveState :
                        IfxGtmAtom_PwmHl_invertActiveState(config->base.ccxActiveState),
               IfxGtmAtom_Ch_ResetEvent_onTrigger, IfxGtmAtom_Ch_OutputTrigger_forward);

        /* Initialize the port */
        IfxGtm_PinMap_setAtomTout(config->ccx[channelIndex],
                config->base.outputMode, config->base.outputDriver);

        /* COUTX */
        channel = config->coutx[channelIndex]->channel;
        driver->coutx[channelIndex] = channel;
        channelMask = 1 << channel;
        channelsMask |= channelMask;

        /* Initialize the timer part */
        IfxGtmAtom_Ch_configurePwmMode(driver->atom, channel, clock,
                driver->base.inverted ?
                        IfxGtmAtom_PwmHl_invertActiveState(config->base.coutxActiveState)
                        : config->base.coutxActiveState,
               IfxGtmAtom_Ch_ResetEvent_onTrigger, IfxGtmAtom_Ch_OutputTrigger_forward);

        /* Initialize the port */
        IfxGtm_PinMap_setAtomTout(config->coutx[channelIndex],
                config->base.outputMode, config->base.outputDriver);
    }

    IfxGtmAtom_Agc_enableChannelsOutput(driver->agc, channelsMask, 0, TRUE);
    IfxGtmAtom_Agc_enableChannels(driver->agc, channelsMask, 0, TRUE);

    IfxGtmAtom_PwmHl_setMode(driver, Pwm_Mode_off);

    TimerVal tOn[IFXGTMATOM_PWMHLCH_MAX_CHANNELS] = {0};
    IfxGtmAtom_PwmHl_updateOff(driver, tOn); /* tOn do not need defined values */

    /* Transfer the shadow registers */
    IfxGtmAtom_Agc_setChannelsForceUpdate(driver->agc, channelsMask, 0, 0, 0);
    IfxGtmAtom_Agc_trigger(driver->agc);
    IfxGtmAtom_Agc_setChannelsForceUpdate(driver->agc, 0, channelsMask, 0, 0);

    /* Enable timer to update the channels */

    for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
    {
        IfxGtmAtom_Timer_addToChannelMask(timer, 1 << driver->ccx[channelIndex]);
        IfxGtmAtom_Timer_addToChannelMask(timer, 1 << driver->coutx[channelIndex]);
    }

    return result;
}

/** Initialize the configuration structure to default
 *
 * \param config Pwm timer and channels configuration. This parameter is Initialised by the function
 *
 */
void IfxGtmAtom_PwmHl_initConfig(IfxGtmAtom_PwmHl_Config* config)
{
    PwmHl_initConfig(&config->base);
    config->atom            = IfxGtmAtom_0;
    config->ccx             = NULL_PTR;
    config->coutx           = NULL_PTR;
}

/** Set the PWM mode, the mode is only applied after setOnTime() + applyUpdate()
 *
 */
boolean IfxGtmAtom_PwmHl_setMode(IfxGtmAtom_PwmHl* driver, Pwm_Mode mode)
{
    boolean result = TRUE;
    PwmHl* base = &driver->base;;
    if (base->mode != mode)
    {
        if (mode > Pwm_Mode_off)
        {
            mode = Pwm_Mode_off;
            result = FALSE;
        }
        base->mode = mode;
        base->functions.setOnTime = (PwmHl_SetOnTime)IfxGtmAtom_PwmHl_modes[mode].update;

        if (base->mode != Pwm_Mode_off)
        {
            base->inverted = IfxGtmAtom_PwmHl_modes[mode].inverted;
        } else { /* Keep previous inverted for off mode */}

        if (base->inverted)
        {
            driver->ccxTemp = driver->coutx;
            driver->coutxTemp = driver->ccx;
        }
        else
        {
            driver->ccxTemp = driver->ccx;
            driver->coutxTemp = driver->coutx;
        }

        { /* Workaround to enable the signal inversion required for center aligned inverted
             and right aligned modes */

          /** \note Changing signal level may produce short circuit at the power stage,
           * in which case the inverter must be disable during this action.*/

          /* Pwm_Mode_centerAligned and Pwm_Mode_LeftAligned use inverted=FALSE */
          /* Pwm_Mode_centerAlignedInverted and Pwm_Mode_RightAligned use inverted=TRUE */
          uint32 channelIndex;
          for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
          {
              IfxGtmAtom_Ch channel;

              channel = driver->ccx[channelIndex];
              IfxGtmAtom_Ch_setSignalLevel(driver->atom, channel,  base->inverted
                      ? base->ccxActiveState
                      :  IfxGtmAtom_PwmHl_invertActiveState(driver->base.ccxActiveState));

              channel = driver->coutx[channelIndex];
              IfxGtmAtom_Ch_setSignalLevel(driver->atom, channel,  base->inverted
                      ? IfxGtmAtom_PwmHl_invertActiveState(base->coutxActiveState)
                      : base->coutxActiveState);
          }
        }
    }
    return result;
}
