/**
 * \file IfxGtmAtom_PwmHl.h
 * \brief GTM ATOM dual top-bottom PWM channel functions functions, with timer
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1479.5
 * $Date: $2013-08-09 16:42:24 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * \defgroup IfxLld_Gtm_atom_pwmhl ATOM PWM standard interface
 * \ingroup IfxLld_Gtm_interfaces
 *
 * This driver implements the PWM functionalities as defined by \ref library_srvsw_if_pwmhl.
 * The user is free to use either the driver specific APIs below or to used the \ref library_srvsw_if_pwmhl "standard interface APIs".
 *
 * \section specific Specific implementation
 * Implementation is similar to \ref IfxLld_Gtm_tom_pwmhl.
 *
 * \todo add documentation
 *
 * For a detailed configuration of the microcontroller, see \ref IfxGtmAtom_PwmHl_init().
 *
 * \section example: Usage example
 * Initialisation is done by, e.g:
 * \code
 * IfxGtmAtom_PwmHl_Config driverConfig;
 * IfxGtmAtom_PwmHl        driverData;
 * IfxGtmAtom_PwmHl_initConfig(&driverConfig, &MODULE_GTM);
 * IfxGtmAtom_PwmHl_init(&driverData, &driverConfig);
 * \endcode
 *
 * During run-time, \ref library_srvsw_if_pwmhl "the interface functions" shall be used, e.g.:
 * \code
 * PwmHl *pwmhl = &driverData.base;
 * Timer* timer = PwmHl_getTimer(pwmhl);
 * TimerVal onTime[3]; // assume configured for three HL channels
 *
 * onTime[0] = 10;
 * onTime[1] = 20;
 * onTime[2] = 30;
 *
 * Timer_disableUpdate(timer);
 * Timer_setPeriod(timer, period);
 * PwmHl_setOnTime(pwmhl, onTime);
 * Timer_applyUpdate(timer);
 * \endcode
 *
 */

/* *INDENT-OFF* Note: this file was indented manually by the author. */

#ifndef IFXGTMATOM_PWMHL_H
#define IFXGTMATOM_PWMHL_H

//________________________________________________________________________________________
// INCLUDES

#include "If/PwmHl.h"
#include "Gtm/Atom/Timer/IfxGtmAtom_Timer.h"

//________________________________________________________________________________________
// DEFINES

/** Maximal number of channels handled by the driver. One channel has a top and bottom pwm output
 * \ingroup IfxLld_Gtm_atom_pwmhl */
#define IFXGTMATOM_PWMHLCH_MAX_CHANNELS (8)

//________________________________________________________________________________________
// DATA STRUCTURES
typedef struct IfxGtmAtom_PwmHl_s IfxGtmAtom_PwmHl;

/** PWM configuration */
typedef struct
{
    boolean             inverted;                                  /**< \brief Inverted configuration for the selected mode */
    void                (*update)(IfxGtmAtom_PwmHl* driver, TimerVal* tOn); /**< \brief update call back function for the selected mode */
} IfxGtmAtom_PwmHl_Mode;

/** GTM ATOM PWM driver */
struct IfxGtmAtom_PwmHl_s
{
    PwmHl             base;                                 /**< \brief PWM channel standard interface */
    void                (*update)(IfxGtmAtom_PwmHl* driver, TimerVal* tOn); /**< \brief Update function for actual selected mode */
    Ifx_GTM_ATOM*       atom;                                       /**< \brief ATOM unit used */
    Ifx_GTM_ATOM_AGC*   agc;                                        /**< \brief AGC unit used */
    IfxGtmAtom_Ch       ccx[IFXGTMATOM_PWMHLCH_MAX_CHANNELS];       /**< \brief ATOM channels used for the CCCX outputs */
    IfxGtmAtom_Ch       coutx[IFXGTMATOM_PWMHLCH_MAX_CHANNELS];     /**< \brief ATOM channels used for the OUTX outputs */
    IfxGtmAtom_Ch*      ccxTemp;                                    /**< \brief cached value */
    IfxGtmAtom_Ch*      coutxTemp;                                  /**< \brief cached value */
};

/** GTM ATOM: PWM HL configuration
 *
 */
typedef struct
{
    PwmHl_Config                base;           /**< \brief PWM HL standard interface configuration */
    IfxGtmAtom                  atom;           /**< \brief ATOM unit used */
    const IfxGtmAtom_ToutMapP*  ccx;            /**< \brief Pointer to an array of size pwmHl.channels.channelCount containing the channels used. Channels must be adjacent channels */
    const IfxGtmAtom_ToutMapP*  coutx;          /**< \brief Pointer to an array of size pwmHl.channels.channelCount containing the channels used. Channels must be adjacent channels */
} IfxGtmAtom_PwmHl_Config;

//________________________________________________________________________________________
// FUNCTION PROTOTYPES

/**
 * \addtogroup  IfxLld_Gtm_atom_pwmhl
 * \{
 */
/** \brief Return a pointer to the standard pwmhl interface
 * \param driver Specifies the driver
 * \return a pointer to the standard pwmhl interface
 */
IFX_INLINE PwmHl* IfxGtmAtom_PwmHl_getInterface(IfxGtmAtom_PwmHl* driver)
{
    return &driver->base;
}

IFX_EXTERN boolean IfxGtmAtom_PwmHl_init      (IfxGtmAtom_PwmHl* driver, const IfxGtmAtom_PwmHl_Config* config);
IFX_EXTERN void    IfxGtmAtom_PwmHl_initConfig(IfxGtmAtom_PwmHl_Config* config);

/** \see PwmHl_getDeadtime() */
IFX_INLINE float32 IfxGtmAtom_PwmHl_getDeadtime(IfxGtmAtom_PwmHl* driver)
{ return driver->base.functions.getDeadtime(&driver->base);}
/** \see PwmHl_getMinPulse() */
IFX_INLINE float32 IfxGtmAtom_PwmHl_getMinPulse(IfxGtmAtom_PwmHl* driver)
{ return driver->base.functions.getMinPulse(&driver->base);}
/** \see PwmHl_getMode() */
IFX_INLINE Pwm_Mode IfxGtmAtom_PwmHl_getMode(IfxGtmAtom_PwmHl* driver)
{ return driver->base.functions.getMode(&driver->base);}
/** \see PwmHl_setDeadtime() */
IFX_INLINE boolean IfxGtmAtom_PwmHl_setDeadtime(IfxGtmAtom_PwmHl* driver, float32 deadtime)
{ return driver->base.functions.setDeadtime(&driver->base, deadtime);}
/** \see PwmHl_setMinPulse() */
IFX_INLINE boolean IfxGtmAtom_PwmHl_setMinPulse(IfxGtmAtom_PwmHl* driver, float32 minPulse)
{ return driver->base.functions.setMinPulse(&driver->base, minPulse);}
IFX_EXTERN boolean IfxGtmAtom_PwmHl_setMode(IfxGtmAtom_PwmHl* driver, Pwm_Mode mode);
/** \see PwmHl_setOnTime() */
IFX_INLINE void IfxGtmAtom_PwmHl_setOnTime(IfxGtmAtom_PwmHl* driver, TimerVal* tOn)
{
    driver->base.functions.setOnTime(&driver->base, tOn);
}
/** \} */

//________________________________________________________________________________________
//
#endif /* IFXGTMATOM_PWMHL_H */
/* *INDENT-ON* Note: this file was indented manually by the author. */
