/**
 * \file IfxGtmAtom_Timer.c
 * \brief GTM ATOM timer and trigger functions
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1479.6
 * $Date: $2013-08-09 16:42:24 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 */

/* *INDENT-OFF* Note: this file was indented manually by the author. */

//________________________________________________________________________________________
// INCLUDES

#include "IfxGtmAtom_Timer.h"
#include "_Utilities/Ifx_Assert.h"
#include "_Reg/IfxGtm_bf.h"
#include "stddef.h"

//________________________________________________________________________________________
// LOCAL FUNCTION PROTOTYPES

//________________________________________________________________________________________
// FUNCTION IMPLEMENTATIONS
float32     IfxGtmAtom_Timer_getInputFrequency(IfxGtmAtom_Timer* driver)
{
    /* TODO */
    IFX_ASSERT(VERBOSE_LEVEL_ERROR, IFX_ASSERT_FEATURE_NOT_IMPLEMENTED);
    return 0.0;
}

boolean IfxGtmAtom_Timer_acknowledgeTimerIrq(IfxGtmAtom_Timer* driver)
{
    boolean event;
    event = IfxGtmAtom_Ch_isZeroNotification(driver->atom, driver->timerChannel);
    if (event)
    {
        IfxGtmAtom_Ch_clearZeroNotification(driver->atom, driver->timerChannel);
    } else {}
    return event;
}

boolean IfxGtmAtom_Timer_acknowledgeTriggerIrq(IfxGtmAtom_Timer* driver)
{
    boolean event;
    event = IfxGtmAtom_Ch_isOneNotification(driver->atom, driver->triggerChannel);
    if (event)
    {
        IfxGtmAtom_Ch_clearOneNotification(driver->atom, driver->triggerChannel);
    } else {}
    return event;
}

/** Add a channel to the channel mask
 *
 * Channels present in the mask are started, stopped, updated at the same time as the timer:
 * IfxGtmAtom_Timer_applyUpdate, IfxGtmAtom_Timer_disableUpdate, IfxGtmAtom_Timer_stop, IfxGtmAtom_Timer_run
 *
 */
void IfxGtmAtom_Timer_addToChannelMask(IfxGtmAtom_Timer* driver, uint16 mask)
{
    driver->channelsMask |= mask;
}

/** Enable the transfer of the shadow registers
 *
 *  \note: The disableUpdate function shall be called previously to avoid not intended early transfer.
 *
 */
void IfxGtmAtom_Timer_applyUpdate(IfxGtmAtom_Timer* driver)
{
    IfxGtmAtom_Agc_enableChannelsUpdate(driver->agc, driver->channelsMask, 0);
}

/** Disable the transfer of the shadow registers
 *
 */
void IfxGtmAtom_Timer_disableUpdate(IfxGtmAtom_Timer* driver)
{
    IfxGtmAtom_Agc_enableChannelsUpdate(driver->agc, 0, driver->channelsMask);
}

const Timer_Funcs IfxGtmAtom_Timer_Funcs =
{
    .getFrequency         = &Timer_getFrequency,
    .getPeriod            = &Timer_getPeriod,
    .getResolution        = &Timer_getResolution,
    .setFrequency         = &Timer_setFrequency,
    .sToTick              = &Timer_sToTick,
    .tickToS              = &Timer_tickToS,
    .updateInputFrequency = (Timer_UpdateInputFrequency)&IfxGtmAtom_Timer_updateInputFrequency,
    .applyUpdate          = (Timer_ApplyUpdate)&IfxGtmAtom_Timer_applyUpdate,
    .disableUpdate        = (Timer_DisableUpdate)&IfxGtmAtom_Timer_disableUpdate,
    .setPeriod            = (Timer_SetPeriod)&IfxGtmAtom_Timer_setPeriod,
    .setSingleMode        = (Timer_SetSingleMode)&IfxGtmAtom_Timer_setSingleMode,
    .setTrigger           = (Timer_SetTrigger)&IfxGtmAtom_Timer_setTrigger,
    .stop                 = (Timer_Stop)&IfxGtmAtom_Timer_stop,
    .run                  = (Timer_Run)&IfxGtmAtom_Timer_run,
    .getInputFrequency    = (Timer_GetInputFrequency)&IfxGtmAtom_Timer_getInputFrequency,
    .ackTimerIrq          = (Timer_AckTimerIrq)         &IfxGtmAtom_Timer_acknowledgeTimerIrq,
    .ackTriggerIrq        = (Timer_AckTriggerIrq)       &IfxGtmAtom_Timer_acknowledgeTriggerIrq
};

/** Initialize the timer object
 *
 */
boolean IfxGtmAtom_Timer_init(IfxGtmAtom_Timer* driver, const IfxGtmAtom_Timer_Config* config)
{
    boolean result = TRUE;
    Timer* base = &driver->base;

    /* driver.base must be at offset 0 to be compatible with the standard interface Timer */
    IFX_ASSERT(VERBOSE_LEVEL_ERROR, offsetof(IfxGtmAtom_Timer, base) == 0);

    if (config->timerChannel != IfxGtmAtom_Ch_none)
    {
        base->driver = driver;
        base->functions = IfxGtmAtom_Timer_Funcs;
        IFX_ASSERT(VERBOSE_LEVEL_ERROR, config->base.countDir == Timer_CountDir_up); /* only this mode is supported */

        driver->gtm = config->gtm;
        driver->atomIndex = config->atom;
        driver->atom = &config->gtm->ATOM[config->atom];
        driver->timerChannel = config->timerChannel;
        driver->triggerChannel = config->triggerOut->channel;

	driver->agc  = (Ifx_GTM_ATOM_AGC *)&driver->atom->AGC_GLB_CTRL;

        /* Initialize the timer part */
        IfxGtmAtom_Ch_configurePwmMode(driver->atom, driver->timerChannel, config->clock,
                config->base.trigger.risingEdgeAtPeriod, IfxGtmAtom_Ch_ResetEvent_onCm0,
                IfxGtmAtom_Ch_OutputTrigger_generate);

        IfxGtmAtom_Timer_updateInputFrequency(driver);
        if ((config->base.minResolution > 0) && ((1.0 / base->clockFreq) > config->base.minResolution))
        {
            result = FALSE;
            IFX_ASSERT(VERBOSE_LEVEL_ERROR, FALSE);
        }
        else {}
        Timer_setFrequency(&driver->base, config->base.frequency);
    }

    /* Initialize the trigger part */
    driver->channelsMask = 1 << driver->timerChannel;
    base->triggerEnabled = config->base.trigger.enabled;
    if (base->triggerEnabled)
    {
        IfxGtmAtom_Ch triggerChannel = driver->triggerChannel;
        uint16 triggerChannelMask = 1 << triggerChannel;

        if (triggerChannel != driver->timerChannel)
        {
            IfxGtmAtom_Ch_configurePwmMode(driver->atom, triggerChannel, config->clock,
                    config->base.trigger.risingEdgeAtPeriod, IfxGtmAtom_Ch_ResetEvent_onTrigger,
                    IfxGtmAtom_Ch_OutputTrigger_forward);
            IfxGtmAtom_Agc_enableChannels(driver->agc, triggerChannelMask, 0, FALSE);
            driver->channelsMask |= triggerChannelMask;
        }
        if (config->base.trigger.outputEnabled)
        {
            IfxGtmAtom_Agc_enableChannelsOutput(driver->agc, triggerChannelMask, 0, FALSE);

            /* Initialize the port */
            IfxGtm_PinMap_setAtomTout(config->triggerOut, config->base.trigger.outputMode, config->base.trigger.outputDriver);
        }
        else
        {
            IfxGtmAtom_Agc_enableChannelsOutput(driver->agc, 0, triggerChannelMask, FALSE);
        }
        Timer_setTrigger(base, config->base.trigger.dutyCycle * base->period);

    }
    else {}

    /* Interrupt configuration */
    {
        volatile Ifx_SRC_SRCR* src;
        boolean timerHasIrq = config->base.isrPriority > 0;
        boolean triggerHasIrq = config->base.trigger.isrPriority > 0;
        if (driver->triggerChannel == driver->timerChannel)
        {
            if (config->timerChannel != IfxGtmAtom_Ch_none)
            {
                IfxGtmAtom_Ch_setNotification(driver->atom, driver->timerChannel, IfxGtm_IrqMode_level, timerHasIrq, triggerHasIrq);
                src = IfxGtmAtom_Ch_getSrcPointer(driver->gtm, config->atom, driver->timerChannel);
                IfxSrc_init(src, config->base.isrProvider, config->base.isrPriority);
                IfxSrc_enable(src);
            }
        }
        else
        {
            IfxGtm_IrqMode irqMode = IfxGtm_IrqMode_pulseNotify;
            if ((timerHasIrq) && (config->timerChannel != IfxGtmAtom_Ch_none))
            {
                IfxGtmAtom_Ch_setNotification(driver->atom, driver->timerChannel, irqMode, TRUE, FALSE);
                src = IfxGtmAtom_Ch_getSrcPointer(driver->gtm, config->atom, driver->timerChannel);
                IfxSrc_init(src, config->base.isrProvider, config->base.isrPriority);
                IfxSrc_enable(src);
            }
            if (triggerHasIrq)
            {
                IfxGtmAtom_Ch_setNotification(driver->atom, driver->triggerChannel, irqMode, FALSE, TRUE);
                src = IfxGtmAtom_Ch_getSrcPointer(driver->gtm, config->atom, driver->triggerChannel);
                IfxSrc_init(src, config->base.trigger.isrProvider, config->base.trigger.isrPriority);
                IfxSrc_enable(src);
            }
        }
    }

    /* Transfer the shadow registers */
    IfxGtmAtom_Agc_setChannelsForceUpdate(driver->agc, driver->channelsMask, 0, 0, 0);
    IfxGtmAtom_Agc_trigger(driver->agc);
    IfxGtmAtom_Agc_setChannelsForceUpdate(driver->agc, 0, driver->channelsMask, 0, 0);
    return result;
}

/** Initialize the configuration structure to default
 *
 * \param config Timer configuration. This parameter is Initialised by the function
 * \param gtm GTM module used
 *
 */
void IfxGtmAtom_Timer_initConfig(IfxGtmAtom_Timer_Config* config, Ifx_GTM* gtm)
{
    Timer_initConfig(&config->base);
    config->gtm           = gtm;
    config->atom          = IfxGtmAtom_0;
    config->timerChannel  = IfxGtmAtom_Ch_0;
    config->triggerOut    = &IfxGtm_ATOM0_0_TOUT0_P02_0;
    config->clock         = IfxGtmCmu_Clk_0;
    config->base.countDir = Timer_CountDir_up; /* Ensure count up is selected, as it is the only supported mode */
}

/** Start the timer
 *
 */
void IfxGtmAtom_Timer_run(IfxGtmAtom_Timer* driver)
{
    IfxGtmAtom_Agc_enableChannels(driver->agc, driver->channelsMask, 0, TRUE);
}

/** Start  the timer and stop at the period end
 *
 */
void IfxGtmAtom_Timer_setSingleMode(IfxGtmAtom_Timer* driver, boolean enabled)
{
    IfxGtmAtom_Ch_setOneShotMode(driver->atom, driver->timerChannel, enabled);
}

/** Set the period value
 *
 * \param period Period value in ticks
 *
 */
boolean IfxGtmAtom_Timer_setPeriod(IfxGtmAtom_Timer* driver, TimerVal period)
{
    driver->base.period = period;
    IfxGtmAtom_Ch_setCompareZeroShadow(driver->atom, driver->timerChannel, period);
    if (driver->triggerChannel != driver->timerChannel)
    {
        IfxGtmAtom_Ch_setCompareZeroShadow(driver->atom, driver->triggerChannel, period);
    }

    return TRUE;
}

/** Set the trigger point
 *
 * \param triggerPoint Trigger point value in ticks
 *
 */
void IfxGtmAtom_Timer_setTrigger(IfxGtmAtom_Timer* driver, TimerVal triggerPoint)
{
    IfxGtmAtom_Ch_setCompareOneShadow(driver->atom, driver->triggerChannel, triggerPoint + 1);
}

/** Stop the timer
 *
 */
void IfxGtmAtom_Timer_stop(IfxGtmAtom_Timer* driver)
{
    IfxGtmAtom_Agc_enableChannels(driver->agc, 0, driver->channelsMask, TRUE);
}

/** Update the cached input clock frequency value
 *
 */
void IfxGtmAtom_Timer_updateInputFrequency(IfxGtmAtom_Timer* driver)
{
    driver->base.clockFreq = IfxGtmAtom_Ch_getClockFrequency(driver->gtm, driver->atom, driver->timerChannel);
}

volatile uint32* IfxGtmAtom_Timer_getPointer(IfxGtmAtom_Timer* driver)
{
  return (volatile uint32*)((uint32)&(driver->atom->CH0_CN0.U) + driver->timerChannel*0x80);
}
