/**
 * \file IfxGtmAtom_Timer.h
 * \brief GTM ATOM timer and trigger functions
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1479.5
 * $Date: $2013-08-09 16:42:24 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * \defgroup IfxLld_Gtm_atom_timer ATOM timer standard interface
 * \ingroup IfxLld_Gtm_interfaces
 *
 * This driver implements the timer functionalities as defined by \ref library_srvsw_if_timer.
 * The user is free to use either the driver specific APIs below or to used the \ref library_srvsw_if_timer "standard interface APIs".
 *
 * \section specific Specific implementation
 * Implementation is similar to \ref IfxLld_Gtm_tom_timer.
 *
 * \todo Add documentation
 *
 * For a detailed configuration of the microcontroller, see \ref IfxGtmAtom_Timer_init().
 *
 * \section example: Usage example
 * Initialisation:
 * \code
 * IfxGtmAtom_Timer_Config driverConfig;
 * IfxGtmAtom_Timer driverData;
 * IfxGtmAtom_Timer_initConfig(&driverConfig, &MODULE_GTM);
 * IfxGtmAtom_Timer_init (&driverData, &driverConfig);
 * \endcode
 *
 * During run-time, \ref library_srvsw_if_timer "the interface functions" should be used:
 * \code
 * Timer *handle = &driverData.base;
 * Timer_run(handle);
 * Timer_setPeriod(handle, period);
 * Timer_applyUpdate(handle);
 * \endcode
 */

/* *INDENT-OFF* Note: this file was indented manually by the author. */

#ifndef IFXGTMATOM_TIMER_H
#define IFXGTMATOM_TIMER_H
//________________________________________________________________________________________
// INCLUDES

#include "_PinMap/IfxGtm_PinMap.h"
#include "Gtm/Std/IfxGtmAtom.h"
#include "Gtm/Std/IfxGtmCmu.h"
#include "If/Timer.h"
//________________________________________________________________________________________
// DATA STRUCTURES

typedef struct
{
    Timer               base;
    Ifx_GTM*            gtm;
    Ifx_GTM_ATOM*       atom;
    Ifx_GTM_ATOM_AGC*   agc;
    IfxGtmAtom          atomIndex;
    IfxGtmAtom_Ch       timerChannel;   /**< \brief ATOM channel used for the timer */
    IfxGtmAtom_Ch       triggerChannel; /**< \brief TOM channel used for the trigger, if identical to the timerChannel, the trigger interrupt is having the same interrupt level as  the timer interrupt */
    uint16              channelsMask;   /**< \brief Mask for channels to be modified together */
} IfxGtmAtom_Timer;

typedef struct
{
    Timer_Config        base;
    Ifx_GTM*            gtm;
    IfxGtmAtom          atom;
    IfxGtmAtom_Ch       timerChannel;   /**< \brief ATOM channel used for the timer */
    IfxGtmAtom_ToutMap* triggerOut;     /**< \brief ATOM channel used for the trigger output, can be identical to the timer channel */
    IfxGtmCmu_Clk       clock;          /**< \brief Timer input clock */
} IfxGtmAtom_Timer_Config;

//________________________________________________________________________________________
// FUNCTION PROTOTYPES

IFX_EXTERN void    IfxGtmAtom_Timer_addToChannelMask(IfxGtmAtom_Timer* driver, uint16 mask);
/**
 * \addtogroup  IfxLld_Gtm_atom_timer
 * \{ */

/** \brief Return a pointer to the standard timer interface
 * \param driver Specifies the driver
 * \return a pointer to the standard timer interface
 */
IFX_INLINE Timer* IfxGtmAtom_Timer_getInterface(IfxGtmAtom_Timer* driver)
{
    return &driver->base;
}

IFX_EXTERN void    IfxGtmAtom_Timer_applyUpdate     (IfxGtmAtom_Timer* driver);
IFX_EXTERN void    IfxGtmAtom_Timer_disableUpdate   (IfxGtmAtom_Timer* driver);
IFX_EXTERN boolean IfxGtmAtom_Timer_init            (IfxGtmAtom_Timer* driver, const IfxGtmAtom_Timer_Config* config);
IFX_EXTERN void    IfxGtmAtom_Timer_initConfig      (IfxGtmAtom_Timer_Config* config, Ifx_GTM* gtm);
IFX_EXTERN void    IfxGtmAtom_Timer_run             (IfxGtmAtom_Timer* driver);
IFX_EXTERN void    IfxGtmAtom_Timer_setSingleMode   (IfxGtmAtom_Timer* driver, boolean enabled);
IFX_EXTERN boolean IfxGtmAtom_Timer_setPeriod       (IfxGtmAtom_Timer* driver, TimerVal period);
IFX_EXTERN void    IfxGtmAtom_Timer_setTrigger      (IfxGtmAtom_Timer* driver, TimerVal triggerPoint);
IFX_EXTERN void    IfxGtmAtom_Timer_stop            (IfxGtmAtom_Timer* driver);
IFX_EXTERN void    IfxGtmAtom_Timer_updateInputFrequency (IfxGtmAtom_Timer* driver);
IFX_EXTERN boolean IfxGtmAtom_Timer_acknowledgeTimerIrq  (IfxGtmAtom_Timer* driver);
IFX_EXTERN boolean IfxGtmAtom_Timer_acknowledgeTriggerIrq(IfxGtmAtom_Timer* driver);

IFX_EXTERN volatile uint32* IfxGtmAtom_Timer_getPointer(IfxGtmAtom_Timer* driver);
/** \} */
//________________________________________________________________________________________
//

#endif /* IFXGTMATOM_TIMER_H */
/* *INDENT-ON* Note: this file was indented manually by the author. */
