/**
 * \file IfxGtmCmu.c
 * \brief GTM CMU basic functionality
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 *
 * $Revision: $1415.5
 * $Date: $2013-08-09 16:42:18 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 */

/******************************************************************************/
#include "IfxGtmCmu.h"
#include "IfxGtmDpll.h"
#include "Scu/Std/IfxScuCcu.h"
#include "Scu/Std/IfxScuWdt.h"
#include <math.h>
#include "IfxGtm_bf.h"
/******************************************************************************/
/*                           Functions                                        */
/******************************************************************************/
/* FIXME chek that CMU register are accessed 32 bitwise (See TS 3.0 27.8.8 CMU Configuration Register Description).
 * If not use different coding way */

/** Enable / Disable the configurable, fixed, and external clocks
 *
 * \param gtm pointer to the GTM module
 * \param clkMask Enable / Disable mask. Mask is from \ref IfxLld_Gtm_cfg_cmu_clock_enable_disable
 *
 * return None.
 */
void IfxGtmCmu_enableClocks(Ifx_GTM *gtm, uint32 clkMask)
{
    gtm->CMU.CLK_EN.U = clkMask;
}


/** Returns the GTM configurable clock frequency in Hz
 *
 * \param gtm pointer to the GTM module
 * \param clkIndex index of the configurable clock 0=CMU_CLK0, 1=CMU_CLK1, ...
 * \param assumeEnabled when TRUE, attempt to calculate the frequency as if the clock is enabled.
 *
 * \return Returns the GTM configurable clock frequency in Hz
 */
float32 IfxGtmCmu_getClkFrequency(Ifx_GTM *gtm, IfxGtmCmu_Clk clkIndex, boolean assumeEnabled)
{
    float32 frequency;

    if ((IfxGtmCmu_isClkClockEnabled(gtm, clkIndex) != FALSE) || (assumeEnabled != FALSE))
    {
        switch (clkIndex)
        {
        case IfxGtmCmu_Clk_0:
        case IfxGtmCmu_Clk_1:
        case IfxGtmCmu_Clk_2:
        case IfxGtmCmu_Clk_3:
        case IfxGtmCmu_Clk_4:
        case IfxGtmCmu_Clk_5:
            frequency = IfxGtmCmu_getGclkFrequency(gtm) / (gtm->CMU.CLK0_5[clkIndex].CTRL.B.CLK_CNT + 1);
            break;
        case IfxGtmCmu_Clk_6:

            if (gtm->CMU.CLK_6.CTRL.B.CLK6_SEL == 0)
            {
                frequency = IfxGtmCmu_getGclkFrequency(gtm) / (gtm->CMU.CLK_6.CTRL.B.CLK_CNT + 1);
            }
            else
            {
                frequency = IfxGtmDpll_getSubIncFrequency(gtm, IfxGtmDpll_SubInc_2);
            }

            break;
        case IfxGtmCmu_Clk_7:

            if (gtm->CMU.CLK_7.CTRL.B.CLK7_SEL == 0)
            {
                frequency = IfxGtmCmu_getGclkFrequency(gtm) / (gtm->CMU.CLK_7.CTRL.B.CLK_CNT + 1);
            }
            else
            {
                frequency = IfxGtmDpll_getSubIncFrequency(gtm, IfxGtmDpll_SubInc_1);
            }

            break;
        default:
            frequency = 0.0;
            break;
        }
    }
    else
    {
        frequency = 0.0;
    }

    return frequency;
}


/** Returns the GTM external clock frequency in Hz
 *
 * \param gtm pointer to the GTM module
 * \param clkIndex index of the external clock 0=CMU_ECLK0, 1=CMU_ECLK1, ...
 * \param assumeEnabled when TRUE, attempt to calculate the frequency as if the clock is enabled.
 *
 * \return Returns the GTM external clock frequency in Hz
 */
float32 IfxGtmCmu_getEclkFrequency(Ifx_GTM *gtm, IfxGtmCmu_Eclk clkIndex, boolean assumeEnabled)
{
    float32 frequency;

    if (IfxGtmCmu_isEclkClockEnabled(gtm, clkIndex) || (assumeEnabled != FALSE))
    {
        float32 Z          = gtm->CMU.ECLK[clkIndex].NUM.B.ECLK_NUM;
        float32 N          = gtm->CMU.ECLK[clkIndex].DEN.B.ECLK_DEN;
        float32 multiplier = N / Z / 2;
        frequency = IfxGtmCmu_getGclkFrequency(gtm) * multiplier;
    }
    else
    {
        frequency = 0.0;
    }

    return frequency;
}


/** Returns the GTM fixed clock frequency in Hz
 *
 * \param gtm pointer to the GTM module
 * \param clkIndex index of the fixed clock 0=CMU_FXCLK0, 1=CMU_FXCLK1, ...
 * \param assumeEnabled when TRUE, attempt to calculate the frequency as if the clock is enabled.
 *
 * \return Returns the GTM fixed clock frequency in Hz
 */
float32 IfxGtmCmu_getFxClkFrequency(Ifx_GTM *gtm, IfxGtmCmu_Fxclk clkIndex, boolean assumeEnabled)
{
    float32 frequency;

    if (IfxGtmCmu_isFxClockEnabled(gtm) || (assumeEnabled != FALSE))
    {
        if (gtm->CMU.FXCLK.CTRL.B.FXCLK_SEL == 0)
        {
            frequency = IfxGtmCmu_getGclkFrequency(gtm);
        }
        else
        {
            frequency = IfxGtmCmu_getClkFrequency(gtm, IfxGtmCmu_Clk_5, assumeEnabled);
        }

        switch (clkIndex)
        {
        case IfxGtmCmu_Fxclk_0:
            frequency = frequency / 1;
            break;
        case IfxGtmCmu_Fxclk_1:
            frequency = frequency / 16;
            break;
        case IfxGtmCmu_Fxclk_2:
            frequency = frequency / 256;
            break;
        case IfxGtmCmu_Fxclk_3:
            frequency = frequency / 4096;
            break;
        case IfxGtmCmu_Fxclk_4:
            frequency = frequency / 65536;
            break;
        default:
            frequency = 0.0;
            break;
        }
    }
    else
    {
        frequency = 0.0;
    }

    return frequency;
}


/** Returns the GTM global clock frequency in Hz
 *
 * \param gtm pointer to the GTM module
 *
 * \return Returns the GTM global clock frequency in Hz
 */
float32 IfxGtmCmu_getGclkFrequency(Ifx_GTM *gtm)
{
    float32 N          = gtm->CMU.GCLK_DEN.B.GCLK_DEN;
    float32 Z          = gtm->CMU.GCLK_NUM.B.GCLK_NUM;
    float32 multiplier = N / Z;

    return IfxGtmCmu_getModuleFrequency(gtm) * multiplier;
}


/** Returns the GTM module frequency in Hz
 *
 * \param gtm pointer to the GTM module
 *
 * \return Returns the GTM module frequency in Hz
 */
float32 IfxGtmCmu_getModuleFrequency(Ifx_GTM *gtm)
{
    return IfxScuCcu_getSpbFrequency();
}


/** Return the configurable clock enable status
 *
 * \param gtm pointer to the GTM module
 *
 * \retval TRUE The clock is enabled
 * \retval FALSE The clock is disabled
 */
boolean IfxGtmCmu_isClkClockEnabled(Ifx_GTM *gtm, IfxGtmCmu_Clk clkIndex)
{
    return ((gtm->CMU.CLK_EN.U >> (2 * clkIndex)) & IFX_GTM_CMU_CLK_EN_EN_CLK0_MSK) == 0x3;
}


/** Return the external clock enable status
 *
 * \param gtm pointer to the GTM module
 *
 * \retval TRUE The clock is enabled
 * \retval FALSE The clock is disabled
 */
boolean IfxGtmCmu_isEclkClockEnabled(Ifx_GTM *gtm, IfxGtmCmu_Eclk clkIndex)
{
    return ((gtm->CMU.CLK_EN.U >> (2 * clkIndex + 16)) & IFX_GTM_CMU_CLK_EN_EN_CLK0_MSK) == 0x3;
}


/** Return the fixed clock enable status
 *
 * \param gtm pointer to the GTM module
 *
 * \retval TRUE The clock is enabled
 * \retval FALSE The clock is disabled
 */
boolean IfxGtmCmu_isFxClockEnabled(Ifx_GTM *gtm)
{
    return gtm->CMU.CLK_EN.B.EN_FXCLK == 0x3;
}


/** Select the clock input for CLK6 and CLK7
 *
 * \note The frequency can only be modified when the corresponding clock is disabled using IfxGtmCmu_enableClocks()
 *
 * \param gtm pointer to the GTM module
 * \param clkIndex index of the configurable clock 0=CMU_CLK0, 1=CMU_CLK1, ...
 * \param useGlobal if TRUE, uses the global clock as an input, else use the SUB_INC input
 *
 * \return None
 */
void IfxGtmCmu_selectClkInput(Ifx_GTM *gtm, IfxGtmCmu_Clk clkIndex, boolean useGlobal)
{
    /* FIXME is endinit required? */
    switch (clkIndex)
    {
    case IfxGtmCmu_Clk_6:
        gtm->CMU.CLK_6.CTRL.B.CLK6_SEL = useGlobal ? 0 : 1;
        break;
    case IfxGtmCmu_Clk_7:
        gtm->CMU.CLK_7.CTRL.B.CLK7_SEL = useGlobal ? 0 : 1;
        break;
    default:
        break;
    }
}


/** Set the GTM configurable clock frequency in Hz
 *
 * \note The frequency can only be modified when the corresponding clock is disabled using IfxGtmCmu_enableClocks()
 *
 * \param gtm pointer to the GTM module
 * \param clkIndex index of the configurable clock 0=CMU_CLK0, 1=CMU_CLK1, ...
 * \param frequency frequency in Hz
 *
 * \return None
 */
void IfxGtmCmu_setClkFrequency(Ifx_GTM *gtm, IfxGtmCmu_Clk clkIndex, float32 frequency)
{
    float32 t   = (IfxGtmCmu_getGclkFrequency(gtm) / frequency) - 1;
    uint32  cnt = (uint32)t;

    if ((t - (float32)cnt) > 0.5)
    {                           /* Round to nearest */
        cnt++;
    }

    uint16 psw = IfxScuWdt_getCpuWatchdogPassword();
    IfxScuWdt_clearCpuEndinit(psw);     /* FIXME is this required ? */

    switch (clkIndex)
    {
    case IfxGtmCmu_Clk_0:
    case IfxGtmCmu_Clk_1:
    case IfxGtmCmu_Clk_2:
    case IfxGtmCmu_Clk_3:
    case IfxGtmCmu_Clk_4:
    case IfxGtmCmu_Clk_5:
        gtm->CMU.CLK0_5[clkIndex].CTRL.B.CLK_CNT = cnt;
        break;
    case IfxGtmCmu_Clk_6:
        gtm->CMU.CLK_6.CTRL.B.CLK_CNT = cnt;
        break;
    case IfxGtmCmu_Clk_7:
        gtm->CMU.CLK_7.CTRL.B.CLK_CNT = cnt;
        break;
    default:
        break;
    }

    IfxScuWdt_setCpuEndinit(psw);
}


/** Set the GTM external clock frequency in Hz
 *
 * \note The frequency can only be modified when the corresponding clock is disabled using IfxGtmCmu_enableClocks()
 *
 * \param gtm pointer to the GTM module
 * \param clkIndex index of the external clock 0=CMU_ECLK0, 1=CMU_ECLK1, ...
 * \param frequency frequency in Hz
 *
 * \return None
 */
void IfxGtmCmu_setEclkFrequency(Ifx_GTM *gtm, IfxGtmCmu_Eclk clkIndex, float32 frequency)
{
    float32 f;
    float32 bestDistance = frequency;
    float32 fIn          = IfxGtmCmu_getGclkFrequency(gtm) * 2;
    uint32  z, n, nBest = 1, zBest = 1;
    float32 t;

    for (z = 1; z < 0xFFFFFF; z++)
    {
        boolean endLoop = FALSE;
        t = fIn / z;

        for (n = z; n > 0; n--)
        {
            float32 distance;
            f        = t * n;
            distance = fabsf(frequency - f);

            if (distance < bestDistance)
            {
                bestDistance = distance;
                nBest        = n;
                zBest        = z;
            }

            if (bestDistance < 0.1)
            {
                endLoop = TRUE;
                break;
            }
        }

        if (endLoop)
        {
            break;
        }
    }

    uint16 psw = IfxScuWdt_getCpuWatchdogPassword();
    IfxScuWdt_clearCpuEndinit(psw);                 /* FIXME is this required ? */
    gtm->CMU.ECLK[clkIndex].NUM.B.ECLK_NUM = zBest;
    gtm->CMU.ECLK[clkIndex].NUM.B.ECLK_NUM = zBest; /* write twice to be sure */
    gtm->CMU.ECLK[clkIndex].DEN.B.ECLK_DEN = nBest;
    IfxScuWdt_setCpuEndinit(psw);
}


/** Set the GTM global clock frequency in Hz
 *
 * \note The frequency can only be modified when all clocks are disabled using IfxGtmCmu_enableClocks()
 * \param gtm pointer to the GTM module
 * \param frequency frequency in Hz
 *
 * \return None
 */
void IfxGtmCmu_setGclkFrequency(Ifx_GTM *gtm, float32 frequency) /* Kapitel 25.8.2 */
{
    float32 f;
    float32 bestDistance = frequency;
    float32 fIn          = IfxGtmCmu_getModuleFrequency(gtm);
    uint32  z, n, nBest = 1, zBest = 1;
    float32 t;

#if 1

    for (z = 1; z < 0xFFFFFF; z++)
    {
        boolean endLoop = FALSE;
        t = fIn / z;

        for (n = z; n > 0; n--)
        {
            float32 distance;
            f        = t * n;
            distance = fabsf(frequency - f);

            if (distance < bestDistance)
            {
                bestDistance = distance;
                nBest        = n;
                zBest        = z;
            }

            if (bestDistance < 0.1)
            {
                endLoop = TRUE;
                break;
            }
        }

        if (endLoop)
        {
            break;
        }
    }

#else

    for (n = 1; n < 0xFFFFFF; n++)
    {
        float32 distance;
        /* get best z */
        z = floorf(frequency * n / fIn);
        t = fIn / n;

        /* lower value */
        f        = t * z;
        distance = fabsf(frequency - f);

        if (distance < bestDistance)
        {
            bestDistance = distance;
            nBest        = n;
            zBest        = z;
        }

        /* upper value */
        f        = t * (z + 1);
        distance = fabsf(frequency - f);

        if (distance < bestDistance)
        {
            bestDistance = distance;
            nBest        = n;
            zBest        = z;
        }

        if (bestDistance == 0.0)
        {
            break;
        }
    }

#endif

    uint16 psw = IfxScuWdt_getCpuWatchdogPassword();
    IfxScuWdt_clearCpuEndinit(psw);         /* FIXME is this required ? */
    gtm->CMU.GCLK_NUM.B.GCLK_NUM = zBest;
    gtm->CMU.GCLK_NUM.B.GCLK_NUM = zBest;   /* write twice to be sure */
    gtm->CMU.GCLK_DEN.B.GCLK_DEN = nBest;
    IfxScuWdt_setCpuEndinit(psw);
}


/******************************************************************************/
