/**
 * \file IfxGtmTom_PwmHl.c
 * \brief GTM TOM dual top-bottom PWM channel functions
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1479.6
 * $Date: $2013-08-09 16:42:24 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 */

//------------------------------------------------------------------------------
#include "IfxGtmTom_PwmHl.h"
#include "_Utilities/Ifx_Assert.h"
#include "Gtm/Mapping/IfxGtm_Mapping.h"
#include "stddef.h"
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
IFX_INLINE ActiveState IfxGtmTom_PwmHl_invertActiveState(ActiveState activeState)
{
    return activeState == ActiveState_low ? ActiveState_high : ActiveState_low;
}


/** Update the x output duty cycle in edge aligned modes (left and right aligned)
 *
 * \return None.
 * \private
 */
static void IfxGtmTom_PwmHl_updateEdgeAligned(IfxGtmTom_PwmHl *driver, TimerVal *tOn)
{
    uint8    channelIndex;
    TimerVal period;
    TimerVal deadtime = driver->base.deadtime;

    period = driver->base.timer->period;

    for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
    {
        TimerVal x;             /* x=period*dutyCycle, x=OnTime+deadTime */
        TimerVal cm0, cm1;
        x = tOn[channelIndex];

        if (driver->base.inverted != FALSE)
        {
            x = period - x;
        }
        else
        {}

        if ((x < driver->base.minPulse) || (x <= deadtime))
        {                       /* For deadtime condition: avoid leading edge of top channel to occur after the trailing edge */
            x = 0;
        }
        else if (x > driver->base.maxPulse)
        {
            x = period;
        }
        else
        {}

        /* Special handling due to GTM issue */
        if (x == period)
        {                       /* 100% duty cycle */
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->ccxTemp[channelIndex],
                period + 1 /* No compare event */,
                2 /* 1st compare event (issue: expected to be 1) */ + deadtime);
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->coutxTemp[channelIndex],
                period + 2 /* No compare event, issues has been seen with +1 */,
                2 /* 1st compare event (issue: expected to be 1) */);
        }
        else if (x == 0)
        {
            cm0 = 1;
            cm1 = period + 2;
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->ccxTemp[channelIndex], cm0, cm1);
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->coutxTemp[channelIndex], cm0 + deadtime, cm1);
        }
        else
        {                       /* x% duty cycle */
            cm1 = 2;            // CM1, set to 2 due to a GTM issue. should be 1 according to spec
            cm0 = x;            // CM0, set to x+2 due to a GTM issue. should be x+1 according to spec
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->ccxTemp[channelIndex], cm0, cm1 + deadtime);
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->coutxTemp[channelIndex], cm0 + deadtime, cm1);
        }
    }
}


/** Update the x output duty cycle in center aligned and center aligned inverted modes
 *
 * \return None.
 * \private
 */
static void IfxGtmTom_PwmHl_updateCenterAligned(IfxGtmTom_PwmHl *driver, TimerVal *tOn)
{
    uint8    channelIndex;
    TimerVal period;
    TimerVal deadtime = driver->base.deadtime;

    period = driver->base.timer->period;

    for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
    {
        TimerVal x;             /* x=period*dutyCycle, x=OnTime+deadTime */
        TimerVal cm0, cm1;
        x = tOn[channelIndex];

        if (driver->base.inverted != FALSE)
        {
            x = period - x;
        }
        else
        {}

        if ((x < driver->base.minPulse) || (x <= deadtime))
        {                       /* For deadtime condition: avoid leading edge of top channel to occur after the trailing edge */
            x = 0;
        }
        else if (x > driver->base.maxPulse)
        {
            x = period;
        }
        else
        {}

        /* Special handling due to GTM issue */
        if (x == period)
        {                       /* 100% duty cycle */
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->ccxTemp[channelIndex],
                period + 1 /* No compare event */,
                2 /* 1st compare event (issue: expected to be 1) */ + deadtime);
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->coutxTemp[channelIndex],
                period + 2 /* No compare event, issues has been seen with +1 */,
                2 /* 1st compare event (issue: expected to be 1) */);
        }
        else if (x == 0)
        {
            cm0 = 1;
            cm1 = period + 2;
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->ccxTemp[channelIndex], cm0, cm1);
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->coutxTemp[channelIndex], cm0 + deadtime, cm1);
        }
        else
        {                           /* x% duty cycle */
            cm1 = (period - x) / 2; // CM1 /* FIXME issue if CM1 <= 1, should be limited for up to AB step at least */
            cm0 = (period + x) / 2; // CM0
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->ccxTemp[channelIndex], cm0, cm1 + deadtime);
            IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->coutxTemp[channelIndex], cm0 + deadtime, cm1);
        }
    }
}


/** Set the outputs to inactive
 *
 * \return None.
 */
static void IfxGtmTom_PwmHl_updateOff(IfxGtmTom_PwmHl *driver, TimerVal *tOn)
{
    uint8    channelIndex;
    TimerVal period;

    period = driver->base.timer->period;

    for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
    {
        IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->ccxTemp[channelIndex],
            2 /* 1 will keep the previous level */, period + 2);
        IfxGtmTom_Ch_setCompareShadow(driver->tom, driver->coutxTemp[channelIndex], period + 1, 2);
    }
}


static void IfxGtmTom_PwmHl_setupChannels(IfxGtmTom_PwmHl *driver, boolean *activeCh, boolean *stuckSt)
{
    /* FIXME TODO */
}


const IfxGtmTom_PwmHl_Mode IfxGtmTom_PwmHl_modes[5] = {
    {FALSE, IfxGtmTom_PwmHl_updateCenterAligned},   // Pwm_Mode_centerAligned
    {TRUE,  IfxGtmTom_PwmHl_updateCenterAligned},   // Pwm_Mode_centerAlignedInverted
    {FALSE, IfxGtmTom_PwmHl_updateEdgeAligned  },   // Pwm_Mode_leftAligned
    {TRUE,  IfxGtmTom_PwmHl_updateEdgeAligned  },   // Pwm_Mode_rightAligned
    {FALSE, IfxGtmTom_PwmHl_updateOff          } // Pwm_Mode_off
};

/** Initialize the timer object
 *
 * \note To ensure that the channels counter are reset by the timer and do not overflow, leading to random signal on the output, the timer must be started before the call to this function.
 * Note: all PWM channels must use the same TGC
 * Note: the current implementation must use the same TGC for the timer and PWM channels
 *
 */
boolean IfxGtmTom_PwmHl_init(IfxGtmTom_PwmHl *driver, const IfxGtmTom_PwmHl_Config *config)
{
    boolean          result       = TRUE;
    uint16           channelMask;
    uint16           channelsMask = 0;
    uint32           channelIndex;
    uint16           maskShift    = 0;
    IfxGtmTom_Timer *timer        = (IfxGtmTom_Timer *)config->base.timer;

    /* driver.base must be at offset 0 to be compatible with the standard interface PwmHl */
    IFX_ASSERT(VERBOSE_LEVEL_ERROR, offsetof(IfxGtmTom_PwmHl, base) == 0);

    driver->base.driver           = driver;
    driver->base.timer            = config->base.timer;
    driver->base.mode             = Pwm_Mode_init;
    driver->base.setMode          = 0;
    driver->base.ccxActiveState   = config->base.ccxActiveState;
    driver->base.coutxActiveState = config->base.coutxActiveState;
    driver->base.inverted         = FALSE;
    driver->base.channelCount     = config->base.channelCount;

    /* Link pwm channel functions */
    driver->base.functions.getDeadtime   = &PwmHl_getDeadtime;
    driver->base.functions.getMinPulse   = &PwmHl_getMinPulse;
    driver->base.functions.getMode       = &PwmHl_getMode;
    driver->base.functions.setDeadtime   = &PwmHl_setDeadtime;
    driver->base.functions.setMinPulse   = &PwmHl_setMinPulse;
    driver->base.functions.setMode       = (PwmHl_SetMode) & IfxGtmTom_PwmHl_setMode;
    driver->base.functions.setOnTime     = NULL_PTR; /* Set by setMode() below */
    driver->base.functions.setupChannels = (PwmHl_SetupChannels) & IfxGtmTom_PwmHl_setupChannels;

    /* wird spter bei der Update PWM Funktion mit beachtet */
    PwmHl_setDeadtime(&driver->base, config->base.deadtime);
    PwmHl_setMinPulse(&driver->base, config->base.minPulse);
    								/* TOM1 */
    driver->tom = &(timer->gtm->TOM[config->tom]);

    /* config->ccx[0] is used for the definition of the TGC */
    if (config->ccx[0]->channel <= 7)
    {
        driver->tgc = (Ifx_GTM_TOM_TGC *)&driver->tom->TGC0_GLB_CTRL;
    }
    else
    {
        driver->tgc = (Ifx_GTM_TOM_TGC *)&driver->tom->TGC1_GLB_CTRL;
    }

    maskShift = (config->ccx[0]->channel <= 7) ? 0 : 8;

    IFX_ASSERT(VERBOSE_LEVEL_ERROR, config->base.channelCount <= IFXGTMTOM_PWMHLCH_MAX_CHANNELS);

    IfxGtmTom_Ch_ClkSrc clock = IfxGtmTom_Ch_getClockSource(timer->tom, timer->timerChannel);

    for (channelIndex = 0; channelIndex < config->base.channelCount; channelIndex++)
    {
        IfxGtmTom_Ch channel;
        /* CCX high level pwm oberer teil B6 brcke*/
        channel                   = config->ccx[channelIndex]->channel;
        driver->ccx[channelIndex] = channel;
        channelMask               = 1 << (channel - maskShift);
        channelsMask             |= channelMask;

        /* Initialize the timer part */
        /* IfxGtmTom_Ch_ClkSrc_cmuFxclk0 ist clock source fr alles */
        IfxGtmTom_Ch_setClockSource(driver->tom, channel, clock);
        /* SL Bit wird gesetzt: High oder Low whrend ON Periode */
        IfxGtmTom_Ch_setSignalLevel(driver->tom, channel, driver->base.inverted
            ? config->base.ccxActiveState
            : IfxGtmTom_PwmHl_invertActiveState(config->base.ccxActiveState));
        /* reset Counter Register CN0 wenn Trigger_[x-1] aktiv */
        IfxGtmTom_Ch_setResetSource(driver->tom, channel, IfxGtmTom_Ch_ResetEvent_onTrigger);
        /* Trigger_x ist Trigger_x-1*/
        /* => alle trigger triggern auf channel 0 : timer channel */
        IfxGtmTom_Ch_setTriggerOutput(driver->tom, channel, IfxGtmTom_Ch_OutputTrigger_forward);

        /*Initialize the port output mode:Push-Pull output driver:speedgrade 2*/
        IfxGtm_PinMap_setTomTout(config->ccx[channelIndex],
                        config->base.outputMode, config->base.outputDriver);
        /* COUTX unterer Teil B6 brcke*/
        channel                     = config->coutx[channelIndex]->channel;
        driver->coutx[channelIndex] = channel;
        channelMask                 = 1 << (channel - maskShift);
        channelsMask               |= channelMask;

        /* Initialize the timer part */
        IfxGtmTom_Ch_setClockSource(driver->tom, channel, clock);
        IfxGtmTom_Ch_setSignalLevel(driver->tom, channel, driver->base.inverted
            ? IfxGtmTom_PwmHl_invertActiveState(config->base.coutxActiveState)
            : config->base.coutxActiveState);
        IfxGtmTom_Ch_setResetSource(driver->tom, channel, IfxGtmTom_Ch_ResetEvent_onTrigger);
        IfxGtmTom_Ch_setTriggerOutput(driver->tom, channel, IfxGtmTom_Ch_OutputTrigger_forward);

        /*Initialize the port */
        IfxGtm_PinMap_setTomTout(config->coutx[channelIndex],
                               config->base.outputMode, config->base.outputDriver);

    }
    /* enable channel output 1-6 */
    IfxGtmTom_Tgc_enableChannelsOutput(driver->tgc, channelsMask, 0, TRUE);
    /* enable channel update trigger */
    IfxGtmTom_Tgc_enableChannels(driver->tgc, channelsMask, 0, TRUE);

    /* open all Transisotrs */
    /* setzte Aktive States */
    IfxGtmTom_PwmHl_setMode(driver, Pwm_Mode_off);

    TimerVal tOn[IFXGTMTOM_PWMHLCH_MAX_CHANNELS] = {0};
    IfxGtmTom_PwmHl_updateOff(driver, tOn);     /* tOn do not need defined values; schaltet outputs ab indem Capture und compare register auf nuerreichbare werte gesetzt werden */
    											/* macht keinen Sinn geschieht schon in  IfxGtmTom_PwmHl_setMode(driver, Pwm_Mode_off); */
    /* Transfer the shadow registers */
    IfxGtmTom_Tgc_setChannelsForceUpdate(driver->tgc, channelsMask, 0, 0, 0);  /* enable force update */
    IfxGtmTom_Tgc_trigger(driver->tgc); /* set trigger request for ENDIS_STAT and OUTEN_STAT */
    									/* => d.h. counter fngt an zu zhlen ? */
    IfxGtmTom_Tgc_setChannelsForceUpdate(driver->tgc, 0, channelsMask, 0, 0);  /* disable force update */

    /* Enable timer to update the channels */
    maskShift = (config->ccx[0]->channel <= 7) ? 0 : 8;

    for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
    {
        IfxGtmTom_Timer_addToChannelMask(timer, 1 << (driver->ccx[channelIndex] - maskShift));
        IfxGtmTom_Timer_addToChannelMask(timer, 1 << (driver->coutx[channelIndex] - maskShift));
    }

    return result;
}


/** Initialize the configuration structure to default
 *
 * \param config Channel configuration. This parameter is Initialised by the function
 *
 */
void IfxGtmTom_PwmHl_initConfig(IfxGtmTom_PwmHl_Config *config)
{
    PwmHl_initConfig(&config->base);
    config->tom   = IfxGtmTom_0;
    config->ccx   = NULL_PTR;
    config->coutx = NULL_PTR;
}


/** Set the PWM mode, the mode is only applied after setOnTime() + applyUpdate()
 *
 */
boolean IfxGtmTom_PwmHl_setMode(IfxGtmTom_PwmHl *driver, Pwm_Mode mode)
{
    boolean result = TRUE;
    PwmHl  *base   = &driver->base;

    if (base->mode != mode)
    {
        if (mode > Pwm_Mode_off)
        {
            mode   = Pwm_Mode_off;
            result = FALSE;
        }

        base->mode                = mode;
        base->functions.setOnTime = (PwmHl_SetOnTime)IfxGtmTom_PwmHl_modes[mode].update; /* setzt funktions pointer auf die funktion die die entsprechenden */
        																				 /* compare register setzt je nachdem welche PWM art gewhlt wird */

        if (base->mode != Pwm_Mode_off)
        {
            base->inverted = IfxGtmTom_PwmHl_modes[mode].inverted;
        }
        else
        {                       /* Keep previous inverted for off mode */
        }

        if (base->inverted)
        {
            driver->ccxTemp   = driver->coutx;
            driver->coutxTemp = driver->ccx;
        }
        else
        {
            driver->ccxTemp   = driver->ccx;
            driver->coutxTemp = driver->coutx;
        }

        {                       /* Workaround to enable the signal inversion required for center aligned inverted and right aligned modes */
            /* Note that changing signal level may produce short circuit at the power stage, in which case the inverter must be disable during this action. */
            /* Pwm_Mode_centerAligned and Pwm_Mode_LeftAligned use inverted=FALSE */
            /* Pwm_Mode_centerAlignedInverted and Pwm_Mode_RightAligned use inverted=TRUE */
            uint32 channelIndex;

            for (channelIndex = 0; channelIndex < driver->base.channelCount; channelIndex++)
            {
                IfxGtmTom_Ch channel;

                channel = driver->ccx[channelIndex];
                IfxGtmTom_Ch_setSignalLevel(driver->tom, channel, base->inverted
                    ? base->ccxActiveState
                    : IfxGtmTom_PwmHl_invertActiveState(base->ccxActiveState));

                channel = driver->coutx[channelIndex];
                IfxGtmTom_Ch_setSignalLevel(driver->tom, channel, base->inverted
                    ? IfxGtmTom_PwmHl_invertActiveState(base->coutxActiveState)
                    : base->coutxActiveState);
            }
        }
    }

    return result;
}
