/**
 * \file IfxGtmTom_PwmHl.h
 * \brief GTM TOM dual top-bottom PWM channel functions
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1479.5
 * $Date: $2013-08-09 16:42:25 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * \defgroup IfxLld_Gtm_tom_pwmhl TOM PWM standard interface
 * \ingroup IfxLld_Gtm_interfaces
 *
 * This driver implements the PWM functionalities as defined by \ref library_srvsw_if_pwmhl.
 * The user is free to use either the driver specific APIs below or to used the \ref library_srvsw_if_pwmhl "standard interface APIs".
 *
 * \section specific Specific implementation
 * The PWM driver is a concatenation of a \ref IfxLld_Gtm_tom_timer "timer" with additional PWM generation cells.
 *
 * The timer is implemented as described in \ref IfxLld_Gtm_tom_timer, it generate an internal trigger
 * that is used to synchronize all the TOM channel used (shown in red in the pictures below).
 *
 * The picture below presents two configuration examples:
 * - On the left, the timer and trigger are generated using the same TOM channel CH0. The 3 dual-complementary
 *   PWMs, CCx and COUTx, are generated using 6 additional TOM channels (CH1 to Ch6).
 * - On the right, the timer and the trigger uses separate TOM channels to allow the triggering of the ADC
 *   conversions. The order of the CCx and COUTx has been allocated freely between CH9 up to CH14.
 *
 * \image html "IfxGtmTom_PwmHl-0.png" "PWM implementations on GTM TOM"
 *
 *
 * Each of the TOM channel used for the generation of the CCx and COUTx signals are configured as in the figure below:
 *
 * \image html "IfxGtmTom_PwmHl-1.png" "TOM channel configuration".
 *
 * The internal trigger (red line) is generated by the \ref IfxLld_Gtm_tom_timer "timer" on timer rest and used by the
 * TOM PWM channels to reset the counters CN0 and transfer the shadow values saved in SR0 and SR1 to the
 * CM0 and CM1 registers. When the counter CN0 reach the values of the CM0 or CM1 compare registers,
 * a rising or falling edge is generated on the output PWM signal. The signal active state configuration
 * define which of the CM0 or CM1 is generating the falling or rising edge.
 *
 * This trigger mechanism ensure that all the PWM channels, period and trigger are updated at the
 * exact same time, avoiding any possibility for glitch or incoherent signals.
 *
 * The clock use is the same as the timer clock.
 *
 * In order to ensure a coherent update of all registers, the internal trigger must be disable before updating
 * the timer, trigger and PWM duty cycles shadow values, and enabled once the update is done. The transfer will ocucrs at the next timer reset.
 *
 * \subsection features Features:
 * - This implementation allows from 1 up to 3 dual-complementary PWM channels.
 * - Resources used:
 *   - Additionally to the linked timer resources used, the following resources are used:
 *     - 2 TOM channels per PWM channels
 *   - All TOM channels used must be own by the same TOM and TGC as for the linked timer
 * - The TOM channel used for the timer must always have a lower index than the TOM channels
 *   used for the pwm.
 * - All TOM channels used (including the timer and the trigger) must be contiguous but the order of the TOM channels is free.
 *
 *
 * For a detailed configuration of the microcontroller, see \ref IfxGtmTom_PwmHl_init().
 *
 */

#ifndef IFXGTMTOM_PWMHLCH_H
#define IFXGTMTOM_PWMHLCH_H
//------------------------------------------------------------------------------
#include "If/PwmHl.h"
#include "Gtm/Tom/Timer/IfxGtmTom_Timer.h"
//------------------------------------------------------------------------------
/** Maximal number of channels handled by the driver. One channel has a top and bottom pwm output
 * Limited to 3 because the PWM channels used must be owned by the same TGC
 * \ingroup IfxLld_Gtm_tom_timer */
#define IFXGTMTOM_PWMHLCH_MAX_CHANNELS (3)

typedef struct IfxGtmTom_PwmHl_s IfxGtmTom_PwmHl;

/** PWM configuration
 *
 */
typedef struct
{
    boolean inverted;                                               /**< \brief Inverted configuration for the selected mode */
    void (*update)(IfxGtmTom_PwmHl *driver, TimerVal *tOn);         /**< \brief update call back function for the selected mode */
} IfxGtmTom_PwmHl_Mode;

/** GTM TOM PWM driver
 *
 */
struct IfxGtmTom_PwmHl_s
{
    PwmHl            base;                                              /**< \brief PWM channel standard interface */
    void             (*update)(IfxGtmTom_PwmHl *driver, TimerVal *tOn); /**< \brief Update function for actual selected mode */
    Ifx_GTM_TOM     *tom;                                               /**< \brief TOM unit used */
    Ifx_GTM_TOM_TGC *tgc;                                               /**< \brief TGC unit used */
    IfxGtmTom_Ch     ccx[IFXGTMTOM_PWMHLCH_MAX_CHANNELS];               /**< \brief TOM channels used for the CCCX outputs */
    IfxGtmTom_Ch     coutx[IFXGTMTOM_PWMHLCH_MAX_CHANNELS];             /**< \brief TOM channels used for the OUTX outputs */
    IfxGtmTom_Ch    *ccxTemp;                                           /**< \brief cached value */
    IfxGtmTom_Ch    *coutxTemp;                                         /**< \brief cached value */
};


/** GTM TOM PWM driver configuration
 *
 */
typedef struct
{
    PwmHl_Config        base;   /**< \brief PWM HL standard interface configuration */
    IfxGtmTom           tom;    /**< \brief TOM unit used */
    const IfxGtmTom_ToutMapP* ccx;    /**< \brief Pointer to an array of size base.channelCount/2 containing the channels used. All channels used for ccx and coutx must be adjacent to the channel used for the timer, order is not important. */
    const IfxGtmTom_ToutMapP* coutx;  /**< \brief Pointer to an array of size base.channelCount/2 containing the channels used. All channels used for ccx and coutx must be adjacent to the channel used for the timer, order is not important. */
} IfxGtmTom_PwmHl_Config;

/**
 * \addtogroup  IfxLld_Gtm_tom_pwmhl
 * \{
 */
/** \brief Return a pointer to the standard pwmhl interface
 * \param driver Specifies the driver
 * \return a pointer to the standard pwmhl interface
 */
IFX_INLINE PwmHl *IfxGtmTom_PwmHl_getInterface(IfxGtmTom_PwmHl *driver)
{
    return &driver->base;
}




IFX_EXTERN boolean IfxGtmTom_PwmHl_init(IfxGtmTom_PwmHl *driver, const IfxGtmTom_PwmHl_Config *config);
IFX_EXTERN void    IfxGtmTom_PwmHl_initConfig(IfxGtmTom_PwmHl_Config *config);
/** \see PwmHl_getDeadtime() */
IFX_INLINE float32 IfxGtmTom_PwmHl_getDeadtime(IfxGtmTom_PwmHl *driver)
{
    return driver->base.functions.getDeadtime(&driver->base);
}


/** \see PwmHl_getMinPulse() */
IFX_INLINE float32 IfxGtmTom_PwmHl_getMinPulse(IfxGtmTom_PwmHl *driver)
{
    return driver->base.functions.getMinPulse(&driver->base);
}


/** \see PwmHl_getMode() */
IFX_INLINE Pwm_Mode IfxGtmTom_PwmHl_getMode(IfxGtmTom_PwmHl *driver)
{
    return driver->base.functions.getMode(&driver->base);
}


/** \see PwmHl_setDeadtime() */
IFX_INLINE boolean IfxGtmTom_PwmHl_setDeadtime(IfxGtmTom_PwmHl *driver, float32 deadtime)
{
    return driver->base.functions.setDeadtime(&driver->base, deadtime);
}


/** \see PwmHl_setMinPulse() */
IFX_INLINE boolean IfxGtmTom_PwmHl_setMinPulse(IfxGtmTom_PwmHl *driver, float32 minPulse)
{
    return driver->base.functions.setMinPulse(&driver->base, minPulse);
}


IFX_EXTERN boolean IfxGtmTom_PwmHl_setMode(IfxGtmTom_PwmHl *driver, Pwm_Mode mode);
/** \see PwmHl_setOnTime() */
IFX_INLINE void IfxGtmTom_PwmHl_setOnTime(IfxGtmTom_PwmHl *driver, TimerVal *tOn)
{
    driver->base.functions.setOnTime(&driver->base, tOn);
}


/** \} */
//------------------------------------------------------------------------------

#endif /* IFXGTMTOM_PWMHLCH_H */
