/**
 * \file IfxGtmTom_Timer.c
 * \brief GTM TOM timer and trigger functions
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1479.6
 * $Date: $2013-08-09 16:42:25 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 */

//------------------------------------------------------------------------------
#include "IfxGtmTom_Timer.h"
#include "Gtm/Mapping/IfxGtm_Mapping.h"
#include "_Utilities/Ifx_Assert.h"
#include "IfxGtm_bf.h"
#include "stddef.h"
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
/** Acknowledge the timer interrupt if it occurred
 * If the interrupt occurred, clear the notification flag.
 *
 * \return return TRUE if the interrupt occurred.
 */
boolean IfxGtmTom_Timer_acknowledgeTimerIrq(IfxGtmTom_Timer *driver)
{
    boolean event;

    event = IfxGtmTom_Ch_isZeroNotification(driver->tom, driver->timerChannel);

    if (event)
    {
        IfxGtmTom_Ch_clearZeroNotification(driver->tom, driver->timerChannel);
    }
    else
    {}

    return event;
}


/** Acknowledge the trigger interrupt if it occurred
 * If the interrupt occurred, clear the notification flag.
 *
 * \return return TRUE if the interrupt occurred.
 */
boolean IfxGtmTom_Timer_acknowledgeTriggerIrq(IfxGtmTom_Timer *driver)
{
    boolean event;

    event = IfxGtmTom_Ch_isOneNotification(driver->tom, driver->triggerChannel);

    if (event)
    {
        IfxGtmTom_Ch_clearOneNotification(driver->tom, driver->triggerChannel);
    }
    else
    {}

    return event;
}


/** Add a channel to the channel mask
 *
 * Channels present in the mask are started, stopped, updated at the same time as the timer:
 * IfxGtmTom_Timer_applyUpdate, IfxGtmTom_Timer_disableUpdate, IfxGtmTom_Timer_stop, IfxGtmTom_Timer_run
 *
 */
void IfxGtmTom_Timer_addToChannelMask(IfxGtmTom_Timer *timer, uint16 mask)
{
    timer->channelsMask |= mask;
}


/** Enable the transfer of the shadow registers
 *
 *  \note: The disableUpdate function shall be called previously to avoid not intended early transfer.
 *
 */
//void IfxGtmTom_Timer_applyUpdate(IfxGtmTom_Timer *driver)
//{
//    IfxGtmTom_Tgc_enableChannelsUpdate(driver->tgc, driver->channelsMask, 0);
//}


/** Disable the transfer of the shadow registers
 *
 */
//void IfxGtmTom_Timer_disableUpdate(IfxGtmTom_Timer *driver){
//    IfxGtmTom_Tgc_enableChannelsUpdate(driver->tgc, 0, driver->channelsMask);
//}

//void IfxGtmTom_Timer_applyUpdate(IfxGtmTom_Timer *driver)
//{
//    IfxGtmTom_Tgc_enableChannelsUpdate(driver->tgc, driver->channelsMask, 0);
//}


const Timer_Funcs IfxGtmTom_Timer_Funcs = {
    .getFrequency         = &Timer_getFrequency,
    .getPeriod            = &Timer_getPeriod,
    .getResolution        = &Timer_getResolution,
    .setFrequency         = &Timer_setFrequency,
    .sToTick              = &Timer_sToTick,
    .tickToS              = &Timer_tickToS,
    .updateInputFrequency = (Timer_UpdateInputFrequency) & IfxGtmTom_Timer_updateInputFrequency,
    // .applyUpdate          = (Timer_ApplyUpdate) & IfxGtmTom_Timer_applyUpdate,
    /* .disableUpdate        = (Timer_DisableUpdate) & IfxGtmTom_Timer_disableUpdate, */
    .setPeriod            = (Timer_SetPeriod) & IfxGtmTom_Timer_setPeriod,
    .setSingleMode        = (Timer_SetSingleMode) & IfxGtmTom_Timer_setSingleMode,
    .setTrigger           = (Timer_SetTrigger) & IfxGtmTom_Timer_setTrigger,
    .stop                 = (Timer_Stop) & IfxGtmTom_Timer_stop,
    .run                  = (Timer_Run) & IfxGtmTom_Timer_run,
    .getInputFrequency    = (Timer_GetInputFrequency)NULL_PTR,                                  /* FIXME: TODO */
    .ackTimerIrq          = (Timer_AckTimerIrq) & IfxGtmTom_Timer_acknowledgeTimerIrq,
    .ackTriggerIrq        = (Timer_AckTriggerIrq) & IfxGtmTom_Timer_acknowledgeTriggerIrq
};

/** Initialize the timer object
 *
 */
boolean IfxGtmTom_Timer_init(IfxGtmTom_Timer *driver, IfxGtmTom_Timer_Config *config)
{
    boolean result = TRUE;
    Timer  *timer  = &driver->base;
    uint16  maskShift;
    IfxGtmTom_Ch trigChannel  = config->triggerChannel->channel;


    /* driver.base must be at offset 0 to be compatible with the standard interface Timer */
    IFX_ASSERT(VERBOSE_LEVEL_ERROR, offsetof(IfxGtmTom_Timer, base) == 0);

    driver->base.functions = IfxGtmTom_Timer_Funcs;
    driver->base.driver    = driver;
    IFX_ASSERT(VERBOSE_LEVEL_ERROR, config->base.countDir == Timer_CountDir_up);    /* only this mode is supported */

    driver->gtm            = config->gtm;
    driver->tomIndex       = config->tom;
    driver->tom            = &config->gtm->TOM[config->tom];
    driver->timerChannel   = config->timerChannel;
    driver->triggerChannel = trigChannel;

    if (config->timerChannel <= 7)
    {
        driver->tgc = (Ifx_GTM_TOM_TGC *)&driver->tom->TGC0_GLB_CTRL;
        /* TGC sind die control submodule fr den TOM */
    }
    else
    {
        driver->tgc = (Ifx_GTM_TOM_TGC *)&driver->tom->TGC1_GLB_CTRL;
    }

    /* Initialize the timer part */
    IfxGtmTom_Ch_setClockSource(driver->tom, driver->timerChannel, config->clock);

    IfxGtmTom_Timer_updateInputFrequency(driver); /* holt sich von irgendwo die Frequenz gar nicht so wichtig ?*/

    /* berprft ob parameter konsistent sind */
    if ((config->base.minResolution > 0) && ((1.0 / timer->clockFreq) > config->base.minResolution))
    {
        result = FALSE;
        IFX_ASSERT(VERBOSE_LEVEL_ERROR, FALSE);
    }
    else
    {}

    /* setzt SR0 fr tigger und Timer */
    Timer_setFrequency(timer, config->base.frequency); /* IfxGtmAtom_Ch_setCompareZeroShadow */


    maskShift            = (config->timerChannel <= 7) ? 0 : 8;
    driver->channelsMask = 1 << (driver->timerChannel - maskShift);
    IfxGtmTom_Ch_setTriggerOutput(driver->tom, driver->timerChannel, IfxGtmTom_Ch_OutputTrigger_generate); /* setzt Trigger output selection Flag in Register */
    																										/*TOM_CHx_CTRL to enable TRIG_CCU0*/
    /* Initialize the trigger part */
    driver->base.triggerEnabled = config->base.trigger.enabled;

    if (driver->base.triggerEnabled)
    {
        IfxGtmTom_Ch triggerChannel;
        triggerChannel = driver->triggerChannel;
        uint16       triggerChannelMask = 1 << (triggerChannel - maskShift);

        IfxGtmTom_Ch_setSignalLevel(driver->tom, triggerChannel, config->base.trigger.risingEdgeAtPeriod);

        if (triggerChannel != driver->timerChannel)
        {
            IfxGtmTom_Ch_setResetSource(driver->tom, triggerChannel, IfxGtmTom_Ch_ResetEvent_onTrigger);
            IfxGtmTom_Ch_setClockSource(driver->tom, triggerChannel, config->clock);
            IfxGtmTom_Ch_setTriggerOutput(driver->tom, triggerChannel, IfxGtmTom_Ch_OutputTrigger_forward);
            IfxGtmTom_Tgc_enableChannels(driver->tgc, triggerChannelMask, 0, FALSE);
            driver->channelsMask |= triggerChannelMask;
        }
        else
        {}

        if (config->base.trigger.outputEnabled)
        {
            IfxGtmTom_Tgc_enableChannelsOutput(driver->tgc, triggerChannelMask, 0, FALSE);
            IfxGtm_PinMap_setTomTout(config->triggerChannel,
                                    config->base.trigger.outputMode, config->base.trigger.outputDriver);
        }
        else
        {
            IfxGtmTom_Tgc_enableChannelsOutput(driver->tgc, 0, triggerChannelMask, FALSE);
        }

        timer->functions.setTrigger(timer, config->base.trigger.dutyCycle * timer->period);
    }
    else
    {}

    /*Initialize the port */
    /* FIXME to be done only for config->base.outputEnabled== TRUE and driver->base.triggerEnabled (see code above)  */
    result &= IfxGtm_Mapping_setPinModeOutput(driver->gtm,
        config->base.trigger.outputFeature,
        config->base.trigger.outputOption,
        config->base.trigger.outputMode, config->base.trigger.outputDriver);

    /* Interrupt configuration */
    if ((config->base.isrPriority > 0) || (config->base.trigger.isrPriority > 0))
    {
        volatile Ifx_SRC_SRCR *src;

        if (driver->triggerChannel == driver->timerChannel)
        {
            IfxGtmTom_Ch_setNotification(driver->tom, driver->timerChannel,  IfxGtm_IrqMode_pulseNotify,
                config->base.isrPriority > 0, config->base.trigger.isrPriority > 0);
            src = IfxGtmTom_Ch_getSrcPointer(driver->gtm, config->tom, driver->timerChannel);
            					/* core */				/*prio*/
            IfxSrc_init(src, config->base.isrProvider, config->base.isrPriority);
            IfxSrc_enable(src);
        }
        else
        {
            IfxGtm_IrqMode irqMode = IfxGtm_IrqMode_pulseNotify;

            if (config->base.isrPriority > 0)
            {
                IfxGtmTom_Ch_setNotification(driver->tom, driver->timerChannel, irqMode, TRUE, FALSE);
                src = IfxGtmTom_Ch_getSrcPointer(driver->gtm, config->tom, driver->timerChannel);
                IfxSrc_init(src, config->base.isrProvider, config->base.isrPriority);
                IfxSrc_enable(src);
            }

            if (config->base.trigger.isrPriority > 0)
            {
                IfxGtmTom_Ch_setNotification(driver->tom, driver->triggerChannel, irqMode, FALSE, TRUE);
                src = IfxGtmTom_Ch_getSrcPointer(driver->gtm, config->tom, driver->triggerChannel);
                IfxSrc_init(src, config->base.trigger.isrProvider, config->base.trigger.isrPriority);
                IfxSrc_enable(src);
            }
        }
    }

    /* Transfer the shadow registers */
    IfxGtmTom_Tgc_setChannelsForceUpdate(driver->tgc, driver->channelsMask, 0, 0, 0);
    IfxGtmTom_Tgc_trigger(driver->tgc);
    IfxGtmTom_Tgc_setChannelsForceUpdate(driver->tgc, 0, driver->channelsMask, 0, 0);

    return result;
}


/** Initialize the configuration structure to default
 *
 * \param config Timer configuration. This parameter is Initialised by the function
 * \param gtm GTM module used
 *
 */
void IfxGtmTom_Timer_initConfig(IfxGtmTom_Timer_Config *config, Ifx_GTM *gtm)
{
    Timer_initConfig(&config->base);
    config->gtm            = gtm;
    config->tom            = IfxGtmTom_0;
    config->timerChannel   = IfxGtmTom_Ch_0;
    config->triggerChannel = IfxGtmTom_Ch_0;
    config->clock          = IfxGtmTom_Ch_ClkSrc_cmuFxclk0;
    config->base.countDir  = Timer_CountDir_up;
}


/** Start the timer
 *
 */
void IfxGtmTom_Timer_run(IfxGtmTom_Timer *driver)
{
    IfxGtmTom_Tgc_enableChannels(driver->tgc, driver->channelsMask, 0, TRUE);
    /* driver->tgc : Base address fr TOM REgister */
}


/** Start  the timer and stop at the period end
 *
 */
void IfxGtmTom_Timer_setSingleMode(IfxGtmTom_Timer *driver, boolean enabled)
{
    IfxGtmTom_Ch_setOneShotMode(driver->tom, driver->timerChannel, enabled);
}


/** Set the period value
 *
 * \param period Period value in ticks
 *
 */
boolean IfxGtmTom_Timer_setPeriod(IfxGtmTom_Timer *driver, TimerVal period)
{
    driver->base.period = period;
    IfxGtmTom_Ch_setCompareZeroShadow(driver->tom, driver->timerChannel, period);

    if (driver->triggerChannel != driver->timerChannel)
    {
        IfxGtmTom_Ch_setCompareZeroShadow(driver->tom, driver->triggerChannel, period);
    }

    return TRUE;
}


/** Set the trigger point
 *
 * \param triggerPoint Trigger point value in ticks
 *
 */
void IfxGtmTom_Timer_setTrigger(IfxGtmTom_Timer *driver, TimerVal triggerPoint)
{
    IfxGtmTom_Ch_setCompareOneShadow(driver->tom, driver->triggerChannel, triggerPoint + 1);
}


/** Stop the timer
 *
 */
void IfxGtmTom_Timer_stop(IfxGtmTom_Timer *driver)
{
    IfxGtmTom_Tgc_enableChannels(driver->tgc, 0, driver->channelsMask, TRUE);
}


/** Update the cached input clock frequency value
 *
 */
void IfxGtmTom_Timer_updateInputFrequency(IfxGtmTom_Timer *driver)
{
    driver->base.clockFreq = IfxGtmTom_Ch_getClockFrequency(driver->gtm, driver->tom, driver->timerChannel);
}
