/**
 * \file IfxGtmTom_Timer.h
 * \brief GTM TOM timer stantard interface
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1479.5
 * $Date: $2013-08-09 16:42:25 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * \defgroup IfxLld_Gtm_tom_timer TOM timer standard interface
 * \ingroup IfxLld_Gtm_interfaces
 *
 * This driver implements the timer functionalities as defined by \ref library_srvsw_if_timer.
 * The user is free to use either the driver specific APIs below or to used the \ref library_srvsw_if_timer "standard interface APIs".
 *
 * \section specific Specific implementation
 * The timer fucntinality is implemented using either a single or 2 TOM channels. The dicision to use 1 or 2 channels is done
 * depending on the following requirements:
 * - relative position of the timer channel to the other channels used if any
 * - need to trigger an other module, for example ADC triggering.
 *
 * The figure below show how the TOM is \ref IfxGtmTom_Timer_Config "configured" when a single TOM channel is used for the timer and trigger functinalities.
 * The internal trigger signal (red line) is generated when the counter CN0 reach the CM0 compare register. The value of CM0 act as the period value. This internal
 * trigger is used to reset the CN0 counter and simultaneously transfer the shadow values for the period value (SR0->CM0)
 * and trigger edge (SR1->CM1). The internal trigger is used as a trigger input to the next TOM channel.
 * The trigger signal (output trigger) is generated by the CM0 and CM1 compare values. Depending on the trigger signal active
 * edge \ref IfxGtmTom_Timer_Config "configuiration", the CM0 will reset, and the CM1 will set the trigger signal, or vice versa.
 *
 * \image html "IfxGtmTom_Timer-0.png" "Timer using one signle TOM channel"
 *
 * In case 2 TOM channels are used for the timer and trigger functionalities,
 * the 1st channel (CHz in the figure below) is used for the generation of the period,
 * and the 2nd channel (CHz+n the below figure, the figure shows n=1 as example) is used for the trigger generation.
 *
 * The CHz TOM channel generates the internal trigger signal (red line) when the counter CN0 reach the CM0 compare register.
 * The value of CM0 of CHz act as the period value. This internal trigger is used to reset the  CN0 counter and simultaneously
 * transfer the shadow period value (SR0->CM0). The internal trigger is used as an input for the next TOM channel.
 *
 * The CHz+n TOM channel, uses the internal trigger signal to simultaneously resets the
 * counter CN0, and transfer the shadow values used for the generation of the trigger falling and rising edges (SR0->CM0 and SR1->CM1).
 * The internal trigger is used as a trigger input to the next TOM channel.
 * Depending on the trigger signal active edge \ref IfxGtmTom_Timer_Config "configuiration", the CM0 will reset, and the CM1 will set the trigger signal, or vice versa.
 * In orter to have a similar behaviour to the single channel implementation,
 * the CM0 is set to the same value as CHz CM0, and CM1 is used for the trigger edge.
 *
 * \image html "IfxGtmTom_Timer-1.png" "Timer using two different TOM channels for the timer and the trigger"
 *
 *
 * In order to ensure a coherent update of all registers, the internal trigger must be disable before updating
 * the timer and trigger shadow values, and enabled once the update is done. The transfer will ocucrs at the next timer reset.
 *
 *
 * - Resources used:
 *   - if the trigger channel is identical to the timer channel, only one TOM channels is used
 *   - if the trigger channel is different from the timer channel, 2 TOM channels are used
 *   - All channels used must be own by the same TOM and TOM TGC
 * - The timer counting direction is limited to \ref Timer_CountDir_up
 * - If the TOM trigger channel is not the same as the TOM timer channels
 *   - The TOM channel used for the trigger must have a lower index than the TOM channels
 *     used for the timer.
 *   - the TOM channels must be contiguous, unless specified by the driver using the timer driver.
 *
 * For a detailed configuration of the microcontroller, see \ref IfxGtmTom_Timer_init().
 *
 *
 */

#ifndef IFXGTMTOM_TIMER_H
#define IFXGTMTOM_TIMER_H
//------------------------------------------------------------------------------
#include "_PinMap/IfxGtm_PinMap.h"
#include "Gtm/Std/IfxGtmTom.h"
#include "Gtm/Std/IfxGtmCmu.h"
#include "If/Timer.h"
//------------------------------------------------------------------------------
typedef struct
{
    Timer            base;
    Ifx_GTM         *gtm;
    Ifx_GTM_TOM     *tom;
    Ifx_GTM_TOM_TGC *tgc;
    IfxGtmTom        tomIndex;
    IfxGtmTom_Ch     timerChannel;      /**< \brief TOM channel used for the timer */
    IfxGtmTom_Ch     triggerChannel;    /**< \brief TOM channel used for the trigger, can be identical to the timer channel */
    uint16           channelsMask;      /**< \brief Mask for channels to be modified together */
} IfxGtmTom_Timer;

typedef struct
{
    Timer_Config        base;
    Ifx_GTM            *gtm;
    IfxGtmTom           tom;
    IfxGtmTom_Ch        timerChannel;   /**< \brief TOM channel used for the timer */
    IfxGtmTom_ToutMap  *triggerChannel; /**< \brief TOM channel used for the trigger, if identical to the timerChannel, the trigger interrupt is having the same interrupt level as  the timer interrupt */
    IfxGtmTom_Ch_ClkSrc clock;          /**< \brief Timer input clock */
} IfxGtmTom_Timer_Config;

IFX_EXTERN void IfxGtmTom_Timer_addToChannelMask(IfxGtmTom_Timer *driver, uint16 mask);
/**
 * \addtogroup  IfxLld_Gtm_tom_timer
 * \{
 */
/** \brief Return a pointer to the standard timer interface
 * \param driver Specifies the driver
 * \return a pointer to the standard timer interface
 */
IFX_INLINE Timer *IfxGtmTom_Timer_getInterface(IfxGtmTom_Timer *driver)
{
    return &driver->base;
}


//IFX_EXTERN void    IfxGtmTom_Timer_applyUpdate(IfxGtmTom_Timer *driver);
IFX_EXTERN void    IfxGtmTom_Timer_disableUpdate(IfxGtmTom_Timer *driver);
IFX_EXTERN boolean IfxGtmTom_Timer_init(IfxGtmTom_Timer *driver, IfxGtmTom_Timer_Config *config);
IFX_EXTERN void    IfxGtmTom_Timer_initConfig(IfxGtmTom_Timer_Config *config, Ifx_GTM *gtm);
IFX_EXTERN void    IfxGtmTom_Timer_run(IfxGtmTom_Timer *driver);
IFX_EXTERN void    IfxGtmTom_Timer_setSingleMode(IfxGtmTom_Timer *driver, boolean enabled);
IFX_EXTERN boolean IfxGtmTom_Timer_setPeriod(IfxGtmTom_Timer *driver, TimerVal period);
IFX_EXTERN void    IfxGtmTom_Timer_setTrigger(IfxGtmTom_Timer *driver, TimerVal triggerPoint);
IFX_EXTERN void    IfxGtmTom_Timer_stop(IfxGtmTom_Timer *driver);
IFX_EXTERN void    IfxGtmTom_Timer_updateInputFrequency(IfxGtmTom_Timer *driver);

IFX_EXTERN boolean IfxGtmTom_Timer_acknowledgeTimerIrq(IfxGtmTom_Timer *driver);
IFX_EXTERN boolean IfxGtmTom_Timer_acknowledgeTriggerIrq(IfxGtmTom_Timer *driver);
/** \} */
//------------------------------------------------------------------------------
#endif /* IFXGTMTOM_TIMER_H */
