/**
 * \file IfxQspi_Spi.c
 * \brief QSPI SPI details
 *
 * \copyright Copyright (c) 2013 Infineon Technologies AG. All rights reserved.
 *
 * $Date: $2013-08-09 17:29:45 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 */

/*******************************************************************************
**                                 Includes                                   **
*******************************************************************************/
#include "IfxQspi_Spi.h"

/** \addtogroup IfxLld_Qspi_Spi_Channel
 * \{ */

/******************************************************************************/
/*                     Private Function prototypes                            */
/******************************************************************************/

/** \brief Gets current active channel
 * \param driver QSPI module handle
 * \return current active channel
 */
static IfxQspi_Spi_Channel *IfxQspi_Spi_activeChannel(IfxQspi_Spi *driver);
/** \} */

/** \addtogroup IfxLld_Qspi_Spi_Support
 * \{ */

/******************************************************************************/
/*                     Private Function prototypes                            */
/******************************************************************************/

/**
 * \return None
 */
static void IfxQspi_Spi_activateSlso(IfxQspi_Spi_Channel *chHandle);

/**
 * \return None
 */
static void IfxQspi_Spi_autoSlso(IfxQspi_Spi_Channel *chHandle);

/**
 * \return None
 */
static void IfxQspi_Spi_deactivateSlso(IfxQspi_Spi_Channel *chHandle);

/**
 */
static SpiIf_Status IfxQspi_Spi_lock(IfxQspi_Spi *driver);

/** \brief Reads data from the Rx FIFO
 * \param chHandle Channel handle
 * \return None
 */
static void IfxQspi_Spi_read(IfxQspi_Spi_Channel *chHandle);

/**
 * \return None
 */
static void IfxQspi_Spi_unlock(IfxQspi_Spi *driver);

/** \brief Writes data into the Tx FIFO
 * \param chHandle Channel handle
 * \return None
 */
static void IfxQspi_Spi_write(IfxQspi_Spi_Channel *chHandle);
/** \} */

/*******************************************************************************
**                   Function implementations                                 **
*******************************************************************************/
static void IfxQspi_Spi_activateSlso(IfxQspi_Spi_Channel *chHandle)
{
    IfxPort_State action = (chHandle->slsoActiveState == ActiveState_low) ? IfxPort_State_low : IfxPort_State_high;
    IfxPort_setPinState(chHandle->slso.port, chHandle->slso.pinIndex, action);
}


static IfxQspi_Spi_Channel *IfxQspi_Spi_activeChannel(IfxQspi_Spi *driver)
{
    return (IfxQspi_Spi_Channel *)driver->base.activeChannel;
}


static void IfxQspi_Spi_autoSlso(IfxQspi_Spi_Channel *chHandle)
{}


static void IfxQspi_Spi_deactivateSlso(IfxQspi_Spi_Channel *chHandle)
{
    IfxPort_State action = (chHandle->slsoActiveState == ActiveState_low) ? IfxPort_State_high : IfxPort_State_low;
    IfxPort_setPinState(chHandle->slso.port, chHandle->slso.pinIndex, action);
}


SpiIf_Status IfxQspi_Spi_exchange(IfxQspi_Spi_Channel *chHandle, const void *src, void *dest, SizeT count)
{
    IfxQspi_Spi *driver = (IfxQspi_Spi *)chHandle->base.driver;
    SpiIf_Status status = IfxQspi_Spi_lock(driver);

    if (status == SpiIf_Status_ok)
    {
        /* initiate transfer when resource is free */
        driver->base.activeChannel      = &chHandle->base;
        chHandle->base.flags.onTransfer = 1;
        chHandle->base.tx.data          = (void *)src;
        chHandle->base.tx.remaining     = count;
        chHandle->firstWrite            = TRUE;
        chHandle->base.rx.remaining     = count;
        chHandle->base.rx.data          = dest;

        if (chHandle->activateSlso != NULL_PTR)
        {
            chHandle->activateSlso(chHandle);
        }

        chHandle->base.txHandler(&chHandle->base);
    }

    return status;
}


SpiIf_Status IfxQspi_Spi_getStatus(IfxQspi_Spi_Channel *chHandle)
{
    SpiIf_Status status = SpiIf_Status_ok;

    if ((chHandle->base.flags.onTransfer != 0) || (chHandle->base.driver->sending != 0))
    {
        status = SpiIf_Status_busy;
    }

    return status;
}


SpiIf_Status IfxQspi_Spi_initChannel(IfxQspi_Spi_Channel *chHandle, const IfxQspi_Spi_ChannelConfig *chConfig)
{
    IfxQspi_Spi *driver  = chConfig->base.driver->driver;
    Ifx_QSPI    *qspiSFR = driver->qspi;

    if (IfxQspi_getMode(qspiSFR) == IfxQspi_Mode_slave)
    {
        chHandle->channelId = chConfig->sls.input.pin->select;
    }
    else
    {
        chHandle->channelId = chConfig->sls.output.pin->slsoNr;
    }

    chHandle->base.driver = chConfig->base.driver;

    {
        uint8 cs = chHandle->channelId % 8;
        qspiSFR->ECON[cs].U = IfxQspi_calculateExtendedConfigurationValue(qspiSFR, cs, &chConfig->base);
        chHandle->bacon.U   = IfxQspi_calculateBasicConfigurationValue(qspiSFR, chHandle->channelId, &chConfig->base.mode, chConfig->base.baudrate);

        {   /* QSPI chip-select line */
            uint32 mask = 1U << chHandle->channelId;
            uint32 oen  = mask << 16;
            uint32 aol  = ((chConfig->base.mode.csActiveLevel == ActiveState_low) ? 0 : 1);
            __ldmst(&qspiSFR->SSOC.U, (mask | (mask << 16)), (aol | oen));
        }
    }
    chHandle->base.baudrate = IfxQspi_calcRealBaudrate(qspiSFR, (chHandle->bacon.B.CS & 0x7U));

    if (IfxQspi_getMode(qspiSFR) != IfxQspi_Mode_slave)
    {
        /* chip-select output pin */
        IfxQspi_Slso_Out *slso = chConfig->sls.output.pin;
        chHandle->slso            = slso->pin;
        chHandle->slsoActiveState = chConfig->base.mode.csActiveLevel;

        if (chHandle->slso.port == NULL_PTR)
        {
            chHandle->activateSlso   = NULL_PTR;
            chHandle->deactivateSlso = NULL_PTR;
        }
        else
        {
            if (!chConfig->base.mode.autoCS)
            {
                chHandle->activateSlso   = &IfxQspi_Spi_activateSlso;
                chHandle->deactivateSlso = &IfxQspi_Spi_deactivateSlso;

                /** - Override the SLSO manually as general-purpose output */
                chHandle->deactivateSlso(chHandle);
                IfxPort_setPinModeOutput(slso->pin.port, slso->pin.pinIndex, chConfig->sls.output.mode, IfxPort_OutputIdx_general);
                IfxPort_setPinPadDriver(slso->pin.port, slso->pin.pinIndex, chConfig->sls.output.driver);
            }
            else
            {
                chHandle->activateSlso   = &IfxQspi_Spi_autoSlso;
                chHandle->deactivateSlso = &IfxQspi_Spi_autoSlso;

                IfxPort_setPinModeOutput(slso->pin.port, slso->pin.pinIndex, chConfig->sls.output.mode, slso->select);
                IfxPort_setPinPadDriver(slso->pin.port, slso->pin.pinIndex, chConfig->sls.output.driver);
            }
        }
    }
    else
    {
        /* chip-select input pin */
        chHandle->activateSlso   = NULL_PTR;
        chHandle->deactivateSlso = NULL_PTR;
        IfxQspi_Slsi_In *slsi = chConfig->sls.input.pin;
        IfxPort_setPinModeInput(slsi->pin.port, slsi->pin.pinIndex, chConfig->sls.input.mode);

        /* PISEL */
        qspiSFR->PISEL.B.SLSIS = chConfig->sls.input.pin->select;
    }

    chHandle->dataWidth      = chConfig->base.mode.dataWidth;
    chHandle->base.txHandler = (TxRxHandler) & IfxQspi_Spi_write;
    chHandle->base.rxHandler = (TxRxHandler) & IfxQspi_Spi_read;

    return SpiIf_Status_ok;
}


void IfxQspi_Spi_initChannelConfig(IfxQspi_Spi_ChannelConfig *chConfig, IfxQspi_Spi *driver)
{
    SpiIf_initChannelConfig(&chConfig->base, &driver->base);
    chConfig->sls.output.pin    = NULL_PTR;
    chConfig->sls.output.mode   = IfxPort_OutputMode_pushPull;
    chConfig->sls.output.driver = IfxPort_PadDriver_cmosAutomotiveSpeed1;
    chConfig->sls.input.pin     = NULL_PTR;
    chConfig->sls.input.mode    = IfxPort_InputMode_noPullDevice;
}


void IfxQspi_Spi_initModule(IfxQspi_Spi *driver, const IfxQspi_Spi_Config *config)
{
    Ifx_QSPI *module = config->qspi;

    /* driver.base must be at offset 0 to be compatible with the standard interface SscIf */
    // TODO: offsetof - where is it defined?
    //IFX_ASSERT(VERBOSE_LEVEL_ERROR, offsetof(IfxQspi_Spi, base) == 0);

    {
        uint16 password = IfxScuWdt_getCpuWatchdogPassword();
        IfxScuWdt_clearCpuEndinit(password);
        IfxQspi_setEnableModuleRequest(module);
        IfxQspi_setSleepMode(module, (config->allowSleepMode != FALSE) ? IfxQspi_SleepMode_enable : IfxQspi_SleepMode_disable);
        IfxScuWdt_setCpuEndinit(password);
    }

    {                                                        /* Configure GLOBAL, Note: at the moment default values for GLOBAL */
        Ifx_QSPI_GLOBALCON globalcon;
        globalcon.U        = 0;
        globalcon.B.TQ     = IfxQspi_calculateTimeQuantumLength(module, config->base.maximumBaudrate);
        globalcon.B.EXPECT = IfxQspi_ExpectTimeout_2097152;  /* 2^(EXPECT+6) : timeout for expect phase in Tqspi */
        //globalcon.B.LB      = 0;                             /* 0 : disable loop-back w*/
        //globalcon.B.DEL0    = 0;                             /* 0 : disable delayed mode for SLSO 0 */
        //globalcon.B.STROBE  = 0;                             /* (STROBE+1) : strobe delay for SLSO 0 in Tq */
        //globalcon.B.SRF     = 0;                             /* 0 : disable stop-on-RXFIFO full feature */
        //globalcon.B.STIP    = 0;                             /* 0 : MRST = 0 when QSPI is deselected in slave mode */
        //globalcon.B.EN      = 0;                             /* 0 : PAUSE requested, 1 : RUN requested */
        globalcon.B.MS      = (config->base.mode == SpiIf_Mode_master) ? IfxQspi_Mode_master : IfxQspi_Mode_slave;
        globalcon.B.AREN    = (config->pauseOnBaudrateSpikeErrors != FALSE) ? 1U : 0U;
        globalcon.B.RESETS  = 1;
        module->GLOBALCON.U = globalcon.U;
    }

    {   /* Configure interrupt requests */
        Ifx_QSPI_GLOBALCON1 globalcon1;
        globalcon1.U           = 0;
        //TODO: globalcon1.B.ERRORENS;
        globalcon1.B.TXEN      = config->enabledInterrupts.tx;
        globalcon1.B.RXEN      = config->enabledInterrupts.rx;
        globalcon1.B.PT1EN     = config->enabledInterrupts.pt1;
        globalcon1.B.PT2EN     = config->enabledInterrupts.pt2;
        globalcon1.B.USREN     = config->enabledInterrupts.usr;
        globalcon1.B.TXFIFOINT = config->txFifoThreshold;
        globalcon1.B.RXFIFOINT = config->rxFifoThreshold;
        module->GLOBALCON1.U   = globalcon1.U;
    }

    IfxQspi_run(module);

    if (config->base.mode != SpiIf_Mode_slave)
    {   /* Configure I/O pins for master mode */
        const IfxQspi_Spi_MasterPins *pinsConfig = config->pinsConfig.master;

        if (pinsConfig != NULL_PTR)
        {
            IfxQspi_Sclk_Out *sclk = pinsConfig->sclk;

            if (sclk != NULL_PTR)
            {
                IfxPort_setPinModeOutput(sclk->pin.port, sclk->pin.pinIndex, pinsConfig->sclkMode, sclk->select);
                IfxPort_setPinPadDriver(sclk->pin.port, sclk->pin.pinIndex, pinsConfig->pinDriver);
            }

            IfxQspi_Mtsr_Out *mtsr = pinsConfig->mtsr;

            if (mtsr != NULL_PTR)
            {
                IfxPort_setPinModeOutput(mtsr->pin.port, mtsr->pin.pinIndex, pinsConfig->mtsrMode, mtsr->select);
                IfxPort_setPinPadDriver(mtsr->pin.port, mtsr->pin.pinIndex, pinsConfig->pinDriver);
            }

            IfxQspi_Mrst_In *mrst = pinsConfig->mrst;

            if (mrst != NULL_PTR)
            {
                IfxPort_setPinModeInput(mrst->pin.port, mrst->pin.pinIndex, pinsConfig->mrstMode);
                module->PISEL.B.MRIS = pinsConfig->mrst->select;
            }
        }
    }
    else
    {   /* Configure I/O pins for slave mode */
        const IfxQspi_Spi_SlavePins *pinsConfig = config->pinsConfig.slave;

        if (pinsConfig != NULL_PTR)
        {
            IfxQspi_Sclk_In *sclk = pinsConfig->sclk;

            if (sclk != NULL_PTR)
            {
                IfxPort_setPinModeInput(sclk->pin.port, sclk->pin.pinIndex, pinsConfig->sclkMode);
            }

            IfxQspi_Mtsr_In *mtsr = pinsConfig->mtsr;

            if (mtsr != NULL_PTR)
            {
                IfxPort_setPinModeInput(mtsr->pin.port, mtsr->pin.pinIndex, pinsConfig->mtsrMode);
            }

            IfxQspi_Mrst_Out *mrst = pinsConfig->mrst;

            if (mrst != NULL_PTR)
            {
                IfxPort_setPinModeOutput(mrst->pin.port, mrst->pin.pinIndex, pinsConfig->mrstMode, mrst->select);
                IfxPort_setPinPadDriver(mrst->pin.port, mrst->pin.pinIndex, pinsConfig->pinDriver);
            }
        }
    }

    {
        /** NOTE: If not DMA, the interrupt service provider is assigned to the CPU ID where
         * this function is called from */
        IfxSrc_Tos tos = config->base.isrProvider;
        IfxQspi_clearAllEventFlags(module);

        if (config->base.txPriority != 0)
        {
            volatile Ifx_SRC_SRCR *src = IfxQspi_getTransmitSrc(module);
            IfxSrc_init(src, tos, config->base.txPriority);
            IfxSrc_enable(src);
        }

        if (config->base.rxPriority != 0)
        {
            volatile Ifx_SRC_SRCR *src = IfxQspi_getReceiveSrc(module);
            IfxSrc_init(src, tos, config->base.rxPriority);
            IfxSrc_enable(src);
        }
    }

    driver->qspi                     = module;
    driver->base.driver              = driver;
    driver->base.sending             = 0U;
    driver->base.activeChannel       = NULL_PTR;

    driver->base.functions.exchange  = (SpiIf_Exchange) & IfxQspi_Spi_exchange;
    driver->base.functions.getStatus = (SpiIf_GetStatus) & IfxQspi_Spi_getStatus;

    driver->base.functions.onTx      = (SpiIf_OnEvent) & IfxQspi_Spi_isrTransmit;
    driver->base.functions.onRx      = (SpiIf_OnEvent) & IfxQspi_Spi_isrReceive;
    driver->base.functions.onError   = (SpiIf_OnEvent) & IfxQspi_Spi_isrError;
}


void IfxQspi_Spi_initModuleConfig(IfxQspi_Spi_Config *config, Ifx_QSPI *qspi)
{
    SpiIf_initConfig(&config->base);
    config->qspi                       = qspi;
    config->allowSleepMode             = FALSE;
    config->pauseOnBaudrateSpikeErrors = FALSE,
    config->pauseRunTransition         = IfxQspi_PauseRunTransition_pause;
    config->enabledInterrupts.tx       = TRUE;
    config->enabledInterrupts.rx       = TRUE;
    config->enabledInterrupts.pt1      = FALSE;
    config->enabledInterrupts.pt2      = FALSE;
    config->enabledInterrupts.usr      = FALSE;
    config->txFifoThreshold            = IfxQspi_TxFifoInt_1;
    config->rxFifoThreshold            = IfxQspi_RxFifoInt_0;
    config->pinsConfig.master          = NULL_PTR;
    config->pinsConfig.slave           = NULL_PTR;
}


void IfxQspi_Spi_isrError(IfxQspi_Spi *driver)
{
    // FIXME implement error handler
}


void IfxQspi_Spi_isrReceive(IfxQspi_Spi *driver)
{
    IfxQspi_Spi_Channel *channel = IfxQspi_Spi_activeChannel(driver);
    channel->base.rxHandler(&channel->base);
    driver->base.rxCount++;
}


void IfxQspi_Spi_isrTransmit(IfxQspi_Spi *driver)
{
    IfxQspi_Spi_Channel *channel = IfxQspi_Spi_activeChannel(driver);
    channel->base.txHandler(&channel->base);
    driver->base.txCount++;
}


static SpiIf_Status IfxQspi_Spi_lock(IfxQspi_Spi *driver)
{
    sint32 sending = __swap((unsigned int *)&driver->base.sending, 1UL);
    return (sending == 0) ? SpiIf_Status_ok : SpiIf_Status_busy;
}


static void IfxQspi_Spi_read(IfxQspi_Spi_Channel *chHandle)
{
    IfxQspi_Spi *driver = chHandle->base.driver->driver;
    Ifx_QSPI    *module = driver->qspi;
    SpiIf_Job   *job    = &chHandle->base.rx;
    SizeT        count  = (SizeT)IfxQspi_getReceiveFifoLevel(module);
    count = __min(job->remaining, count);

    if (job->data == NULL_PTR)
    {
        // no data should be buffered: do dummy reads
        int i;

        for (i = 0; i < count; ++i)
        {
            IfxQspi_readReceiveFifo(module);
        }
    }
    else
    {
        if (chHandle->dataWidth <= 8)
        {
            IfxQspi_read8(module, job->data, count);
            job->data = &(((uint8 *)job->data)[count]);
        }
        else if (chHandle->dataWidth <= 16)
        {
            IfxQspi_read16(module, job->data, count);
            job->data = &(((uint16 *)job->data)[count]);
        }
        else
        {
            IfxQspi_read32(module, job->data, count);
            job->data = &(((uint32 *)job->data)[count]);
        }
    }

    job->remaining = job->remaining - count;

    if (job->remaining == 0)
    {
        if (chHandle->deactivateSlso != NULL_PTR)
        {
            chHandle->deactivateSlso(chHandle);
        }

        chHandle->base.flags.onTransfer = 0;
        IfxQspi_Spi_unlock((IfxQspi_Spi *)chHandle->base.driver);
    }
}


static void IfxQspi_Spi_unlock(IfxQspi_Spi *driver)
{
    driver->base.sending = 0UL;
}

#if 0
static void IfxQspi_Spi_write(IfxQspi_Spi_Channel *chHandle)
{
    SpiIf_Job *job = &chHandle->base.tx;

    if (job->remaining > 0)
    {
        IfxQspi_Spi *driver = chHandle->base.driver->driver;
        Ifx_QSPI    *module = driver->qspi;
        uint8        cs     = chHandle->bacon.B.CS;

        // following operation must be atomic (FIXME actually, we would only have to stall the Tx interrupt)
        boolean      interruptState = IfxCpu_disableInterrupts();
        sint64       count          = (SizeT)(IFXQSPI_CFG_HWFIFO_DEPTH - 1 - IfxQspi_getTransmitFifoLevel(module)); // -1, since BACON allocates one FIFO entry
        count = __min(job->remaining, count);

        if (count > 0)
        {
            job->remaining = job->remaining - count;

            // push BACON into FIFO before first data word
            if (chHandle->firstWrite == TRUE)
            {
                chHandle->firstWrite = FALSE;
                IfxQspi_writeBasicConfigurationBeginStream(module, chHandle->bacon.U);
            }

            // we have to push another BACON into FIFO before the last data word
            boolean lastWrite = job->remaining == 0;

            if (job->data == NULL_PTR)
            {
                // no data should be sent (only received): send all-1
                int i;

                if (lastWrite)
                {
                    for (i = 0; i < count - 1; ++i)
                    {
                        IfxQspi_writeTransmitFifo(module, cs, ~0);
                    }

                    IfxQspi_writeBasicConfigurationEndStream(module, chHandle->bacon.U);
                    IfxQspi_writeTransmitFifo(module, cs, ~0);
                }
                else
                {
                    for (i = 0; i < count; ++i)
                    {
                        IfxQspi_writeTransmitFifo(module, cs, ~0);
                    }
                }
            }
            else
            {
                if (chHandle->dataWidth <= 8)
                {
                    if (lastWrite)
                    {
                        if (count >= 2)
                        {
                            IfxQspi_write8(module, cs, job->data, count - 1);
                        }

                        IfxQspi_writeBasicConfigurationEndStream(module, chHandle->bacon.U);
                        IfxQspi_writeTransmitFifo(module, cs, ((uint8 *)job->data)[count - 1]);
                    }
                    else
                    {
                        IfxQspi_write8(module, cs, job->data, count);
                    }

                    job->data = &(((uint8 *)job->data)[count]);
                }
                else if (chHandle->dataWidth <= 16)
                {
                    if (lastWrite)
                    {
                        if (count >= 2)
                        {
                            IfxQspi_write8(module, cs, job->data, count - 1);
                        }

                        IfxQspi_writeBasicConfigurationEndStream(module, chHandle->bacon.U);
                        IfxQspi_writeTransmitFifo(module, cs, ((uint16 *)job->data)[count - 1]);
                    }
                    else
                    {
                        IfxQspi_write16(module, cs, job->data, count);
                        job->data = &(((uint16 *)job->data)[count]);
                    }
                }
                else
                {
                    if (lastWrite)
                    {
                        if (count >= 2)
                        {
                            IfxQspi_write8(module, cs, job->data, count - 1);
                        }

                        IfxQspi_writeBasicConfigurationEndStream(module, chHandle->bacon.U);
                        IfxQspi_writeTransmitFifo(module, cs, ((uint32 *)job->data)[count - 1]);
                    }
                    else
                    {
                        IfxQspi_write32(module, cs, job->data, count);
                        job->data = &(((uint32 *)job->data)[count]);
                    }
                }
            }
        }

        IfxCpu_restoreInterrupts(interruptState);
    }
}
#else
static void IfxQspi_Spi_write(IfxQspi_Spi_Channel *chHandle)
{
	SpiIf_Job *job = &chHandle->base.tx;

	if (job->remaining > 0)
	{
		IfxQspi_Spi *driver = chHandle->base.driver->driver;
		Ifx_QSPI    *module = driver->qspi;
		uint8        cs     = chHandle->bacon.B.CS;
		int i;
		// following operation must be atomic (FIXME actually, we would only have to stall the Tx interrupt)
		boolean      interruptState = IfxCpu_disableInterrupts();
		sint64       count          = (SizeT)((IFXQSPI_CFG_HWFIFO_DEPTH>>1) - IfxQspi_getTransmitFifoLevel(module)); // -1, since BACON allocates one FIFO entry per word
		count = __min(job->remaining, count);

		if (count > 0)
		{
			job->remaining = job->remaining - count;

			if (job->data == NULL_PTR)
			{
				// no data should be sent (only received): send all-1
				int i;
				for (i = 0; i < count; ++i)
				{
					IfxQspi_writeBasicConfigurationEndStream(module, chHandle->bacon.U);
					IfxQspi_writeTransmitFifo(module, cs, ~0);
				}
			}
			else
			{
				if (chHandle->dataWidth <= 8)
				{
					for (i = 0; i < count; i++)
					{
						IfxQspi_writeBasicConfigurationEndStream(module, chHandle->bacon.U);
						IfxQspi_writeTransmitFifo(module, cs,  ((uint8 *)job->data)[i]);
					}
					job->data = &(((uint8 *)job->data)[count]);

				}
				else if (chHandle->dataWidth <= 16)
				{

					for (i = 0; i < count; i++)
					{
						IfxQspi_writeBasicConfigurationEndStream(module, chHandle->bacon.U);
						IfxQspi_writeTransmitFifo(module, cs, ((uint16 *)job->data)[i]);
					}
					job->data = &(((uint16 *)job->data)[count]);
				}
				else
				{
					for (i = 0; i < count; i++)
					{
						IfxQspi_writeBasicConfigurationEndStream(module, chHandle->bacon.U);
						IfxQspi_writeTransmitFifo(module, cs, ((uint32 *)job->data)[i]);
					}
					job->data = &(((uint32 *)job->data)[count]);
				}
			}
		}

		IfxCpu_restoreInterrupts(interruptState);
	}
}
#endif
