/**
 * \file IfxQspi_Spi.h
 * \brief QSPI SPI details
 * \ingroup IfxLld_Qspi
 *
 * \copyright Copyright (c) 2013 Infineon Technologies AG. All rights reserved.
 *
 * $Date: $2013-08-12 08:55:01 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * \defgroup IfxLld_Qspi QSPI
 * \addtogroup IfxLld_Qspi
 * \{
 * \defgroup IfxLld_Qspi_Spi_Usage How to use the SPI Interface driver?
 * \addtogroup IfxLld_Qspi_Spi_Usage
 * \{
 *
 * The SPI interface driver provides a default QSPI configuration for a bidirectional serial communication of data words.
 *
 * Data transactions are buffered by the hardware based FIFOs. Incoming and outgoing data is transfered in background from/to the QSPI peripheral by interrupt service handlers, which are part of this driver as well. This allows a nonblocking communication without stalling the thread(s) from where data is sent and received.
 *
 * In the following sections it will be described, how to integrate the driver into the application framework.
 *
 * \section IfxLld_Qspi_Spi_Preparation Preparation
 * \subsection IfxLld_Qspi_Spi_Include Include Files
 *
 * Include following header file into your C code:
 * \code
 * #include <Qspi/Spi/IfxQspi_Spi.h>
 * \endcode
 *
 * \subsection IfxLld_Qspi_Spi_Variables Variables
 *
 * Declare QSPI module and channel handles:
 *
 * \code
 * IfxQspi_Spi spi;
 * IfxQspi_Spi_Channel spiChannel;
 * \endcode
 *
 * In addition, declare global transmit and receive buffers for the data transfers:
 * \code
 * #define SPI_BUFFER_SIZE 8
 * uint8	spiTxBuffer[SPI_BUFFER_SIZE];
 * uint8	spiRxBuffer[SPI_BUFFER_SIZE];
 * \endcode
 *
 * \subsection IfxLld_Qspi_Spi_Interrupt Interrupt Handler Installation
 *
 * See also \ref IfxLld_Cpu_Interrupt_Usage
 *
 * Define priorities for the Interrrupt handlers. This is normally done in the Ifx_IntPrioDef.h file:
 * \code
 * // priorities are normally defined in Ifx_IntPrioDef.h
 * #define IFX_INTPRIO_QSPI0_TX  1
 * #define IFX_INTPRIO_QSPI0_RX  3
 * #define IFX_INTPRIO_QSPI0_ER  5
 * \endcode
 *
 * Add the interrupt service routines to your C code. They have to call the QSPI interrupt handlers by passing the spi handle:
 * \code
 * IFX_INTERRUPT(qspi0TxISR, 0, IFX_INTPRIO_QSPI0_TX)
 * {
 *     IfxQspi_Spi_isrTransmit(&spiMaster);
 * }
 *
 * IFX_INTERRUPT(qspi0RxISR, 0, IFX_INTPRIO_QSPI0_RX)
 * {
 *     IfxQspi_Spi_isrReceive(&spiMaster);
 * }
 *
 * IFX_INTERRUPT(qspi0ErISR, 0, IFX_INTPRIO_QSPI0_ER)
 * {
 *     IfxQspi_Spi_isrError(&spiMaster);
 * }
 * \endcode
 *
 * Finally install the interrupt handlers in your initialisation function:
 * \code
 *     // install interrupt handlers
 *     IfxCpu_Irq_installInterruptHandler(IFX_INTPRIO_QSPI0_TX, &qspi0TxISR);
 *     IfxCpu_Irq_installInterruptHandler(IFX_INTPRIO_QSPI0_RX, &qspi0RxISR);
 *     IfxCpu_Irq_installInterruptHandler(IFX_INTPRIO_QSPI0_ER, &qspi0ErISR);
 *     IfxCpu_enableInterrupts();
 * \endcode
 *
 * \subsection IfxLld_Qspi_Spi_Init Module Initialisation
 *
 * The module initialisation can be done in the same function.
 *
 * Here an example for master mode:
 * \code
 *     // create module config
 *     IfxQspi_Spi_Config spiMasterConfig;
 *     IfxQspi_Spi_initModuleConfig(&spiMasterConfig, &MODULE_QSPI0);
 *
 *     // set the desired mode and maximum baudrate
 *     spiMasterConfig.base.mode             = SpiIf_Mode_master;
 *     spiMasterConfig.base.maximumBaudrate  = 10000000;
 *
 *     // ISR priorities and interrupt target
 *     spiMasterConfig.base.txPriority       = IFX_INTPRIO_QSPI0_TX;
 *     spiMasterConfig.base.rxPriority       = IFX_INTPRIO_QSPI0_RX;
 *     spiMasterConfig.base.erPriority       = IFX_INTPRIO_QSPI0_ER;
 *     spiMasterConfig.base.isrProvider      = (IfxSrc_Tos)IfxCpu_getCoreId();
 *
 *     // pin configuration
 *     const IfxQspi_Spi_MasterPins masterPins = {
 *         &IfxQspi0_SCLK_P20_11_OUT, IfxPort_OutputMode_pushPull, // SCLK
 *         &IfxQspi0_MTSR_P20_14_OUT, IfxPort_OutputMode_pushPull, // MTSR
 *         &IfxQspi0_MRSTA_P20_12_IN, IfxPort_InputMode_pullDown,  // MRST
 *         IfxPort_PadDriver_cmosAutomotiveSpeed3 // pad driver mode
 *     };
 *     spiMasterConfig.pinsConfig.master     = (IfxQspi_Spi_MasterPins *)&masterPins;
 *
 *     // initialize module
 *     //IfxQspi_Spi spi; // defined globally
 *     IfxQspi_Spi_initModule(&spi, &spiMasterConfig);
 * \endcode
 *
 * And here an example for slave mode:
 * \code
 *     // create module config
 *     IfxQspi_Spi_Config spiSlaveConfig;
 *     IfxQspi_Spi_initModuleConfig(&spiSlaveConfig, &MODULE_QSPI2);
 *
 *     // set the desired mode and maximum baudrate
 *     spiSlaveConfig.base.mode             = SpiIf_Mode_slave;
 *     spiSlaveConfig.base.maximumBaudrate  = 10000000;
 *
 *     // ISR priorities and interrupt target
 *     spiSlaveConfig.base.txPriority       = IFX_INTPRIO_QSPI2_TX;
 *     spiSlaveConfig.base.rxPriority       = IFX_INTPRIO_QSPI2_RX;
 *     spiSlaveConfig.base.erPriority       = IFX_INTPRIO_QSPI2_ER;
 *     spiSlaveConfig.base.isrProvider      = (IfxSrc_Tos)IfxCpu_getCoreId();
 *
 *     // pin configuration
 *     const IfxQspi_Spi_SlavePins slavePins = {
 *         &IfxQspi2_SCLKA_P15_3_IN, IfxPort_InputMode_pullDown,   // SCLK Pin
 *         &IfxQspi2_MTSRA_P15_5_IN, IfxPort_InputMode_pullDown,   // MTSR Pin
 *         &IfxQspi2_MRST_P15_7_OUT, IfxPort_OutputMode_pushPull,  // MRST Pin
 *         IfxPort_PadDriver_cmosAutomotiveSpeed3 // pad driver mode
 *     };
 *     spiSlaveConfig.pinsConfig.slave      = (IfxQspi_Spi_SlavePins *)&slavePins;
 *
 *     // initialize module
 *     //IfxQspi_Spi spi; // defined globally
 *     IfxQspi_Spi_initModule(&spi, &spiSlaveConfig);
 * \endcode
 *
 *
 * \subsection IfxLld_Qspi_Spi_ChannelInit SPI Channel Initialisation
 *
 * After the module has been initialized, one or more SPI channels can be configured.
 * Each channel has a dedicated select line.
 *
 * Here an example for a SPI channel in master mode:
 * \code
 *     // create channel config
 *     IfxQspi_Spi_ChannelConfig spiMasterChannelConfig;
 *     IfxQspi_Spi_initChannelConfig(&spiMasterChannelConfig, &spi);
 *
 *     // set the baudrate for this channel
 *     spiMasterChannelConfig.base.baudrate = 5000000;
 *
 *     // select pin configuration
 *     const IfxQspi_Spi_Output slsOutput = {
 *         &IfxQspi0_SLSO7_P33_5_OUT,
 *         IfxPort_OutputMode_pushPull,
 *         IfxPort_PadDriver_cmosAutomotiveSpeed1
 *     };
 *     spiMasterChannelConfig.sls.output = (IfxQspi_Spi_Output)slsOutput;
 *
 *     // initialize channel
 *     //IfxQspi_Spi_Channel spiChannel; // defined globally
 *     IfxQspi_Spi_initChannel(&spiChannel, &spiMasterChannelConfig);
 * \endcode
 *
 * Here an example for a SPI channel in slave mode:
 * \code
 *     // create channel config
 *     IfxQspi_Spi_ChannelConfig spiSlaveChannelConfig;
 *     IfxQspi_Spi_initChannelConfig(&spiSlaveChannelConfig, &spi);
 *
 *     // set the expected baudrate for this channel (for baudrate error detection)
 *     spiSlaveChannelConfig.base.baudrate = 5000000;
 *
 *     // select pin configuration
 *     const IfxQspi_Spi_Input slsInput = {
 *         &IfxQspi2_SLSIA_P15_2_IN,
 *         IfxPort_InputMode_pullDown
 *     };
 *     spiSlaveChannelConfig.sls.input = (IfxQspi_Spi_Input)slsInput;
 *
 *     // initialize channel
 *     //IfxQspi_Spi_Channel spiChannel; // defined globally
 *     IfxQspi_Spi_initChannel(&spiChannel, &spiSlaveChannelConfig);
 * \endcode
 *
 * The QSPI is ready for use now!
 *
 *
 * \section IfxLld_Qspi_Spi_DataTransfers Data Transfers
 *
 * In following examples we assume, that following buffers are declared globally:
 * \code
 * // declared somewhere globally
 * #define SPI_BUFFER_SIZE 8
 * uint8 spiTxBuffer[SPI_BUFFER_SIZE] = { 1, 2, 3, 4, 5, 6, 7, 8 };
 * uint8 spiRxBuffer[SPI_BUFFER_SIZE] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 * \endcode
 *
 * Sending and Receiving a data stream:
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_Spi_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // send/receive new stream
 *     IfxQspi_Spi_exchange(&spiChannel, &spiTxBuffer[i], &spiRxBuffer[i], SPI_BUFFER_SIZE);
 * \endcode
 *
 * Send only, discard received data:
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_Spi_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // send new stream
 *     IfxQspi_Spi_exchange(&spiChannel, &spiTxBuffer[i], NULL_PTR, SPI_BUFFER_SIZE);
 * \endcode
 *
 * Receive only, send all-1
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_Spi_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // receive new stream
 *     IfxQspi_Spi_exchange(&spiChannel, NULL_PTR, &spiRxBuffer[i], SPI_BUFFER_SIZE);
 * \endcode
 *
 * The handling of slave transfers is identical.
 *
 * \}
 * \}
 *
 * \defgroup IfxLld_Qspi_Spi Interface Driver
 * \ingroup IfxLld_Qspi
 * \defgroup IfxLld_Qspi_Spi_DataStructures Data Structures
 * \ingroup IfxLld_Qspi_Spi
 * \defgroup IfxLld_Qspi_Spi_Module Module Functions
 * \ingroup IfxLld_Qspi_Spi
 * \defgroup IfxLld_Qspi_Spi_Channel Channel Functions
 * \ingroup IfxLld_Qspi_Spi
 * \defgroup IfxLld_Qspi_Spi_Support Support Functions
 * \ingroup IfxLld_Qspi_Spi
 * \defgroup IfxLld_Qspi_Spi_Com Communication
 * \ingroup IfxLld_Qspi_Spi
 * \defgroup IfxLld_Qspi_Spi_InterruptFunctions Interrupt Functions
 * \ingroup IfxLld_Qspi_Spi
 * \defgroup IfxLld_Qspi_Spi_DirectFifo Direct FIFO Access
 * \ingroup IfxLld_Qspi_Spi
 */

#ifndef IFXQSPI_SPI_H
#define IFXQSPI_SPI_H

/*******************************************************************************
**                                 Includes                                   **
*******************************************************************************/
#include "Qspi/Std/IfxQspi.h"
#include "If/SpiIf.h"
#include "_PinMap/IfxQspi_PinMap.h"

/*******************************************************************************
**                           Type Definitions                                 **
*******************************************************************************/
typedef struct IfxQspi_Spi_Channel_s IfxQspi_Spi_Channel;

typedef void                       (*IfxQspi_Spi_AutoSlso)(IfxQspi_Spi_Channel *chHandle);

/*******************************************************************************
**                           Data Structures                                  **
*******************************************************************************/
/** \addtogroup IfxLld_Qspi_Spi_DataStructures
 * \{ */
typedef struct
{
    IfxQspi_Slsi_In  *pin;
    IfxPort_InputMode mode;
} IfxQspi_Spi_Input;

typedef struct
{
    const IfxQspi_Sclk_Out *sclk;
    IfxPort_OutputMode      sclkMode;
    const IfxQspi_Mtsr_Out *mtsr;
    IfxPort_OutputMode      mtsrMode;
    const IfxQspi_Mrst_In  *mrst;
    IfxPort_InputMode       mrstMode;
    IfxPort_PadDriver       pinDriver;
} IfxQspi_Spi_MasterPins;

typedef struct
{
    IfxQspi_Slso_Out  *pin;
    IfxPort_OutputMode mode;
    IfxPort_PadDriver  driver;
} IfxQspi_Spi_Output;

typedef struct
{
    const IfxQspi_Sclk_In  *sclk;
    IfxPort_InputMode       sclkMode;
    const IfxQspi_Mtsr_In  *mtsr;
    IfxPort_InputMode       mtsrMode;
    const IfxQspi_Mrst_Out *mrst;
    IfxPort_OutputMode      mrstMode;
    IfxPort_PadDriver       pinDriver;
} IfxQspi_Spi_SlavePins;

/** \} */

/** \addtogroup IfxLld_Qspi_Spi_DataStructures
 * \{ */
typedef struct
{
    SpiIf     base;
    Ifx_QSPI *qspi;
} IfxQspi_Spi;

typedef struct
{
    boolean tx;
    boolean rx;
    boolean pt1;
    boolean pt2;
    boolean usr;
} IfxQspi_Spi_EnabledInterrupts;

typedef union
{
    IfxQspi_Spi_Input  input;
    IfxQspi_Spi_Output output;
} IfxQspi_Spi_InputOutput;

typedef union
{
    IfxQspi_Spi_MasterPins *master;
    IfxQspi_Spi_SlavePins  *slave;
} IfxQspi_Spi_PinsConfig;

/** \} */

/** \addtogroup IfxLld_Qspi_Spi_DataStructures
 * \{ */
typedef struct
{
    SpiIf_ChConfig          base;
    IfxQspi_Spi_InputOutput sls;
} IfxQspi_Spi_ChannelConfig;

struct IfxQspi_Spi_Channel_s
{
    SpiIf_Ch             base;
    IfxQspi_Spi         *module;
    Ifx_QSPI_BACON       bacon;
    IfxPort_Pin          slso;
    IfxQspi_Spi_AutoSlso activateSlso;
    IfxQspi_Spi_AutoSlso deactivateSlso;
    IfxQspi_ChannelId    channelId;
    ActiveState          slsoActiveState;
    uint8                dataWidth;     /**< \brief Number of bits which will be written into the FIFO */
    boolean              firstWrite;
};

typedef struct
{
    SpiIf_Config                  base;
    Ifx_QSPI                     *qspi;
    boolean                       allowSleepMode;
    boolean                       pauseOnBaudrateSpikeErrors;
    IfxQspi_PauseRunTransition    pauseRunTransition;
    IfxQspi_TxFifoInt             txFifoThreshold;
    IfxQspi_RxFifoInt             rxFifoThreshold;
    IfxQspi_Spi_EnabledInterrupts enabledInterrupts;
    IfxQspi_Spi_PinsConfig        pinsConfig;
} IfxQspi_Spi_Config;

/** \} */

/** \addtogroup IfxLld_Qspi_Spi_Channel
 * \{ */

/******************************************************************************/
/*                        Global Function prototypes                          */
/******************************************************************************/

/** \brief Initializes the channel
 * \param chHandle Channel handle
 * \param chConfig Channel configuration structure
 * \return Status of Channel (busy or ok or failure)
 *
 * Configuration example for a QSPI channel in master mode:
 * \code
 *     // create channel config
 *     IfxQspi_Spi_ChannelConfig spiMasterChannelConfig;
 *     IfxQspi_Spi_initChannelConfig(&spiMasterChannelConfig, &spi);
 *
 *     // set the baudrate for this channel
 *     spiMasterChannelConfig.base.baudrate = 5000000;
 *
 *     // select pin configuration
 *     const IfxQspi_Spi_Output slsOutput = {
 *         &IfxQspi0_SLSO7_P33_5_OUT,
 *         IfxPort_OutputMode_pushPull,
 *         IfxPort_PadDriver_cmosAutomotiveSpeed1
 *     };
 *     spiMasterChannelConfig.sls.output = (IfxQspi_Spi_Output)slsOutput;
 *
 *     // initialize channel
 *     //IfxQspi_Spi_Channel spiChannel; // defined globally
 *     IfxQspi_Spi_initChannel(&spiChannel, &spiMasterChannelConfig);
 * \endcode
 *
 * Configuration example for a QSPI channel in slave mode:
 * \code
 *     // create channel config
 *     IfxQspi_Spi_ChannelConfig spiSlaveChannelConfig;
 *     IfxQspi_Spi_initChannelConfig(&spiSlaveChannelConfig, &spi);
 *
 *     // set the expected baudrate for this channel (for baudrate error detection)
 *     spiSlaveChannelConfig.base.baudrate = 5000000;
 *
 *     // select pin configuration
 *     const IfxQspi_Spi_Input slsInput = {
 *         &IfxQspi2_SLSIA_P15_2_IN,
 *         IfxPort_InputMode_pullDown
 *     };
 *     spiSlaveChannelConfig.sls.input = (IfxQspi_Spi_Input)slsInput;
 *
 *     // initialize channel
 *     //IfxQspi_Spi_Channel spiChannel; // defined globally
 *     IfxQspi_Spi_initChannel(&spiChannel, &spiSlaveChannelConfig);
 * \endcode
 *
 */
IFX_EXTERN SpiIf_Status IfxQspi_Spi_initChannel(IfxQspi_Spi_Channel *chHandle, const IfxQspi_Spi_ChannelConfig *chConfig);

/** \brief Fills the config structure with default values
 * \param chConfig Configuration structure which should be initialized.
 * \param driver Module handle
 * \return None
 *
 * Usage example: see \ref IfxQspi_Spi_initChannel
 *
 */
IFX_EXTERN void IfxQspi_Spi_initChannelConfig(IfxQspi_Spi_ChannelConfig *chConfig, IfxQspi_Spi *driver);

/** \} */

/** \addtogroup IfxLld_Qspi_Spi_Com
 * \{ */

/******************************************************************************/
/*                        Global Function prototypes                          */
/******************************************************************************/

/** \brief Exchanges data between source and data
 * \param chHandle Channel handle
 * \param src Source of data. Can be set to NULL_PTR if nothing to receive (transmit only)
 * \param dest Destination to which to be sent. Can be set to NULL_PTR if nothing to transmit (receive only) - in this case, all-1 will be sent.
 * \param count Number of data in pending
 * \return Status of exchange of data
 *
 * In following examples we assume, that following buffers are declared globally:
 * \code
 * // declared somewhere globally
 * uint8 spiTxBuffer[8] = { 1, 2, 3, 4, 5, 6, 7, 8 };
 * uint8 spiRxBuffer[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 * \endcode
 *
 * Sending and Receiving a data stream:
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_Spi_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // send/receive new stream
 *     IfxQspi_Spi_exchange(&spiChannel, &spiTxBuffer[i], &spiRxBuffer[i], 8);
 * \endcode
 *
 * Send only, discard received data:
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_Spi_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // send new stream
 *     IfxQspi_Spi_exchange(&spiChannel, &spiTxBuffer[i], NULL_PTR, 8);
 * \endcode
 *
 * Receive only, send all-1
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_Spi_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // receive new stream
 *     IfxQspi_Spi_exchange(&spiChannel, NULL_PTR, &spiRxBuffer[i], 8);
 * \endcode
 *
 * The handling of slave transfers is identical.
 *
 */
IFX_EXTERN SpiIf_Status IfxQspi_Spi_exchange(IfxQspi_Spi_Channel *chHandle, const void *src, void *dest, SizeT count);

/** \brief Gets the transmission status
 * \param chHandle QSPI channel handle
 * \return Transmission status
 *
 * Usage example: see \ref IfxQspi_Spi_exchange
 *
 */
IFX_EXTERN SpiIf_Status IfxQspi_Spi_getStatus(IfxQspi_Spi_Channel *chHandle);

/** \} */

/** \addtogroup IfxLld_Qspi_Spi_DirectFifo
 * \{ */

/******************************************************************************/
/*                     Inline Function prototypes                             */
/******************************************************************************/

/** \brief Reads data or status in RxFIFO
 * \param driver Module handle
 * \return Data or Status in RxFIFO
 */
IFX_INLINE uint32 IfxQspi_Spi_readReceiveFifo(IfxQspi_Spi *driver);

/** \brief Writes Basic configuration value to Tx FIFO
 * \param driver QSPI module handle
 * \param baconVal Basic configuration value to be entered in TxFIFO
 * \return None
 */
IFX_INLINE void IfxQspi_Spi_writeBasicConfiguration(IfxQspi_Spi *driver, uint32 baconVal);

/** \brief Writes the data to TxFIFO
 * \param chHandle Channel handle
 * \param data Data to be entered in Tx FIFO
 * \return None
 */
IFX_INLINE void IfxQspi_Spi_writeDataTransmitFifo(IfxQspi_Spi_Channel *chHandle, uint32 data);

/** \brief Writes extended configuration of the channel
 * \param chHandle QSPI channel handle
 * \param econVal Channel Timing configuration value
 * \return None
 */
IFX_INLINE void IfxQspi_Spi_writeExtendedConfiguration(IfxQspi_Spi_Channel *chHandle, uint32 econVal);

/** \brief Writes data and configuration mixed value to Tx FIFO
 * \param driver QSPI module handle
 * \param mixEntryVal Mixed of Data and configuration
 * \return None
 */
IFX_INLINE void IfxQspi_Spi_writeMixedDataConfiguration(IfxQspi_Spi *driver, uint32 mixEntryVal);

/** \} */

/** \addtogroup IfxLld_Qspi_Spi_InterruptFunctions
 * \{ */

/******************************************************************************/
/*                        Global Function prototypes                          */
/******************************************************************************/

/** \brief Error Interrupt handler
 * \param driver Module handle
 * \return None
 */
IFX_EXTERN void IfxQspi_Spi_isrError(IfxQspi_Spi *driver);

/** \brief Receive Interrupt handler
 * \param driver Module handle
 * \return None
 */
IFX_EXTERN void IfxQspi_Spi_isrReceive(IfxQspi_Spi *driver);

/** \brief Transmit interrupt handler
 * \param driver Module handle
 * \return None
 */
IFX_EXTERN void IfxQspi_Spi_isrTransmit(IfxQspi_Spi *driver);

/** \} */

/** \addtogroup IfxLld_Qspi_Spi_Module
 * \{ */

/******************************************************************************/
/*                        Global Function prototypes                          */
/******************************************************************************/

/** \brief Initialises the module
 * \param driver Module handle
 * \param config Predefined configuration structure of the module
 * \return None
 *
 * Configuration example for QSPI in master mode:
 * \code
 *     // create module config
 *     IfxQspi_Spi_Config spiMasterConfig;
 *     IfxQspi_Spi_initModuleConfig(&spiMasterConfig, &MODULE_QSPI0);
 *
 *     // set the desired mode and maximum baudrate
 *     spiMasterConfig.base.mode             = SpiIf_Mode_master;
 *     spiMasterConfig.base.maximumBaudrate  = 10000000;
 *
 *     // ISR priorities and interrupt target
 *     spiMasterConfig.base.txPriority       = IFX_INTPRIO_QSPI0_TX;
 *     spiMasterConfig.base.rxPriority       = IFX_INTPRIO_QSPI0_RX;
 *     spiMasterConfig.base.erPriority       = IFX_INTPRIO_QSPI0_ER;
 *     spiMasterConfig.base.isrProvider      = (IfxSrc_Tos)IfxCpu_getCoreId();
 *
 *     // pin configuration
 *     const IfxQspi_Spi_MasterPins masterPins = {
 *         &IfxQspi0_SCLK_P20_11_OUT, IfxPort_OutputMode_pushPull, // SCLK
 *         &IfxQspi0_MTSR_P20_14_OUT, IfxPort_OutputMode_pushPull, // MTSR
 *         &IfxQspi0_MRSTA_P20_12_IN, IfxPort_InputMode_pullDown,  // MRST
 *         IfxPort_PadDriver_cmosAutomotiveSpeed3 // pad driver mode
 *     };
 *     spiMasterConfig.pinsConfig.master     = (IfxQspi_Spi_MasterPins *)&masterPins;
 *
 *     // initialize module
 *     //IfxQspi_Spi spi; // defined globally
 *     IfxQspi_Spi_initModule(&spi, &spiMasterConfig);
 * \endcode
 *
 * Configuration example for QSPI in slave mode:
 * \code
 *     // create module config
 *     IfxQspi_Spi_Config spiSlaveConfig;
 *     IfxQspi_Spi_initModuleConfig(&spiSlaveConfig, &MODULE_QSPI2);
 *
 *     // set the desired mode and maximum baudrate
 *     spiSlaveConfig.base.mode             = SpiIf_Mode_slave;
 *     spiSlaveConfig.base.maximumBaudrate  = 10000000;
 *
 *     // ISR priorities and interrupt target
 *     spiSlaveConfig.base.txPriority       = IFX_INTPRIO_QSPI2_TX;
 *     spiSlaveConfig.base.rxPriority       = IFX_INTPRIO_QSPI2_RX;
 *     spiSlaveConfig.base.erPriority       = IFX_INTPRIO_QSPI2_ER;
 *     spiSlaveConfig.base.isrProvider      = (IfxSrc_Tos)IfxCpu_getCoreId();
 *
 *     // pin configuration
 *     const IfxQspi_Spi_SlavePins slavePins = {
 *         &IfxQspi2_SCLKA_P15_3_IN, IfxPort_InputMode_pullDown,   // SCLK Pin
 *         &IfxQspi2_MTSRA_P15_5_IN, IfxPort_InputMode_pullDown,   // MTSR Pin
 *         &IfxQspi2_MRST_P15_7_OUT, IfxPort_OutputMode_pushPull,  // MRST Pin
 *         IfxPort_PadDriver_cmosAutomotiveSpeed3 // pad driver mode
 *     };
 *     spiSlaveConfig.pinsConfig.slave      = (IfxQspi_Spi_SlavePins *)&slavePins;
 *
 *     // initialize module
 *     //IfxQspi_Spi spi; // defined globally
 *     IfxQspi_Spi_initModule(&spi, &spiSlaveConfig);
 * \endcode
 *
 */
IFX_EXTERN void IfxQspi_Spi_initModule(IfxQspi_Spi *driver, const IfxQspi_Spi_Config *config);

/** \brief Fills the config structure with default values
 * \param config Configuration structure which should be initialized.
 * \param qspi pointer to QSPI registers
 * \return None
 *
 * Usage example: see \ref IfxQspi_Spi_initModule
 *
 */
IFX_EXTERN void IfxQspi_Spi_initModuleConfig(IfxQspi_Spi_Config *config, Ifx_QSPI *qspi);

/** \} */

/*******************************************************************************
**            Inline function implementations                                 **
*******************************************************************************/
IFX_INLINE uint32 IfxQspi_Spi_readReceiveFifo(IfxQspi_Spi *driver)
{
    Ifx_QSPI *qspiSFR = driver->qspi;
    uint32    data    = IfxQspi_readReceiveFifo(qspiSFR);
    return data;
}


IFX_INLINE void IfxQspi_Spi_writeBasicConfiguration(IfxQspi_Spi *driver, uint32 baconVal)
{
    Ifx_QSPI *qspiSFR = driver->qspi;
    IfxQspi_writeBasicConfiguration(qspiSFR, baconVal);
}


IFX_INLINE void IfxQspi_Spi_writeDataTransmitFifo(IfxQspi_Spi_Channel *chHandle, uint32 data)
{
    IfxQspi_writeTransmitFifo(chHandle->module->qspi, chHandle->channelId, data);
}


IFX_INLINE void IfxQspi_Spi_writeExtendedConfiguration(IfxQspi_Spi_Channel *chHandle, uint32 econVal)
{
    IfxQspi_writeExtendedConfiguration(chHandle->module->qspi, chHandle->channelId, econVal);
}


IFX_INLINE void IfxQspi_Spi_writeMixedDataConfiguration(IfxQspi_Spi *driver, uint32 mixEntryVal)
{
    Ifx_QSPI *qspiSFR = driver->qspi;
    IfxQspi_writeMixedDataTransmitFifo(qspiSFR, mixEntryVal);
}


#endif /* IFXQSPI_SPI_H */
