/**
 * \file IfxQspi_SpiMaster.h
 * \brief QSPI SPIMASTER details
 * \ingroup IfxLld_Qspi
 *
 * \version iLLD_0_1_0_11
 * \copyright Copyright (c) 2013 Infineon Technologies AG. All rights reserved.
 *
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 */

#ifndef IFXQSPI_SPIMASTER_H
#define IFXQSPI_SPIMASTER_H 1

/******************************************************************************/
/*----------------------------------Includes----------------------------------*/
/******************************************************************************/

#include "Qspi/Std/IfxQspi.h"
#include "If/SpiIf.h"
#include "Dma/Dma/IfxDma_Dma.h"
#include "_PinMap/IfxQspi_PinMap.h"
#include "Cpu/Std/IfxCpu.h"

/******************************************************************************/
/*------------------------------Type Definitions------------------------------*/
/******************************************************************************/

typedef struct IfxQspi_SpiMaster_Channel_s IfxQspi_SpiMaster_Channel;

typedef void                             (*IfxQspi_SpiMaster_AutoSlso)(IfxQspi_SpiMaster_Channel *chHandle);

/******************************************************************************/
/*-----------------------------Data Structures--------------------------------*/
/******************************************************************************/

/** \addtogroup IfxLld_Qspi_SpiMaster_DataStructures
 * \{ */
/** \brief Dma handle
 */
typedef struct
{
    IfxDma_Dma_Channel rxDmaChannel;       /**< \brief receive DMA channel handle */
    IfxDma_Dma_Channel txDmaChannel;       /**< \brief transmit DMA channel handle */
    IfxDma_ChannelId   rxDmaChannelId;     /**< \brief DMA channel no for the Spi recieve */
    IfxDma_ChannelId   txDmaChannelId;     /**< \brief DMA channel no for the Spi transmit */
    boolean            useDma;             /**< \brief use Dma for Data transfer/s */
} IfxQspi_SpiMaster_Dma;

/** \brief SLSI  pin configuration structure
 */
typedef struct
{
    const IfxQspi_Slsi_In *pin;      /**< \brief Pointer to SLSI in pin */
    IfxPort_InputMode      mode;     /**< \brief The SLSI pin input mode */
} IfxQspi_SpiMaster_Input;


/** \brief SLSO  pin configuration structure
 */
typedef struct
{
    const IfxQspi_Slso_Out *pin;        /**< \brief Pointer to SLSO out pin */
    IfxPort_OutputMode      mode;       /**< \brief The SLSO pin output mode */
    IfxPort_PadDriver       driver;     /**< \brief The pad driver mode which should be configured */
} IfxQspi_SpiMaster_Output;

/** \} */

/** \addtogroup IfxLld_Qspi_SpiMaster_DataStructures
 * \{ */
/** \brief Module handle data structure
 */
typedef struct
{
    SpiIf                 base;     /**< \brief Module SPI interface handle */
    Ifx_QSPI             *qspi;     /**< \brief Pointer to QSPI module registers */
    IfxQspi_SpiMaster_Dma dma;      /**< \brief dma handle */
} IfxQspi_SpiMaster;

/** \brief Dma configuration
 */
typedef struct
{
    IfxDma_ChannelId rxDmaChannelId;     /**< \brief DMA channel no for the Spi recieve */
    IfxDma_ChannelId txDmaChannelId;     /**< \brief DMA channel no for the Spi transmit */
    boolean          useDma;             /**< \brief use Dma for Data transfer/s */
} IfxQspi_SpiMaster_DmaConfig;

/** \brief Union of Slave Select pins
 */
typedef union
{
    IfxQspi_SpiMaster_Input  input;      /**< \brief SLSI  pin configuration structure */
    IfxQspi_SpiMaster_Output output;     /**< \brief SLSO  pin configuration structure */
} IfxQspi_SpiMaster_InputOutput;

/** \brief Master pin IO configuration structure
 */
typedef struct
{
    const IfxQspi_Sclk_Out *sclk;          /**< \brief Pointer to SLCK out pin */
    IfxPort_OutputMode      sclkMode;      /**< \brief The SCLK pin output mode */
    const IfxQspi_Mtsr_Out *mtsr;          /**< \brief Pointer to MTSR out pin */
    IfxPort_OutputMode      mtsrMode;      /**< \brief The MTSR pin output mode */
    const IfxQspi_Mrst_In  *mrst;          /**< \brief Pointer to MRST in pin */
    IfxPort_InputMode       mrstMode;      /**< \brief The MRST pin input mode */
    IfxPort_PadDriver       pinDriver;     /**< \brief The pad driver mode which should be configured */
} IfxQspi_SpiMaster_Pins;

/** \} */

/** \addtogroup IfxLld_Qspi_SpiMaster_DataStructures
 * \{ */
/** \brief Module Channel configuration structure
 */
typedef struct
{
    SpiIf_ChConfig                base;     /**< \brief SPI interface channel configuration structure */
    IfxQspi_SpiMaster_InputOutput sls;      /**< \brief Union of Slave Select pins */
} IfxQspi_SpiMaster_ChannelConfig;

/** \brief Module Channel handle structure
 */
struct IfxQspi_SpiMaster_Channel_s
{
    SpiIf_Ch                   base;                /**< \brief SPI interface channel handle structure */
    IfxQspi_SpiMaster         *module;              /**< \brief Module handle data structure */
    Ifx_QSPI_BACON             bacon;               /**< \brief basic configuration register */
    IfxPort_Pin                slso;                /**< \brief Defines SLSO pin */
    IfxQspi_SpiMaster_AutoSlso activateSlso;        /**< \brief Specifies  function for Auto SLSO activation */
    IfxQspi_SpiMaster_AutoSlso deactivateSlso;      /**< \brief Specifies  function for Auto SLSO deactivation */
    IfxQspi_ChannelId          channelId;           /**< \brief QSPI channel Number */
    ActiveState            slsoActiveState;     /**< \brief Specifies the current state of SLSO */
    uint8                      dataWidth;           /**< \brief Number of bits which will be written into the FIFO */
    boolean                    firstWrite;          /**< \brief Spepcifies whether the data id first write or not. */
};

/** \brief Module configuration structure
 */
typedef struct
{
    SpiIf_Config                  base;                           /**< \brief SPI interface configuration structure */
    Ifx_QSPI                     *qspi;                           /**< \brief Pointer to QSPI module registers */
    boolean                       allowSleepMode;                 /**< \brief Specifies module sleep mode */
    boolean                       pauseOnBaudrateSpikeErrors;     /**< \brief Specifies module pause on baudrate or spike errors */
    IfxQspi_PauseRunTransition    pauseRunTransition;             /**< \brief Specifies module run or pause mode */
    IfxQspi_TxFifoInt             txFifoThreshold;                /**< \brief Specifies the TXFIFO interrupt threshold */
    IfxQspi_RxFifoInt             rxFifoThreshold;                /**< \brief Specifies the RXFIFO interrupt threshold */
    const IfxQspi_SpiMaster_Pins *pins;                           /**< \brief structure for QSPI Master pins */
    IfxQspi_SpiMaster_DmaConfig   dma;                            /**< \brief Dma configuration */
} IfxQspi_SpiMaster_Config;

/** \} */

/** \addtogroup IfxLld_Qspi_SpiMaster_Module
 * \{ */

/******************************************************************************/
/*-------------------------Global Function Prototypes-------------------------*/
/******************************************************************************/

/** \brief Initialises the module
 * \param handle Module handle
 * \param config Predefined configuration structure of the module
 * \return None
 *
 * Configuration example for QSPI in master mode:
 * \code
 *     // create module config
 *     IfxQspi_SpiMaster_Config spiMasterConfig;
 *     IfxQspi_SpiMaster_initModuleConfig(&spiMasterConfig, &MODULE_QSPI0);
 *
 *     // set the desired mode and maximum baudrate
 *     spiMasterConfig.base.mode             = SpiIf_Mode_master;
 *     spiMasterConfig.base.maximumBaudrate  = 10000000;
 *
 *     // ISR priorities and interrupt target
 *     spiMasterConfig.base.txPriority       = IFX_INTPRIO_QSPI0_TX;
 *     spiMasterConfig.base.rxPriority       = IFX_INTPRIO_QSPI0_RX;
 *     spiMasterConfig.base.erPriority       = IFX_INTPRIO_QSPI0_ER;
 *     spiMasterConfig.base.isrProvider      = (IfxSrc_Tos)IfxCpu_getCoreId();
 *
 *     // pin configuration
 *     const IfxQspi_SpiMaster_Pins pins = {
 *         &IfxQspi0_SCLK_P20_11_OUT, IfxPort_OutputMode_pushPull, // SCLK
 *         &IfxQspi0_MTSR_P20_14_OUT, IfxPort_OutputMode_pushPull, // MTSR
 *         &IfxQspi0_MRSTA_P20_12_IN, IfxPort_InputMode_pullDown,  // MRST
 *         IfxPort_PadDriver_cmosAutomotiveSpeed3 // pad driver mode
 *     };
 *     spiMasterConfig.pins = &pins;
 *
 *     // initialize module
 *     //IfxQspi_SpiMaster spi; // defined globally
 *     IfxQspi_SpiMaster_initModule(&spi, &spiMasterConfig);
 * \endcode
 *
 */
IFX_EXTERN void IfxQspi_SpiMaster_initModule(IfxQspi_SpiMaster *handle, const IfxQspi_SpiMaster_Config *config);

/** \brief Fills the config structure with default values
 * \param config Configuration structure which should be initialized.
 * \param qspi pointer to QSPI registers
 * \return None
 *
 * Usage example: see \ref IfxQspi_SpiMaster_initModule
 *
 */
IFX_EXTERN void IfxQspi_SpiMaster_initModuleConfig(IfxQspi_SpiMaster_Config *config, Ifx_QSPI *qspi);

/** \} */

/** \addtogroup IfxLld_Qspi_SpiMaster_Channel
 * \{ */

/******************************************************************************/
/*-------------------------Global Function Prototypes-------------------------*/
/******************************************************************************/

/** \brief Initializes the channel
 * \param chHandle Module Channel handle
 * \param chConfig Channel configuration structure
 * \return Status of Channel (busy or ok or failure)
 *
 * Configuration example for a QSPI channel in master mode:
 * \code
 *     // create channel config
 *     IfxQspi_SpiMaster_ChannelConfig spiMasterChannelConfig;
 *     IfxQspi_SpiMaster_initChannelConfig(&spiMasterChannelConfig, &spi);
 *
 *     // set the baudrate for this channel
 *     spiMasterChannelConfig.base.baudrate = 5000000;
 *
 *     // select pin configuration
 *     const IfxQspi_SpiMaster_Output slsOutput = {
 *         &IfxQspi0_SLSO7_P33_5_OUT,
 *         IfxPort_OutputMode_pushPull,
 *         IfxPort_PadDriver_cmosAutomotiveSpeed1
 *     };
 *     spiMasterChannelConfig.sls.output = (IfxQspi_SpiMaster_Output)slsOutput;
 *
 *     // initialize channel
 *     //IfxQspi_SpiMaster_Channel spiChannel; // defined globally
 *     IfxQspi_SpiMaster_initChannel(&spiChannel, &spiMasterChannelConfig);
 * \endcode
 *
 */
IFX_EXTERN SpiIf_Status IfxQspi_SpiMaster_initChannel(IfxQspi_SpiMaster_Channel *chHandle, const IfxQspi_SpiMaster_ChannelConfig *chConfig);

/** \brief Fills the config structure with default values
 * \param chConfig Configuration structure which should be initialized.
 * \param handle Module handle
 * \return None
 *
 * Usage example: see \ref IfxQspi_SpiMaster_initChannel
 *
 */
IFX_EXTERN void IfxQspi_SpiMaster_initChannelConfig(IfxQspi_SpiMaster_ChannelConfig *chConfig, IfxQspi_SpiMaster *handle);

/** \} */

/** \addtogroup IfxLld_Qspi_SpiMaster_Com
 * \{ */

/******************************************************************************/
/*-------------------------Global Function Prototypes-------------------------*/
/******************************************************************************/

/** \brief Exchanges data between source and data
 * \param chHandle Module Channel handle
 * \param src Source of data. Can be set to NULL_PTR if nothing to receive (transmit only)
 * \param dest Destination to which to be sent. Can be set to NULL_PTR if nothing to transmit (receive only) - in this case, all-1 will be sent.
 * \param count Number of data in pending
 * \return Status of exchange of data
 *
 * In following examples we assume, that following buffers are declared globally:
 * \code
 * // declared somewhere globally
 * uint8 spiTxBuffer[8] = { 1, 2, 3, 4, 5, 6, 7, 8 };
 * uint8 spiRxBuffer[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 * \endcode
 *
 * Sending and Receiving a data stream:
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_SpiMaster_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // send/receive new stream
 *     IfxQspi_SpiMaster_exchange(&spiChannel, &spiTxBuffer[i], &spiRxBuffer[i], 8);
 * \endcode
 *
 * Send only, discard received data:
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_SpiMaster_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // send new stream
 *     IfxQspi_SpiMaster_exchange(&spiChannel, &spiTxBuffer[i], NULL_PTR, 8);
 * \endcode
 *
 * Receive only, send all-1
 * \code
 *     // wait until transfer of previous data stream is finished
 *     while( IfxQspi_SpiMaster_getStatus(&spiChannel) == SpiIf_Status_busy );
 *
 *     // receive new stream
 *     IfxQspi_SpiMaster_exchange(&spiChannel, NULL_PTR, &spiRxBuffer[i], 8);
 * \endcode
 *
 * The handling of slave transfers is identical.
 *
 */
IFX_EXTERN SpiIf_Status IfxQspi_SpiMaster_exchange(IfxQspi_SpiMaster_Channel *chHandle, const void *src, void *dest, SizeT count);

/** \brief Gets the transmission status
 * \param chHandle Module Channel handle
 * \return Transmission status
 *
 * Usage example: see \ref IfxQspi_SpiMaster_exchange
 *
 */
IFX_EXTERN SpiIf_Status IfxQspi_SpiMaster_getStatus(IfxQspi_SpiMaster_Channel *chHandle);

/** \} */

/** \addtogroup IfxLld_Qspi_SpiMaster_InterruptFunctions
 * \{ */

/******************************************************************************/
/*-------------------------Global Function Prototypes-------------------------*/
/******************************************************************************/

/** \brief Transmit interrupt handler
 * \param qspiHandle Module handle
 * \return None
 */
IFX_EXTERN void IfxQspi_SpiMaster_isrDmaReceive(IfxQspi_SpiMaster *qspiHandle);

/** \brief Transmit interrupt handler
 * \param qspiHandle Module handle
 * \return None
 */
IFX_EXTERN void IfxQspi_SpiMaster_isrDmaTransmit(IfxQspi_SpiMaster *qspiHandle);

/** \brief Error Interrupt handler
 * \param handle Module handle
 * \return None
 */
IFX_EXTERN void IfxQspi_SpiMaster_isrError(IfxQspi_SpiMaster *handle);

/** \brief Receive Interrupt handler
 * \param handle Module handle
 * \return None
 */
IFX_EXTERN void IfxQspi_SpiMaster_isrReceive(IfxQspi_SpiMaster *handle);

/** \brief Transmit interrupt handler
 * \param handle Module handle
 * \return None
 */
IFX_EXTERN void IfxQspi_SpiMaster_isrTransmit(IfxQspi_SpiMaster *handle);

/** \} */

/** \addtogroup IfxLld_Qspi_SpiMaster_DirectFifo
 * \{ */

/******************************************************************************/
/*-------------------------Inline Function Prototypes-------------------------*/
/******************************************************************************/

/** \brief Reads data or status in RxFIFO
 * \param handle Module handle
 * \return Data or Status in RxFIFO
 */
IFX_INLINE uint32 IfxQspi_SpiMaster_readReceiveFifo(IfxQspi_SpiMaster *handle);

/** \brief Writes Basic configuration value to Tx FIFO
 * \param handle Module handle
 * \param baconVal Basic configuration value to be entered in TxFIFO
 * \return None
 */
IFX_INLINE void IfxQspi_SpiMaster_writeBasicConfiguration(IfxQspi_SpiMaster *handle, uint32 baconVal);

/** \brief Writes extended configuration of the channel
 * \param chHandle Module Channel handle
 * \param econVal Channel Timing configuration value
 * \return None
 */
IFX_INLINE void IfxQspi_SpiMaster_writeExtendedConfiguration(IfxQspi_SpiMaster_Channel *chHandle, uint32 econVal);

/** \brief Writes data and configuration mixed value to Tx FIFO
 * \param handle Module handle
 * \param mixEntryVal Mixed of Data and configuration
 * \return None
 */
IFX_INLINE void IfxQspi_SpiMaster_writeMixedDataConfiguration(IfxQspi_SpiMaster *handle, uint32 mixEntryVal);

/** \brief Writes the data to TxFIFO
 * \param chHandle Module Channel handle
 * \param data Data to be entered in Tx FIFO
 * \return None
 */
IFX_INLINE void IfxQspi_SpiMaster_writeTransmitFifo(IfxQspi_SpiMaster_Channel *chHandle, uint32 data);

/** \} */

/******************************************************************************/
/*---------------------Inline Function Implementations------------------------*/
/******************************************************************************/

IFX_INLINE uint32 IfxQspi_SpiMaster_readReceiveFifo(IfxQspi_SpiMaster *handle)
{
    Ifx_QSPI *qspiSFR = handle->qspi;
    uint32    data    = IfxQspi_readReceiveFifo(qspiSFR);
    return data;
}


IFX_INLINE void IfxQspi_SpiMaster_writeBasicConfiguration(IfxQspi_SpiMaster *handle, uint32 baconVal)
{
    Ifx_QSPI *qspiSFR = handle->qspi;
    IfxQspi_writeBasicConfiguration(qspiSFR, baconVal);
}


IFX_INLINE void IfxQspi_SpiMaster_writeExtendedConfiguration(IfxQspi_SpiMaster_Channel *chHandle, uint32 econVal)
{
    IfxQspi_writeExtendedConfiguration(chHandle->module->qspi, chHandle->channelId, econVal);
}


IFX_INLINE void IfxQspi_SpiMaster_writeMixedDataConfiguration(IfxQspi_SpiMaster *handle, uint32 mixEntryVal)
{
    Ifx_QSPI *qspiSFR = handle->qspi;
    IfxQspi_writeMixedDataTransmitFifo(qspiSFR, mixEntryVal);
}


IFX_INLINE void IfxQspi_SpiMaster_writeTransmitFifo(IfxQspi_SpiMaster_Channel *chHandle, uint32 data)
{
    IfxQspi_writeTransmitFifo(chHandle->module->qspi, chHandle->channelId, data);
}


#endif /* IFXQSPI_SPIMASTER_H */
