/**
 * \file IfxStm.c
 * \brief STM basic functionality
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1532.3
 * $Date: $2013-06-24 18:37:20 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 */

/******************************************************************************/
#include "Stm/Std/IfxStm.h"
#include "Scu/Std/IfxScuCcu.h"
#include "Src/Std/IfxSrc.h"

/******************************************************************************/
/** \brief Initialise stm compare register.
 *
 * \param stm System timer module
 * \param cfg pointer to configuration structure
 */
boolean IfxStm_initCompare(Ifx_STM *stm, IfxStm_CompareConfig *cfg)
{
    sint32        index;
    boolean       result;
    Ifx_STM_CMCON comcon = stm->CMCON;
    Ifx_STM_ICR   icr    = stm->ICR;

    index = IfxStm_getIndex(stm);

    if ((cfg->triggerInterruptEnabled > 0) && (index >= 0))
    {                           /* configure interrupt */
        volatile Ifx_SRC_SRCR *srcr;

        if (cfg->comparatorInterrupt == IfxStm_ComparatorInterrupt_Ir0)
        {
            srcr = &(MODULE_SRC.STM.STM[index].SR0);
        }
        else
        {
            srcr = &(MODULE_SRC.STM.STM[index].SR1);
        }

        IfxSrc_init(srcr, cfg->servProvider, cfg->triggerInterruptEnabled);
        IfxSrc_enable(srcr);
    }

    /*Configure the comparator ticks */
    stm->CMP[cfg->comparator].U = IfxStm_getOffsetTimer(stm, cfg->compareOffset) + cfg->ticks;

    if (cfg->comparator == 0)
    {
        comcon.B.MSIZE0  = cfg->compareSize;
        comcon.B.MSTART0 = cfg->compareOffset;
        icr.B.CMP0OS     = cfg->comparatorInterrupt;
        icr.B.CMP0EN     = cfg->triggerInterruptEnabled ? 1 : 0;
        result           = TRUE;
    }
    else if (cfg->comparator == 1)
    {
        comcon.B.MSIZE1  = cfg->compareSize;
        comcon.B.MSTART1 = cfg->compareOffset;
        icr.B.CMP1OS     = cfg->comparatorInterrupt;
        icr.B.CMP1EN     = cfg->triggerInterruptEnabled ? 1 : 0;
        result           = TRUE;
    }
    else
    {
        /*Invalid value */
        result = FALSE;
    }

    stm->ICR   = icr;
    stm->CMCON = comcon;

    return result;
}


/** \brief Initialise compare configuration with default values.
 *
 * \param cfg pointer to configuration structure
 */
void IfxStm_initCompareConfig(IfxStm_CompareConfig *cfg)
{
    cfg->comparator              = IfxStm_Comparator_0;
    cfg->compareOffset           = IfxStm_ComparatorOffset_0;
    cfg->compareSize             = IfxStm_ComparatorSize_32Bits;
    cfg->triggerInterruptEnabled = FALSE;
    cfg->comparatorInterrupt     = IfxStm_ComparatorInterrupt_Ir0; /*User must select the interrupt output */
    cfg->ticks                   = 0xFFFFFFFF;
    /* TODO add interrupt configuration */
}


/** \brief API to get the resource index of the STM specified.
 *
 * \param stm System timer module
 *
 * \return Returns system timer resource index.
 */
IfxStm_ResourceStm IfxStm_getIndex(Ifx_STM *stm)
{
    uint32             index;
    IfxStm_ResourceStm result;

    result = IfxStm_ResourceStm_none;

    for (index = 0; index < IFXSTM_COUNT; index++)
    {
        if (IfxStm_cfg_indexMap[index].module == stm)
        {
            result = IfxStm_cfg_indexMap[index].index;
            break;
        }
    }

    return result;
}


/** \brief Return the system timer frequency
 *
 * \return Return the system timer frequency in Hz
 */
float32 IfxStm_getFrequency(Ifx_STM *stm)
{
    float32 result;

    result = IfxScuCcu_getStmFrequency();

    return result;
}
