/**
 * \file IfxStm.h
 * \brief This file contains the APIs for System Timer related functions.
 * \ingroup IfxLld_Stm
 *
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 * $Revision: $1532.4
 * $Date: $2013-06-24 18:37:28 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * \defgroup IfxLld_Stm STM
 * \ingroup IfxLld
 *
 */

#ifndef IFXSTM_TIMER_H
#define IFXSTM_TIMER_H
/******************************************************************************/
#include "_Impl/IfxStm_cfg.h"
#include "Src/Std/IfxSrc.h"
/******************************************************************************/
/*                           Type definitions                              */
/******************************************************************************/

/** \addtogroup IfxLld_Stm
 * \{ */
/** \name Comparator Configuration Structure
 * \{ */
/** \brief Comparator Configuration Structure
 */
typedef struct
{
    Priority                   triggerInterruptEnabled;     /**< \brief Interrupt priority. Range = 0 .. 255. 0 = interrupt is disabled */
    IfxStm_ComparatorInterrupt comparatorInterrupt;         /*Interrupt request output */
    IfxStm_Comparator          comparator;                  /*Comparator Selection */
    IfxStm_ComparatorOffset    compareOffset;               /* Bit offset for compare operation with STM timer */
    IfxStm_ComparatorSize      compareSize;                 /*Data size for compare operation in bits */
    uint32                     ticks;                       /*count for next comparison from current timer count */
    IfxSrc_Tos                 servProvider;                /*Type of service */
} IfxStm_CompareConfig;

/** \} */
/******************************************************************************/
/*                           Function prototypes                              */
/******************************************************************************/

/** \name Timer functions
 * \{ */
IFX_EXTERN float32 IfxStm_getFrequency(Ifx_STM *stm);
IFX_INLINE uint32  IfxStm_getLower(Ifx_STM *stm);
IFX_INLINE uint32  IfxStm_getOffsetTimer(Ifx_STM *stm, uint8 offset);
IFX_INLINE uint32  IfxStm_getOffset4Timer(Ifx_STM *stm);
IFX_INLINE uint32  IfxStm_getOffset8Timer(Ifx_STM *stm);
IFX_INLINE uint32  IfxStm_getOffset12Timer(Ifx_STM *stm);
IFX_INLINE uint32  IfxStm_getOffset16Timer(Ifx_STM *stm);
IFX_INLINE uint32  IfxStm_getOffset20Timer(Ifx_STM *stm);
IFX_INLINE uint32  IfxStm_getOffset32Timer(Ifx_STM *stm);
/** \} */

/** \name Compare functions
 * \{ */
IFX_EXTERN void    IfxStm_initCompareConfig(IfxStm_CompareConfig *cfg);
IFX_EXTERN boolean IfxStm_initCompare(Ifx_STM *stm, IfxStm_CompareConfig *cfg);
IFX_INLINE void    IfxStm_updateCompare(Ifx_STM *stm, IfxStm_Comparator comparator, uint32 ticks);
IFX_INLINE uint32  IfxStm_getCompare(Ifx_STM *stm, IfxStm_Comparator comparator);
/** \} */

/** \name Other functions
 * \{ */
IFX_EXTERN IfxStm_ResourceStm IfxStm_getIndex(Ifx_STM *stm);
IFX_INLINE void               IfxStm_enableOcdsSuspend(Ifx_STM *stm);
/** \} */

/** \} */

/******************************************************************************/
/*                           Functions                                         */
/******************************************************************************/
/** \brief enable suspend by debugger.
 *
 * \param stm System timer module
 */
IFX_INLINE void IfxStm_enableOcdsSuspend(Ifx_STM *stm)
{
    Ifx_STM_OCS ocs = stm->OCS;

    ocs.B.SUS_P      = 1;
    ocs.B.SUS        = 2;
    stm->OCS         = ocs;
    stm->OCS.B.SUS_P = 0;
}


/** \brief Return system timer value.
 *
 * \param stm System timer module
 *
 * \return Returns system timer value.
 */
IFX_INLINE uint64 IfxStm_get(Ifx_STM *stm)
{
    uint64 result;

    result  = stm->TIM0.U;
    result |= ((uint64)stm->CAP.U) << 32;

    return result;
}


/** \brief Return the lower system timer value.
 *
 * \param stm System timer module
 * \return Returns the lower system timer value.
 */
IFX_INLINE uint32 IfxStm_getLower(Ifx_STM *stm)
{
    return stm->TIM0.U;
}


/** \brief Update the compare register value.
 *
 * \param stm System timer module
 * \param comparator selection comparator
 */
IFX_INLINE void IfxStm_updateCompare(Ifx_STM *stm, IfxStm_Comparator comparator, uint32 ticks)
{
    stm->CMP[comparator].B.CMPVAL = ticks;
}


/** \brief Update the compare register value increased with given ticks.
 *
 * \param stm System timer module
 * \param comparator selection comparator
 */
IFX_INLINE void IfxStm_increaseCompare(Ifx_STM *stm, IfxStm_Comparator comparator, uint32 ticks)
{
    stm->CMP[comparator].B.CMPVAL = stm->CMP[comparator].B.CMPVAL + ticks;
}


/** \brief Update the compare register value.
 *
 * \param stm System timer module
 * \param comparator selection comparator
 * \return Returns the compare value.
 */
IFX_INLINE uint32 IfxStm_getCompare(Ifx_STM *stm, IfxStm_Comparator comparator)
{
    return stm->CMP[comparator].B.CMPVAL;
}


/** \brief Clear the compare interrupt flag.
 *
 * \param stm System timer module
 * \param comparator selection comparator
 */
IFX_INLINE void IfxStm_clearCompareFlag(Ifx_STM *stm, IfxStm_Comparator comparator)
{
    if (comparator == IfxStm_Comparator_0)
    {
        stm->ISCR.B.CMP0IRR = 1U;
    }
    else if (comparator == IfxStm_Comparator_1)
    {
        stm->ISCR.B.CMP1IRR = 1U;
    }
}


/** \brief Return the timer value shifted right by offset.
 *
 * \param stm System timer module
 * \param offset offset value
 * \return Returns the lower system timer value shifted by offset
 */
IFX_INLINE uint32 IfxStm_getOffsetTimer(Ifx_STM *stm, uint8 offset)
{
    uint64 now;

    now = IfxStm_get(stm);

    return (uint32)(now >> offset);
}


IFX_INLINE uint32 IfxStm_getOffset4Timer(Ifx_STM *stm)
{
    return stm->TIM1.U;
}


IFX_INLINE uint32 IfxStm_getOffset8Timer(Ifx_STM *stm)
{
    return stm->TIM2.U;
}


IFX_INLINE uint32 IfxStm_getOffset12Timer(Ifx_STM *stm)
{
    return stm->TIM3.U;
}


IFX_INLINE uint32 IfxStm_getOffset16Timer(Ifx_STM *stm)
{
    return stm->TIM4.U;
}


IFX_INLINE uint32 IfxStm_getOffset20Timer(Ifx_STM *stm)
{
    return stm->TIM5.U;
}


IFX_INLINE uint32 IfxStm_getOffset32Timer(Ifx_STM *stm)
{
    return stm->TIM6.U;
}


/** Wait for requested time
 *
 * The macro waits in while loop for the specified time in system timer ticks
 * \param stm System timer module
 * \param ticks Wait time in system timer ticks
 * \return None
 *
 */
IFX_INLINE void IfxStm_waitTicks(Ifx_STM *stm, uint32 ticks)
{
    uint32 beginTime;

    beginTime = IfxStm_getLower(stm);

    /*below code will work because of unsigned 32 bit calculation even at timer wrapping condition
     * As an example if beginTime = 0xFFFFFFFE and current time = 2 (after overflow), unsigned calculation
     * 2 - 0xFFFFFFFE will be 4*/
    while ((IfxStm_getLower(stm) - beginTime) < ticks)
    {}
}


/******************************************************************************/
#endif /* IFXSTM_TIMER_H */
