/**
 * \file IfxVadc_Adc.c
 * \brief VADC functions
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 *
 * $Revision: $1532.6
 * $Date: $2013-07-08 19:53:56 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 */

/*________________________________________________________________________________________ */
/* INCLUDES */

#include "IfxVadc_Adc.h"

/*________________________________________________________________________________________ */
/* LOCAL FUNCTION IMPLEMENTATIONS (MODULE BASE ADDRESS) */

/** \brief Enable write access to the VADC config/control registers
 * \param vadc pointer to the base of VADC registers
 * \param protectionSet Index of the bit in the ACCPROTx {x=0/1} for which write access is to be enabled
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_accessEnable(Ifx_VADC *vadc, IfxVadc_Protection protectionSet)
{
    uint16 passwd = IfxScuWdt_getCpuWatchdogPassword();

    IfxScuWdt_clearSafetyEndinit(passwd);

    if (protectionSet < IFXVADC_MAXIMUM_BITFIELDS_IN_ACCPROT0_REGISTER)
    {
        vadc->ACCPROT0.U &= ~(0x00000001 << protectionSet);
    }
    else
    {
        vadc->ACCPROT1.U &= ~(0x00000001 << (protectionSet & 0x1F));
    }

    IfxScuWdt_setSafetyEndinit(passwd);
}


/** \brief Disable write access to the VADC config/control registers
 * \param vadc pointer to the base of VADC registers
 * \param protectionSet Index of the bit in the ACCPROTx {x=0/1} for which write access is to be disabled
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_accessDisable(Ifx_VADC *vadc, IfxVadc_Protection protectionSet)
{
    uint16 passwd = IfxScuWdt_getCpuWatchdogPassword();

    IfxScuWdt_clearSafetyEndinit(passwd);

    if (protectionSet < IFXVADC_MAXIMUM_BITFIELDS_IN_ACCPROT0_REGISTER)
    {
        vadc->ACCPROT0.U |= (0x00000001 << protectionSet);
    }
    else
    {
        vadc->ACCPROT1.U |= (0x00000001 << (protectionSet & 0x1F));
    }

    IfxScuWdt_setSafetyEndinit(passwd);
}


/** \brief Disable post calibration for the specified group
 * \param vadc pointer to the base of VADC registers
 * \param group Index of the group
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_disablePostCalibration(Ifx_VADC *vadc, uint8 group)
{
    IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_globalConfig);
    vadc->GLOBCFG.U |= (0x00010000 << group);
    IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_globalConfig);
}


/** \brief Enable VADC kernel
 * \param vadc pointer to the base of VADC registers
 * \return void
 */
static void IfxVadc_Adc_enableModule(Ifx_VADC *vadc)
{
    uint16 passwd = IfxScuWdt_getCpuWatchdogPassword();

    IfxScuWdt_clearCpuEndinit(passwd);
    vadc->CLC.U = 0x00000000;
    IfxScuWdt_setCpuEndinit(passwd);
}


/** \brief Reset VADC kernel
 * \param vadc pointer to the base of VADC registers
 * \return void
 */
static void IfxVadc_Adc_resetModule(Ifx_VADC *vadc)
{
    uint16 passwd = IfxScuWdt_getCpuWatchdogPassword();

    IfxScuWdt_clearSafetyEndinit(passwd);
    IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_globalConfig);
    vadc->KRST1.B.RST = 1;      /* Only if both Kernel reset bits are set a reset is executed */
    vadc->KRST0.B.RST = 1;

    while (vadc->KRST0.B.RSTSTAT == 0)
    {
        /* Wait until reset is executed */
    }

    vadc->KRSTCLR.B.CLR = 1;    /* Clear Kernel reset status bit */
    IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_globalConfig);
    IfxScuWdt_clearSafetyEndinit(passwd);
}


/** \brief Configure the Fadci vadc analog clock.
 * \param vadc pointer to the base of VADC registers
 * \param FAdcI Specifies the required ADC analog clock clock frequency in Hz. Range = [5000000, 10000000].
 * \return Returns the configured Fadci vadc analog clock frequency in Hz.
 */
static uint32 IfxVadc_Adc_initializeFAdcI(Ifx_VADC *vadc, uint32 fAdcI)
{
    uint32 divA;
    uint32 result;
    uint32 fadc = IfxScuCcu_getSpbFrequency();

    /*    DivA = min(max(0, Fadc / FAdcI - 1), 0x3F); */
    divA = (fadc << 2) / fAdcI;

    if ((divA & 0x3) >= 2)
    {
        divA += 4;
    }                           /* Round to nearest integer */
    else
    {
        /* do nothing */
    }

    divA = divA >> 2;

    if ((divA - 1) > 0x1Fu)
    {
        divA = 0x1Fu;
    }
    else
    {
        divA = (divA - 1);
    }

    result = fadc / (divA + 1);

    if (result > IFXVADC_ANALOG_FREQUENCY_MAX)
    {
        if ((divA + 1) > 0x1Fu)
        {
            divA = 0x1Fu;
        }
        else
        {
            divA = (divA + 1);
        }

        result = fadc / (divA + 1);
    }
    else
    {
        /* do nothing */
    }

    if (!((result >= IFXVADC_ANALOG_FREQUENCY_MIN) && (result <= IFXVADC_ANALOG_FREQUENCY_MAX)))
    {
        result = 0;             /* Min / Max FAdcI frequency */
    }
    else
    {
        Ifx_VADC_GLOBCFG tempGLOBCFG;
        tempGLOBCFG.U       = (vadc->GLOBCFG.U & 0xFFFFFFE0u);
        tempGLOBCFG.B.DIVA  = divA;
        tempGLOBCFG.B.DIVWC = 1;
        IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_globalConfig);
        vadc->GLOBCFG.U     = tempGLOBCFG.U;
        IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_globalConfig);
    }

    return result;
}


/** \brief Configure the FadcD vadc digital clock.
 * \param vadc pointer to the base of VADC registers
 * \param FAdcD Specifies the required ADC digital clock clock frequency in Hz.
 * \return Returns the configured Fadcd vadc digital clock frequency in Hz.
 */
static uint32 IfxVadc_Adc_initializeFAdcD(Ifx_VADC *vadc, uint32 fAdcD)
{
    uint32 divD;
    uint32 result;
    uint32 fadc = IfxScuCcu_getSpbFrequency();

    divD = (fadc / fAdcD - 1);

    if (divD > 0x3u)
    {
        divD = 0x3u;
    }
    else
    {
        /* do nothing */
    }

    result              = fadc / (divD + 1);
    Ifx_VADC_GLOBCFG tempGLOBCFG;
    tempGLOBCFG.U       = (vadc->GLOBCFG.U & 0xFFFFFCFFu);
    tempGLOBCFG.B.DIVD  = divD;
    tempGLOBCFG.B.DIVWC = 1;
    IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_globalConfig);
    vadc->GLOBCFG.U     = tempGLOBCFG.U;
    IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_globalConfig);

    return result;
}


/** \brief Returns the configured Fadci vadc analog clock frequency in Hz.
 * \param vadc pointer to the base of VADC registers
 * \return Returns the configured Fadci vadc analog clock frequency in Hz.
 */
static uint32 IfxVadc_Adc_getAnalogFrequency(Ifx_VADC *vadc)
{
    uint32 fadc = IfxScuCcu_getSpbFrequency();

    fadc = fadc / (vadc->GLOBCFG.B.DIVA + 1);

    return fadc;
}


/** \brief Start ADC calibration and wait for the end of the calibration process.
 * \param vadc pointer to the base of VADC registers
 * \return void
 */
static void IfxVadc_Adc_calibrate(Ifx_VADC *vadc)
{
    boolean calibration = FALSE;
    uint8   i;

    /* Start calibration */
    IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_globalConfig);
    vadc->GLOBCFG.B.SUCAL = 1;  /* Set SUCAL bit */
    IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_globalConfig);

    /* Wait for hardware self-test and calibration to complete */
    /* Wait until Calibration is done */
    while (calibration != TRUE) /* wait until calibration of all calibrated kernels are done */
    {
        calibration = TRUE;

        for (i = 0; i < IFXVADC_NUM_ADC_GROUPS; i++)
        {
            if (vadc->G[i].ARBCFG.B.CAL != 0)   /* Check ADC Calibration Flag CAL */
            {
                calibration = FALSE;
            }
            else
            {
                /* do nothing */
            }
        }
    }
}


/*________________________________________________________________________________________ */
/* LOCAL FUNCTION IMPLEMENTATIONS (GROUP BASE ADDRESS) */

IFX_INLINE void IfxVadc_Adc_G_clearAllResultRequests(Ifx_VADC_G *vadc)
{
    vadc->REFCLR.U = 0x0000FFFFu;
}


IFX_INLINE void IfxVadc_Adc_G_clearAllChannelRequests(Ifx_VADC_G *vadc)
{
    vadc->CEFCLR.U = 0x000000FFu;
}


IFX_INLINE void IfxVadc_Adc_G_SetArbiterPriority(Ifx_VADC_G *vadc, boolean slotEnable, IfxVadc_ReqSlotPrio prio, IfxVadc_RequestSlotStartMode mode, IfxVadc_RequestSource slot)
{
    if (slotEnable != FALSE)
    {
        vadc->ARBPR.U |= slotEnable << (IFX_VADC_G_ARBPR_ASEN0_OFF + slot); /* enable Slot */
        vadc->ARBPR.U &= ~(IFX_VADC_G_ARBPR_CSM0_OFF << (slot * 4u));       /* clear Priority */
        vadc->ARBPR.U |= (prio << (slot * 4u));                             /* Set Priority */

        if (mode != IfxVadc_RequestSlotStartMode_waitForStart)
        {
            vadc->ARBPR.U |= 0x1u << (IFX_VADC_G_ARBPR_CSM0_OFF + (slot * 4u)); /* Set cancel inject mode */
        }
        else
        {
            vadc->ARBPR.U &= ~(0x1u << (IFX_VADC_G_ARBPR_CSM0_OFF + (slot * 4u)));  /* Set Wait for Start mode */
        }
    }
    else
    {
        vadc->ARBPR.U &= ~(slotEnable << (IFX_VADC_G_ARBPR_ASEN0_OFF + slot));  /* disable Slot */
    }
}


/*________________________________________________________________________________________*/
/* LOCAL FUNCTION IMPLEMENTATIONS (HELPER) */

static Ifx_SRC_SRCR *IfxVadc_Adc_getSrcAddress(IfxVadc_GroupId group, IfxVadc_SrcNr index)
{
    Ifx_SRC_SRCR *base;

    if (IfxVadc_SrcNr_shared0 <= index)
    {
        index -= 4;

        if ((group & 0x1) != 0)
        {
#if defined(IFX_TC24x) || defined(IFX_TC23x) || defined(IFX_TC22x)
	    group = IfxVadc_GroupId_global0;    /* FIXME */
#else
            group = IfxVadc_GroupId_global1;    /* Shared interrupt common 1 is used */
#endif
        }
        else
        {
            group = IfxVadc_GroupId_global0;    /* Shared interrupt common 0 is used */
        }
    }
    else
    {
        /* do nothing */
    }

    base = (Ifx_SRC_SRCR *)IfxVadc_cfg_srcAddresses[(group * 4) + index];

    return &(base[index]);
}


static uint32 IfxVadc_Adc_calculateSampleTimeTicks(uint32 analogFrequency, float32 sampleTime)
{
    uint32 ticks;

    ticks = (uint32)((float32)sampleTime * analogFrequency);

    if (0xFFu < ticks)
    {
        ticks = 0xFFu;
    }
    else
    {
        /* do nothing */
    }

    return ticks;
}


typedef uint8 IfxVadc_Adc_SYNCTR_STSEL;

/** Configuration value for SYNCTR.STSEL
 * First index is slave
 * Second index is master */

#if IFXVADC_NUM_ADC_GROUPS == 4
static const IfxVadc_Adc_SYNCTR_STSEL IfxVadc_Adc_masterIndex[IFXVADC_NUM_ADC_GROUPS][IFXVADC_NUM_ADC_GROUPS] = {
    /* 0   1   2   3   4   5   6   7 */
    {0, 1, 2, 3},               /* Grp 0 */
    {1, 0, 2, 3},               /* Grp 1 */
    {1, 2, 0, 3},               /* Grp 2 */
    {1, 2, 3, 0},               /* Grp 3 */
};
#elif IFXVADC_NUM_ADC_GROUPS == 8
static const IfxVadc_Adc_SYNCTR_STSEL IfxVadc_Adc_masterIndex[IFXVADC_NUM_ADC_GROUPS][IFXVADC_NUM_ADC_GROUPS] = {
    /* 0   1   2   3   4   5   6   7 */
    {0, 1, 2, 3, 0, 0, 0, 0},   /* Grp 0 */
    {1, 0, 2, 3, 0, 0, 0, 0},   /* Grp 1 */
    {1, 2, 0, 3, 0, 0, 0, 0},   /* Grp 2 */
    {1, 2, 3, 0, 0, 0, 0, 0},   /* Grp 3 */
    {0, 0, 0, 0, 0, 1, 2, 3},   /* Grp 4 */
    {0, 0, 0, 0, 1, 0, 2, 3},   /* Grp 5 */
    {0, 0, 0, 0, 1, 2, 0, 3},   /* Grp 6 */
    {0, 0, 0, 0, 1, 2, 3, 0},   /* Grp 7 */
};
#else
# error "not prepared for this IFXVADC_NUM_ADC_GROUPS"
#endif

IFX_INLINE IfxVadc_Adc_SYNCTR_STSEL IfxVadc_Adc_getMasterKernelIndex(IfxVadc_GroupId slave, IfxVadc_GroupId master)
{
    return IfxVadc_Adc_masterIndex[slave][master];
}


IFX_INLINE IfxVadc_GroupId IfxVadc_Adc_getMasterId(IfxVadc_GroupId slave, IfxVadc_Adc_SYNCTR_STSEL masterIndex)
{
    uint8           i, idxOffset;
    IfxVadc_GroupId masterId = slave;

    if (masterIndex == 0)
    {
        masterId = slave;
    }
    else
    {
        idxOffset = (slave < 4) ? 0 : 4;    /* 4 is the index in the above IfxVadc_Adc_masterIndex about which symmetry is observed */

        for (i = 0; i < 4; i++)
        {
            if (IfxVadc_Adc_masterIndex[slave][i + idxOffset] == masterIndex)
            {
                return i + idxOffset;
            }
        }
    }

    return masterId;
}


/*________________________________________________________________________________________ */
/* PUBLIC FUNCTION IMPLEMENTATIONS (USING MODULE HANDLE) */

void IfxVadc_Adc_deInitModule(IfxVadc_Adc *vadc)
{
    Ifx_VADC *vadcSFR = vadc->vadc;

    IfxVadc_Adc_resetModule(vadcSFR);
}


void IfxVadc_Adc_getModuleConfig(IfxVadc_Adc *vadc, IfxVadc_Adc_Config *config)
{
    uint32 i;
    uint32 divD;
    uint32 fadc = IfxScuCcu_getSpbFrequency();

    config->vadc            = vadc->vadc;
    config->analogFrequency = IfxVadc_Adc_getAnalogFrequency(vadc->vadc);

    for (i = 0; i < IFXVADC_NUM_ADC_GROUPS; i++)
    {
        config->disablePostCalibrationGroup[i] = (vadc->vadc->GLOBCFG.U >> (16 + i)) & 0x1;
    }

    divD                    = (vadc->vadc->GLOBCFG.U >> 8) & 0x3;
    config->moduleFrequency = (fadc / (divD + 1));
}


IfxVadc_Status IfxVadc_Adc_initModule(IfxVadc_Adc *vadc, const IfxVadc_Adc_Config *config)
{
    uint8          i;
    IfxVadc_Status status  = IfxVadc_Status_noError;
    Ifx_VADC      *vadcSFR = config->vadc;

    vadc->vadc = vadcSFR;

    IfxVadc_Adc_enableModule(vadcSFR);

    /* Set Analog Frequency */
    if (IfxVadc_Adc_initializeFAdcI(vadcSFR, config->analogFrequency) == 0)
    {
        return IfxVadc_Status_notInitialised;
    }
    else
    {
        /* do nothing */
    }

    /* Set digital Frequency */
    IfxVadc_Adc_initializeFAdcD(vadcSFR, config->moduleFrequency);

    /* Set Post Calibration */
    for (i = 0; i < IFXVADC_NUM_ADC_GROUPS; i++)
    {
        if (config->disablePostCalibrationGroup[i] != FALSE)
        {
            IfxVadc_Adc_disablePostCalibration(vadcSFR, i);
        }
        else
        {
            /* do nothing */
        }
    }

    /* Enable all groups */
    for (i = 0; i < IFXVADC_NUM_ADC_GROUPS; i++)
    {
        vadcSFR->G[i].ARBCFG.B.ANONC = IFXVADC_NORMAL_MODE; /* turn on group */
    }

    /* Start up calibration is requested */
    IfxVadc_Adc_calibrate(vadcSFR);

    return status;
}


void IfxVadc_Adc_initModuleConfig(IfxVadc_Adc_Config *config, Ifx_VADC *vadc)
{
    uint8 i;

    config->vadc            = vadc;
    config->analogFrequency = IFXVADC_DEFAULT_ANALOG_FREQ;

    for (i = 0; i < IFXVADC_NUM_ADC_GROUPS; i++)
    {
        config->disablePostCalibrationGroup[i] = FALSE;
    }

    config->moduleFrequency = IfxScuCcu_getSpbFrequency();
}


/*________________________________________________________________________________________ */
/* FUNCTION IMPLEMENTATIONS (USING GROUP HANDLE) */

IFX_INLINE Ifx_VADC *IfxVadc_Adc_getVadcFromGroup(const IfxVadc_Adc_Group *group)
{
    return group->module.vadc;
}


IFX_INLINE Ifx_VADC_G *IfxVadc_getGroupRegsFromGroup(const IfxVadc_Adc_Group *group)
{
    return group->group;
}


void IfxVadc_Adc_deInitGroup(IfxVadc_Adc_Group *group)
{
    Ifx_VADC       *vadc       = IfxVadc_Adc_getVadcFromGroup(group);
    Ifx_VADC_G     *vadcG      = IfxVadc_getGroupRegsFromGroup(group);
    /* Get group index */
    IfxVadc_GroupId groupIndex = group->groupId;

    /* Request Access to configuration registers */
    IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_initGroup0 + groupIndex);
    vadcG->ARBCFG.B.ANONC = IFXVADC_ANALOG_CONVERTER_OFF;   /* turn off group */
    IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_initGroup0 + groupIndex);
}


void IfxVadc_Adc_getGroupConfig(IfxVadc_Adc_Group *group, IfxVadc_Adc_GroupConfig *config)
{
    uint8       i;
    Ifx_VADC_G *vadcG           = group->group;
    Ifx_VADC   *vadc            = group->module.vadc;
    uint32      analogFrequency = IfxVadc_Adc_getAnalogFrequency(vadc);

    config->groupId = group->groupId;
    config->module  = &group->module;

    for (i = 0; i < IFXVADC_NUMBER_OF_REQUESTSLOTS; i++)
    {
        config->arbiter.usedReqSlots[i] = (vadcG->ARBPR.U >> (24 + i)) & 0x1;
        config->arbiter.reqSlotPrio[i]  = (vadcG->ARBPR.U >> (i * 4)) & 0x3;
        config->arbiter.reqSlotMode[i]  = (vadcG->ARBPR.U >> (i * 4 + 3)) & 0x1;
    }

    config->arbiter.arbiterRoundLength = vadcG->ARBCFG.B.ARBRND;

    config->inputClass[0].resolution   = vadcG->ICLASS[0].B.CMS;
    config->inputClass[0].sampleTime   = (float32)vadcG->ICLASS[0].B.STCS / analogFrequency;
    config->inputClass[1].resolution   = vadcG->ICLASS[1].B.CMS;
    config->inputClass[1].sampleTime   = (float32)vadcG->ICLASS[1].B.STCS / analogFrequency;

    /* Setup queue request if enabled */

    if (config->arbiter.usedReqSlots[IfxVadc_RequestSource_scan] != FALSE)
    {
        config->scanRequest.triggerConfig.triggerInput = (vadcG->ASCTRL.U >> 8) & 0xF;
        config->scanRequest.triggerConfig.triggermode  = (vadcG->ASCTRL.U >> 13) & 0x3;

        if (config->scanRequest.triggerConfig.triggerInput != IfxVadc_TriggerSource_15)
        {
            config->scanRequest.triggerConfig.gatemode = vadcG->ASMR.B.ENGT;
        }
        else if (config->scanRequest.triggerConfig.triggermode != IfxVadc_TriggerMode_noExternalTrigger)
        {
            config->scanRequest.triggerConfig.gatemode = IfxVadc_GatingMode_always;
        }

        config->scanRequest.triggerConfig.gateInput = (vadcG->ASCTRL.U >> 16) & 0xF;
        config->scanRequest.autoscanEnable          = ((vadcG->ASMR.B.SCAN == 1) ? TRUE : FALSE);
    }
    else
    {
        config->scanRequest.autoscanEnable             = FALSE;
        config->scanRequest.triggerConfig.gatemode     = IfxVadc_GatingMode_disabled;
        config->scanRequest.triggerConfig.gateInput    = IfxVadc_GatingSource_0;     /* Use CCU6061 TRIG0 */
        config->scanRequest.triggerConfig.triggermode  = IfxVadc_TriggerMode_noExternalTrigger;
        config->scanRequest.triggerConfig.triggerInput = IfxVadc_TriggerSource_0;    /* Trigger source taken from Gating Input */
    }

    if (config->arbiter.usedReqSlots[IfxVadc_RequestSource_queue] != FALSE)
    {
        config->scanRequest.triggerConfig.triggerInput = (vadcG->QCTRL0.U >> 8) & 0xF;
        config->scanRequest.triggerConfig.triggermode  = (vadcG->QCTRL0.U >> 13) & 0x3;

        if (config->scanRequest.triggerConfig.triggerInput != IfxVadc_TriggerSource_15)
        {
            config->scanRequest.triggerConfig.gatemode = vadcG->QMR0.B.ENGT;
        }
        else if (config->scanRequest.triggerConfig.triggermode != IfxVadc_TriggerMode_noExternalTrigger)
        {
            config->scanRequest.triggerConfig.gatemode = IfxVadc_GatingMode_always;
        }

        config->scanRequest.triggerConfig.gateInput = (vadcG->QCTRL0.U >> 16) & 0xF;
        config->queueRequest.flushQueueAfterInit    = FALSE;
    }
    else
    {
        config->queueRequest.flushQueueAfterInit        = FALSE;
        config->queueRequest.triggerConfig.gatemode     = IfxVadc_GatingMode_disabled;
        config->queueRequest.triggerConfig.gateInput    = IfxVadc_GatingSource_0;    /* Use CCU6061 TRIG0 */
        config->queueRequest.triggerConfig.triggermode  = IfxVadc_TriggerMode_noExternalTrigger;
        config->queueRequest.triggerConfig.triggerInput = IfxVadc_TriggerSource_0;   /* Trigger source taken from Gating Input */
    }

    if (config->arbiter.usedReqSlots[IfxVadc_RequestSource_background] != FALSE)
    {
        config->scanRequest.triggerConfig.triggerInput = (vadc->BRSCTRL.U >> 8) & 0xF;
        config->scanRequest.triggerConfig.triggermode  = (vadc->BRSCTRL.U >> 13) & 0x3;

        if (config->scanRequest.triggerConfig.triggerInput != IfxVadc_TriggerSource_15)
        {
            config->scanRequest.triggerConfig.gatemode = vadc->BRSMR.B.ENGT;
        }
        else if (config->scanRequest.triggerConfig.triggermode != IfxVadc_TriggerMode_noExternalTrigger)
        {
            config->scanRequest.triggerConfig.gatemode = IfxVadc_GatingMode_always;
        }

        config->scanRequest.triggerConfig.gateInput = (vadc->BRSCTRL.U >> 16) & 0xF;
        config->scanRequest.autoscanEnable          = ((vadc->BRSMR.B.SCAN == 1) ? TRUE : FALSE);
    }
    else
    {
        config->backgroundScanReq.autoscanEnable             = FALSE;
        config->backgroundScanReq.triggerConfig.gatemode     = IfxVadc_GatingMode_disabled;
        config->backgroundScanReq.triggerConfig.gateInput    = IfxVadc_GatingSource_0;   /* Use CCU6061 TRIG0 */
        config->backgroundScanReq.triggerConfig.triggermode  = IfxVadc_TriggerMode_noExternalTrigger;
        config->backgroundScanReq.triggerConfig.triggerInput = IfxVadc_TriggerSource_0;  /* Trigger source taken from Gating Input */
    }

    config->master = IfxVadc_Adc_getMasterId(group->groupId, group->group->SYNCTR.B.STSEL);
}


IfxVadc_Status IfxVadc_Adc_initGroup(IfxVadc_Adc_Group *group, const IfxVadc_Adc_GroupConfig *config)
{
    IfxVadc_Status status = IfxVadc_Status_noError;
    Ifx_VADC      *vadc   = config->module->vadc;
    Ifx_VADC_G    *vadcG  = &vadc->G[config->groupId];

    /* check for write access */
    group->group  = vadcG;
    group->module = *config->module;
    uint8           i;
    IfxVadc_GroupId groupIndex      = config->groupId;
    group->groupId = config->groupId;
    uint32          analogFrequency = IfxVadc_Adc_getAnalogFrequency(vadc);
    /* Request Access to configuration registers */
    IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_initGroup0 + groupIndex);

    /* Setup Arbitration priority and turn on enabled slots */
    for (i = 0; i < IFXVADC_NUMBER_OF_REQUESTSLOTS; i++)
    {
        if (config->arbiter.usedReqSlots[i] != FALSE)
        {
            /* Enable Arbiter slot, set Priority and start mode */
            IfxVadc_Adc_G_SetArbiterPriority(vadcG, config->arbiter.usedReqSlots[i], config->arbiter.reqSlotPrio[i],
                config->arbiter.reqSlotMode[i], i);
        }
        else
        {
            /* do nothing */
        }
    }

    /* master slave configuration - what does this do ? */
    if (config->master != group->groupId)
    {
        uint8 masterIndex = IfxVadc_Adc_getMasterKernelIndex(group->groupId, config->master);
        vadcG->SYNCTR.B.STSEL = masterIndex;
        vadcG->SYNCTR.U      |= (0x00000008u << masterIndex);
    }
    else
    {
        /* do nothing */
    }

    /* Setup arbiter */
    vadcG->ARBCFG.B.ANONC  = IFXVADC_NORMAL_MODE;   /* turn on group */
    vadcG->ARBCFG.B.ARBRND = config->arbiter.arbiterRoundLength;

    /* Setup queue request if enabled */
    if (config->arbiter.usedReqSlots[IfxVadc_RequestSource_queue] != FALSE)
    {
        const IfxVadc_Adc_QueueConfig *queueSlot = &config->queueRequest;

        /* configure external Trigger if enabled */
        if (queueSlot->triggerConfig.triggermode != IfxVadc_TriggerMode_noExternalTrigger)
        {
            uint32 ctrlConfig = vadcG->QCTRL0.U & 0xFFFF00FF;
            vadcG->QMR0.B.ENTR = 1; /* enable external trigger */
            vadcG->QCTRL0.U    =
                ctrlConfig | (queueSlot->triggerConfig.triggermode << 13) | (queueSlot->triggerConfig.
                                                                             triggerInput << 8) | 0x8000u;

            if (queueSlot->triggerConfig.triggerInput == IfxVadc_TriggerSource_15)   /* if last input is used the trigger input selection is extend by Gating inputs */
            {
                ctrlConfig         = vadcG->QCTRL0.U & 0xFF00FFFF;
                vadcG->QCTRL0.U    = ctrlConfig | (queueSlot->triggerConfig.gateInput << 16) | 0x800000u;
                vadcG->QMR0.B.ENGT = 0x01;
            }
            else
            {
                /* do nothing */
            }
        }
        else
        {
            /* do nothing */
        }

        /* configure Gating if enabled */
        if ((queueSlot->triggerConfig.triggerInput != IfxVadc_TriggerSource_15))
        {
            uint32 ctrlConfig = vadcG->QCTRL0.U & 0xFF00FFFF;
            vadcG->QMR0.B.ENGT = queueSlot->triggerConfig.gatemode;
            vadcG->QCTRL0.U    = ctrlConfig | (queueSlot->triggerConfig.gateInput << 16) | 0x800000u;
        }
        else
        {
            /* do nothing */
        }

        if (config->queueRequest.flushQueueAfterInit != FALSE)
        {
            vadcG->QMR0.B.FLUSH = 1;
        }
        else
        {
            /* do nothing */
        }
    }                           /* End if(config->arbiter.usedReqSlots[IfxVadc_RequestSource_queue] != FALSE) */
    else
    {
        /* do nothing */
    }

    if (config->arbiter.usedReqSlots[IfxVadc_RequestSource_scan] != FALSE)
    {
        const IfxVadc_Adc_ScanConfig *scanSlot = &config->scanRequest;

        /* configure external Trigger if enabled */
        if (scanSlot->triggerConfig.triggermode != IfxVadc_TriggerMode_noExternalTrigger)
        {
            uint32 ctrlConfig = vadcG->ASCTRL.U & 0xFFFF00FF;
            vadcG->ASMR.B.ENTR = 1; /* enable external trigger */
            vadcG->ASCTRL.U    =
                ctrlConfig | (scanSlot->triggerConfig.triggermode << 13) | (scanSlot->triggerConfig.
                                                                            triggerInput << 8) | 0x8000u;

            if (scanSlot->triggerConfig.triggerInput == IfxVadc_TriggerSource_15)    /* if last input is used the trigger input selection is extend by Gating inputs */
            {
                ctrlConfig         = vadcG->ASCTRL.U & 0xFF00FFFF;
                vadcG->ASCTRL.U    = ctrlConfig | (scanSlot->triggerConfig.gateInput << 16) | 0x800000u;
                vadcG->ASMR.B.ENGT = 0x01;
            }
            else
            {
                /* do nothing */
            }
        }
        else
        {
            /* do nothing */
        }

        /* configure Gating if enabled */
        if ((scanSlot->triggerConfig.triggerInput != IfxVadc_TriggerSource_15))
        {
            uint32 ctrlConfig = vadcG->ASCTRL.U & 0xFF00FFFF;
            vadcG->ASMR.B.ENGT = scanSlot->triggerConfig.gatemode;
            vadcG->ASCTRL.U    = ctrlConfig | (scanSlot->triggerConfig.gateInput << 16) | 0x800000u;
        }
        else
        {
            /* do nothing */
        }

        if (scanSlot->autoscanEnable != FALSE)
        {
            vadcG->ASMR.B.SCAN = 1;
        }
        else
        {
            /* do nothing */
        }
    }                           /* End if(config->arbiter.usedReqSlots[IfxVadc_RequestSource_scan] != FALSE) */
    else
    {
        /* do nothing */
    }

    if (config->arbiter.usedReqSlots[IfxVadc_RequestSource_background] != FALSE)
    {
        const IfxVadc_Adc_ScanConfig *scanSlot = &config->backgroundScanReq;

        /* configure external Trigger if enabled */
        if (scanSlot->triggerConfig.triggermode != IfxVadc_TriggerMode_noExternalTrigger)
        {
            uint32 ctrlConfig = vadc->BRSCTRL.U & 0xFFFF00FF;
            vadc->BRSMR.B.ENTR = 1; /* enable external trigger */
            vadc->BRSCTRL.U    =
                ctrlConfig | (scanSlot->triggerConfig.triggermode << 13) | (scanSlot->triggerConfig.
                                                                            triggerInput << 8) | 0x8000u;

            if (scanSlot->triggerConfig.triggerInput == IfxVadc_TriggerSource_15)    /* if last input is used the trigger input selection is extend by Gating inputs */
            {
                ctrlConfig         = vadc->BRSCTRL.U & 0xFF00FFFF;
                vadc->BRSCTRL.U    = ctrlConfig | (scanSlot->triggerConfig.gateInput << 16) | 0x800000u;
                vadc->BRSMR.B.ENGT = 0x01;
            }
            else
            {
                /* do nothing */
            }
        }
        else
        {
            /* do nothing */
        }

        /* configure Gating if enabled */
        if ((scanSlot->triggerConfig.triggerInput != IfxVadc_TriggerSource_15))
        {
            uint32 ctrlConfig = vadcG->ASCTRL.U & 0xFF00FFFF;
            vadc->BRSMR.B.ENGT = scanSlot->triggerConfig.gatemode;
            vadc->BRSCTRL.U    = ctrlConfig | (scanSlot->triggerConfig.gateInput << 16) | 0x800000u;
        }
        else
        {
            /* do nothing */
        }

        if (scanSlot->autoscanEnable != FALSE)
        {
            vadc->BRSMR.B.SCAN = 1;
        }
        else
        {
            /* do nothing */
        }
    }                           /* End if(config->arbiter.usedReqSlots[IfxVadc_RequestSource_scan] != FALSE) */

    /* configure Group input class registers */
    vadcG->ICLASS[0].B.CMS  = config->inputClass[0].resolution;
    /* Calculate Sample time ticks */
    vadcG->ICLASS[0].B.STCS = IfxVadc_Adc_calculateSampleTimeTicks(analogFrequency, config->inputClass[0].sampleTime);
    /* Configure Group input class registers */
    vadcG->ICLASS[1].B.CMS  = config->inputClass[1].resolution;
    /* Calculate Sample time ticks */
    vadcG->ICLASS[1].B.STCS = IfxVadc_Adc_calculateSampleTimeTicks(analogFrequency, config->inputClass[1].sampleTime);
    IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_initGroup0 + groupIndex);

    return status;
}


static const IfxVadc_Adc_GroupConfig IfxVadc_Adc_defaultGroupConfig = {
    .arbiter                        = {
        .arbiterRoundLength = IfxVadc_ArbitrationMode_4_slots,
        .reqSlotMode[0]     = IfxVadc_RequestSlotStartMode_cancelInjectRepeat,
        .reqSlotMode[1]     = IfxVadc_RequestSlotStartMode_waitForStart,
        .reqSlotMode[2]     = IfxVadc_RequestSlotStartMode_waitForStart,
        .reqSlotPrio[0]     = IfxVadc_ReqSlotPrio_highest,
        .reqSlotPrio[1]     = IfxVadc_ReqSlotPrio_low,
        .reqSlotPrio[2]     = IfxVadc_ReqSlotPrio_lowest,
        .usedReqSlots[0]    = TRUE,         /* Enable Queue mode */
        .usedReqSlots[1]    = TRUE,         /* Enable Scan mode */
        .usedReqSlots[2]    = FALSE,        /* Disable Background scan */
    },
    .backgroundScanReq              = {
        .autoscanEnable             = FALSE,
        .triggerConfig.gatemode     = IfxVadc_GatingMode_disabled,
        .triggerConfig.triggermode  = IfxVadc_TriggerMode_noExternalTrigger,
        .triggerConfig.gateInput    = IfxVadc_GatingSource_0,
        .triggerConfig.triggerInput = IfxVadc_TriggerSource_0,
    },
    .scanRequest                    = {
        .autoscanEnable             = FALSE,
        .triggerConfig.gatemode     = IfxVadc_GatingMode_always,
        .triggerConfig.gateInput    = IfxVadc_GatingSource_2,             /* Use CCU6061 TRIG0 */
        .triggerConfig.triggermode  = IfxVadc_TriggerMode_uponRisingEdge,
        .triggerConfig.triggerInput = IfxVadc_TriggerSource_15,           /* Trigger source taken from Gating Input */
    },
    .queueRequest                   = {
        .flushQueueAfterInit        = TRUE,
        .triggerConfig.gatemode     = IfxVadc_GatingMode_always,
        .triggerConfig.gateInput    = IfxVadc_GatingSource_2,   /* Use CCU6061 TRIG0  */
        .triggerConfig.triggermode  = IfxVadc_TriggerMode_uponFallingEdge,
        .triggerConfig.triggerInput = IfxVadc_TriggerSource_14, /* Trigger source taken from Gating Input */
    },
    .inputClass[0].resolution = IfxVadc_ChannelResolution_12bit,
    .inputClass[0].sampleTime = 1.0e-6,                        /* Set sample time to 1us */
    .inputClass[1].resolution = IfxVadc_ChannelResolution_12bit,
    .inputClass[1].sampleTime = 1.0e-6,                        /* Set sample time to 1us */
};

void IfxVadc_Adc_initGroupConfig(IfxVadc_Adc_GroupConfig *config, const IfxVadc_Adc *vadc)
{
    *config         = IfxVadc_Adc_defaultGroupConfig;
    config->groupId = IfxVadc_GroupId_0;
    config->module  = vadc;
    config->master  = config->groupId;
}


/*________________________________________________________________________________________ */
/* FUNCTION IMPLEMENTATIONS (USING CHANNEL HANDLE) */

void IfxVadc_Adc_getChannelConfig(IfxVadc_Adc_Channel *channel, IfxVadc_Adc_ChannelConfig *config)
{
    Ifx_VADC_G       *vadcG        = IfxVadc_getGroupRegsFromGroup(channel->group);
    IfxVadc_ChannelId channelIndex = channel->channel;

    config->channelId           = channel->channel;
    config->group               = channel->group;
    config->inputClass          = vadcG->CHCTR[channelIndex].B.ICLSEL;
    config->reference           = vadcG->CHCTR[channelIndex].B.REFSEL;
    config->resultRegister      = vadcG->CHCTR[channelIndex].B.RESREG;
    config->globalResultUsage   = vadcG->CHCTR[channelIndex].B.RESTBS;
    config->lowerBoundary       = vadcG->CHCTR[channelIndex].B.BNDSELL;
    config->upperBoundary       = vadcG->CHCTR[channelIndex].B.BNDSELU;
    config->boundaryMode        = vadcG->CHCTR[channelIndex].B.BNDSELX;
    config->limitCheck          = vadcG->CHCTR[channelIndex].B.CHEVMODE;
    config->synchonize          = vadcG->CHCTR[channelIndex].B.SYNC;
    config->backgroundChannel   = (vadcG->CHASS.U & (1 << channelIndex)) ? FALSE : TRUE;
    config->rightAlignedStorage = vadcG->CHCTR[channelIndex].B.RESPOS;
    /* Get Channel index */
    IfxVadc_GroupId groupIndex               = channel->group->groupId;
    uint32          channelServiceReqNodePtr = (vadcG->CEVNP0.U >> (channel->channel * 4)) & 0xF;
    Ifx_SRC_SRCR   *src                      = IfxVadc_Adc_getSrcAddress(groupIndex, channelServiceReqNodePtr);

    if (src->B.SRE == 1)
    {
        config->channelSrcNr        = channelServiceReqNodePtr;
        config->channelPriority     = src->B.SRPN;
        config->channelServProvider = src->B.TOS;
    }
    else
    {
        config->channelSrcNr        = 0;
        config->channelPriority     = 0;
        config->channelServProvider = 0;
    }

    uint32 resultServiceReqNodePtr;

    if (config->resultRegister < IfxVadc_ChannelResult_8)
    {
        resultServiceReqNodePtr = (vadcG->REVNP0.U >> (channel->channel * 4)) & 0xF;
    }
    else
    {
        resultServiceReqNodePtr = (vadcG->REVNP1.U >> ((channel->channel - IfxVadc_ChannelResult_8) * 4)) & 0xF;
    }

    src = IfxVadc_Adc_getSrcAddress(groupIndex, resultServiceReqNodePtr);

    if (src->B.SRE == 1)
    {
        config->resultSrcNr        = resultServiceReqNodePtr;
        config->resultPriority     = src->B.SRPN;
        config->resultServProvider = src->B.TOS;
    }
    else
    {
        config->resultSrcNr        = 0;
        config->resultPriority     = 0;
        config->resultServProvider = 0;
    }
}


IfxVadc_Status IfxVadc_Adc_initChannel(IfxVadc_Adc_Channel *channel, const IfxVadc_Adc_ChannelConfig *config)
{
    IfxVadc_Status    Status = IfxVadc_Status_noError;
    Ifx_VADC_G       *vadcG  = IfxVadc_getGroupRegsFromGroup(config->group);

    channel->group = config->group;
    /* Get Channel index */
    IfxVadc_GroupId   groupIndex   = channel->group->groupId;
    IfxVadc_ChannelId channelIndex = config->channelId;
    /* Request Access to configuration registers */
   // IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_channelControl0 + groupIndex);
    /* Configure Channel */
    vadcG->CHCTR[channelIndex].B.REFSEL   = config->reference;
    vadcG->CHCTR[channelIndex].B.RESREG   = config->resultRegister;
    vadcG->CHCTR[channelIndex].B.BNDSELL  = config->lowerBoundary;
    vadcG->CHCTR[channelIndex].B.BNDSELU  = config->upperBoundary;
    vadcG->CHCTR[channelIndex].B.SYNC     = config->synchonize;
    vadcG->CHCTR[channelIndex].B.ICLSEL   = config->inputClass;
    vadcG->CHCTR[channelIndex].B.CHEVMODE = config->limitCheck;
    vadcG->CHCTR[channelIndex].B.RESPOS   = config->rightAlignedStorage;

    //IfxVadc_Adc_accessEnable(vadc, IfxVadc_Protection_initGroup0 + groupIndex);

    if (config->backgroundChannel == FALSE)
    {
        vadcG->CHASS.U |= (1 << channelIndex);
    }

    //IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_initGroup0 + groupIndex);

    /* TK: renamed in newer XMLs */
    /* vadcG->CHCTR[channelIndex].B.RESTGT = config->globalResultUsage; */
    vadcG->CHCTR[channelIndex].B.RESTBS  = config->globalResultUsage;

    vadcG->CHCTR[channelIndex].B.BNDSELX = config->boundaryMode;

    if (config->channelPriority > 0)
    {
        Ifx_SRC_SRCR *src               = IfxVadc_Adc_getSrcAddress(groupIndex, config->channelSrcNr);
        uint32        serviceReqNodePtr = config->channelSrcNr << (channel->channel);
        vadcG->CEVNP0.U &= ~(serviceReqNodePtr);
        vadcG->CEVNP0.U |= (serviceReqNodePtr);
        IfxVadc_Adc_G_clearAllChannelRequests(vadcG);
        IfxSrc_init(src, config->channelServProvider, config->channelPriority);
        IfxSrc_enable(src);
    }
    else
    {
        /* do nothing */
    }

    if (config->resultPriority > 0)
    {
        uint32        serviceReqNodePtr;
        Ifx_SRC_SRCR *src = IfxVadc_Adc_getSrcAddress(groupIndex, config->resultSrcNr);
        vadcG->RCR[config->resultRegister].B.SRGEN = 1;

        if (config->resultRegister < IfxVadc_ChannelResult_8)
        {
            serviceReqNodePtr = config->resultSrcNr << (config->resultRegister * 4);
            vadcG->REVNP0.U  &= ~(serviceReqNodePtr);
            vadcG->REVNP0.U  |= (serviceReqNodePtr);
        }
        else
        {
            serviceReqNodePtr = config->resultSrcNr << ((config->resultRegister - IfxVadc_ChannelResult_8) * 4);
            vadcG->REVNP1.U  &= ~(serviceReqNodePtr);
            vadcG->REVNP1.U  |= (serviceReqNodePtr);
        }

        IfxVadc_Adc_G_clearAllResultRequests(vadcG);
        IfxSrc_init(src, config->resultServProvider, config->resultPriority);
        IfxSrc_enable(src);
    }
    else
    {
        /* do nothing */
    }

   // IfxVadc_Adc_accessDisable(vadc, IfxVadc_Protection_channelControl0 + groupIndex);
    /* Software initialization */
    channel->resultreg = config->resultRegister;
    channel->channel   = config->channelId;

    return Status;
}


static const IfxVadc_Adc_ChannelConfig IfxVadc_Adc_defaultChannelConfig = {
    .channelId           = IfxVadc_ChannelId_0,
    .group               = NULL_PTR,
    .inputClass          = IfxVadc_InputClass_group0,
    .reference           = IfxVadc_ChannelReference_standard,
    .resultRegister      = IfxVadc_ChannelResult_0,
    .globalResultUsage   = FALSE,
    .lowerBoundary       = IfxVadc_BoundarySelection_group0,
    .upperBoundary       = IfxVadc_BoundarySelection_group0,
    .boundaryMode        = IfxVadc_BoundaryExtension_standard,
    .limitCheck          = IfxVadc_LimitCheck_noCheck,
    .synchonize          = FALSE,
    .backgroundChannel   = FALSE,
    .rightAlignedStorage = FALSE,
    .resultPriority      = 0,
    .resultSrcNr         = IfxVadc_SrcNr_group0,
    .resultServProvider  = IfxSrc_Tos_cpu0,
    .channelPriority     = 0,
    .channelSrcNr        = IfxVadc_SrcNr_group0,
    .channelServProvider = IfxSrc_Tos_cpu0
};

void IfxVadc_Adc_initChannelConfig(IfxVadc_Adc_ChannelConfig *config, const IfxVadc_Adc_Group *group)
{
    *config       = IfxVadc_Adc_defaultChannelConfig;
    config->group = group;
}


/*________________________________________________________________________________________ */
/* EOF */
