/**
 * \file IfxVadc_Adc.h
 * \brief VADC functions
 *
 * \copyright Copyright (c) 2012 Infineon Technologies AG. All rights reserved.
 *
 *
 * $Revision: $0.7
 * $Date: $2013-08-07 14:30:33 GMT
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * \defgroup IfxLld_Vadc_Interface Interface Drivers
 * \ingroup  IfxLld_Vadc
 * \defgroup IfxLld_Vadc_Interface_DataStrucutures Data structures
 * \ingroup  IfxLld_Vadc_Interface
 * \defgroup IfxLld_Vadc_Interface_Module Module functions
 * \ingroup  IfxLld_Vadc_Interface
 * \defgroup IfxLld_Vadc_Interface_Group  Group functions
 * \ingroup  IfxLld_Vadc_Interface
 * \defgroup IfxLld_Vadc_Interface_Channel Channel functions
 * \ingroup  IfxLld_Vadc_Interface
 * \defgroup IfxLld_Vadc_Interface_BackGround_AutoScan Background autoScan functions
 * \ingroup  IfxLld_Vadc_Interface
 * \defgroup IfxLld_Vadc_Interface_ChannelScan Channel Scan functions
 * \ingroup  IfxLld_Vadc_Interface
 * \defgroup IfxLld_Vadc_Interface_Queue Queue functions
 * \ingroup  IfxLld_Vadc_Interface
 */

#ifndef IFXVADC_ADC_H_
#define IFXVADC_ADC_H_

#include <Vadc/Std/IfxVadc.h>
#include <_PinMap/IfxVadc_PinMap.h>

/** \addtogroup IfxLld_Vadc_Interface_DataStrucutures
 * \{ */

/** VADC handle data structure */
typedef struct
{
    Ifx_VADC *vadc;                          /**< \brief Specifies the pointer to the VADC module registers    */
} IfxVadc_Adc;

/** VADC module configuration structure */
typedef struct
{
    Ifx_VADC *vadc;                                                     /**< \brief Specifies the pointer to the VADC module registers    */
    uint32    moduleFrequency;                                          /**< \brief Specifies digital ADC Frequency    */
    uint32    analogFrequency;                                          /**< \brief Specifies analog ADC Frequency    */
    boolean   disablePostCalibrationGroup[IFXVADC_NUM_ADC_GROUPS];      /**< \brief Specifies    if calibration after conversion is disabled for GroupX    */
} IfxVadc_Adc_Config;

/** Request arbiter configuration structure */
typedef struct
{
    boolean                      usedReqSlots[IFXVADC_NUMBER_OF_REQUESTSLOTS];      /**< \brief Specifies used request slots    */
    IfxVadc_ReqSlotPrio          reqSlotPrio[IFXVADC_NUMBER_OF_REQUESTSLOTS];       /**< \brief Specifies priority of used request slots    */
    IfxVadc_RequestSlotStartMode reqSlotMode[IFXVADC_NUMBER_OF_REQUESTSLOTS];       /**< \brief Specifies start mode for request source    */
    IfxVadc_ArbitrationRounds    arbiterRoundLength;                                /**< \brief Specifies arbiter round length    */
} IfxVadc_Adc_ArbiterConfig;

/** Gating/Trigger configuration structure */
typedef struct
{
    IfxVadc_GatingSource  gateInput;                /**< \brief Specifies used gate input for group    */
    IfxVadc_TriggerSource triggerInput;             /**< \brief Specifies used Trigger input for group    */
    IfxVadc_GatingMode    gatemode;                 /**< \brief Specifies gating mode. High level, Low Level  or Gating disabled    */
    IfxVadc_TriggerMode   triggermode;              /**< \brief Specifies trigger mode. Rising, falling any edge leads to an trigger event    */
} IfxVadc_Adc_GatingTriggerConfig;

/** Input class configuration structure */
typedef struct
{
    IfxVadc_ChannelResolution resolution;           /**< \brief Specifies the conversion Mode 8,10,12Bit or 10bit fast compare    */
    float32                   sampleTime;           /**< \brief Specifies the requested sample time for input class    */
} IfxVadc_Adc_ClassConfig;

/** Scan mode configuration structure */
typedef struct
{
    IfxVadc_Adc_GatingTriggerConfig triggerConfig;  /**< \brief Specifies trigger and gating configuration    */
    boolean                         autoscanEnable; /**< \brief Specifies autoscan functionality     */
} IfxVadc_Adc_ScanConfig;

/** Queue configuration structure */
typedef struct
{
    IfxVadc_Adc_GatingTriggerConfig triggerConfig;          /**< \brief Specifies trigger and gating configuration    */
    boolean                         flushQueueAfterInit;    /**< \brief Specifies if the queue is flushed after configuration    */
} IfxVadc_Adc_QueueConfig;

/** Group configuration structure */
typedef struct
{
    IfxVadc_GroupId           groupId;                                      /**< \brief Specifies the group/kernel id */
    const IfxVadc_Adc        *module;                                       /**< \brief Specifies pointer to the IfxVadc_Adc module handle */
    IfxVadc_Adc_ArbiterConfig arbiter;                                      /**< \brief Specifies arbiter configuration    */
    IfxVadc_Adc_ClassConfig   inputClass[IFXVADC_NUMBER_OF_INPUTCLASS];     /**< \brief Specifies conversion settings one and two    */
    IfxVadc_Adc_ScanConfig    scanRequest;                                  /**< \brief Specifies scan mode configuration    */
    IfxVadc_Adc_QueueConfig   queueRequest;                                 /**< \brief Specifies queued mode configuration    */
    IfxVadc_Adc_ScanConfig    backgroundScanReq;                            /**< \brief Specifies back ground scan configuration    */
    IfxVadc_GroupId           master;                                       /**< \brief Specifies the master group if own group is different master slave is configured    */
} IfxVadc_Adc_GroupConfig;

/** Group handle data structure */
typedef struct
{
    IfxVadc_Adc     module;                         /**< \brief The VADC handle structure */
    Ifx_VADC_G     *group;                          /**< \brief Pointer to the group registers */
    IfxVadc_GroupId groupId;                        /**< \brief Specifies the group index */
} IfxVadc_Adc_Group;

/** Channel configuration structure */
typedef struct
{
    IfxVadc_ChannelId         channelId;                /**< \brief Specifies the channel index    */
    const IfxVadc_Adc_Group  *group;                    /**< \brief Specifies pointer to the IfxVadc_Adc_Group group handle    */
    IfxVadc_InputClasses      inputClass;               /**< \brief Specifies input class selection    */
    IfxVadc_ChannelReference  reference;                /**< \brief Specifies Reference selection    */
    IfxVadc_ChannelResult     resultRegister;           /**< \brief Specifies Result register selection    */
    boolean                   globalResultUsage;        /**< \brief Specifies storage in global result register    */
    IfxVadc_BoundarySelection lowerBoundary;            /**< \brief Specifies lower boundary selection    */
    IfxVadc_BoundarySelection upperBoundary;            /**< \brief Specifies upper boundary selection    */
    IfxVadc_BoundaryExtension boundaryMode;             /**< \brief Specifies Standard mode of fast compare mode    */
    IfxVadc_LimitCheck        limitCheck;               /**< \brief Specifies boundary band selection upper/lower    */
    boolean                   synchonize;               /**< \brief Specifies synchronized conversion channel    */
    boolean                   backgroundChannel;        /**< \brief Specifies channel is used as background channel    */
    boolean                   rightAlignedStorage;      /**< \brief Specifies result is right aligned    */
    Priority                  resultPriority;           /**< \brief Interrupt priority of the result trigger interrupt, if 0 the interrupt is disable */
    IfxVadc_SrcNr             resultSrcNr;              /**< \brief Service node of the result trigger */
    IfxSrc_Tos                resultServProvider;       /**< \brief Interrupt service provider for the result trigger interrupt */
    Priority                  channelPriority;          /**< \brief Interrupt priority of the channel trigger interrupt, if 0 the interrupt is disable */
    IfxVadc_SrcNr             channelSrcNr;             /**< \brief Service node of the channel trigger */
    IfxSrc_Tos                channelServProvider;      /**< \brief Interrupt service provider for the channel trigger interrupt */
} IfxVadc_Adc_ChannelConfig;

/** Channel handle data structure */
typedef struct
{
    const IfxVadc_Adc_Group *group;                 /**< \brief Specifies the group of the channel     */
    IfxVadc_ChannelId        channel;               /**< \brief Specifies the channel index            */
    IfxVadc_ChannelResult    resultreg;             /**< \brief Specifies allocated result register    */
} IfxVadc_Adc_Channel;

/** \} */
/** \addtogroup IfxLld_Vadc_Interface_Module
 * \{ */

/** \brief Reset the VADC module
 * \param vadc pointer to the VADC module
 * \return void
 */
IFX_EXTERN void IfxVadc_Adc_deInitModule(IfxVadc_Adc *vadc);

/** \brief Get the current VADC configuration (e.g. VADC frequency)
 * \param vadc pointer to the VADC module
 * \param config pointer to the VADC module configuration
 * \return void
 */
IFX_EXTERN void IfxVadc_Adc_getModuleConfig(IfxVadc_Adc *vadc, IfxVadc_Adc_Config *config);

/** \brief Get conversion result based on the Request Source. (Function does not care about the alignment)
 * value = raw * gain + offset.
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *    adcGroupConfig.scanRequest.autoscanEnable = TRUE;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel;
 *    IfxVadc_Adc_initChannel(&adcChannel, &adcChannelConfig);
 *
 *    uint32 channels = (1 << 2); // enable channel #2
 *    uint32 mask = (1 << 7) | (1 << 2); // modify the selection for channel #7 and #2; channel #7 will be disabled
 *    boolean continuous = TRUE; // use continuous autoscan
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel, TRUE);
 *
 *    // configure autoscan
 *    IfxVadc_Adc_setScan(&adcGroup, channels, mask, continuous);
 *
 *    // start the autoscan
 *    IfxVadc_Adc_startScan(&adcGroup);
 *
 *    // wait for valid result
 *    Ifx_VADC_RES resultChannel;
 *    do {
 *         resultChannel = IfxVadc_Adc_getResultBasedOnRequestSource(&adcGroup, IfxVadc_ChannelId2, IfxVadc_RequestSource_scan);
 *     } while( !resultChannel.B.VF );
 * \endcode
 *
 * \param group pointer to the VADC group
 * \param channel channel number
 * \param sourceType type of request source
 * \return scaled Conversion result
 */
IFX_INLINE Ifx_VADC_RES IfxVadc_Adc_getResultBasedOnRequestSource(IfxVadc_Adc_Group *group, IfxVadc_ChannelId channel, IfxVadc_RequestSource sourceType)
{
    return IfxVadc_getResultBasedOnRequestSource(group->module.vadc, group->group, channel, sourceType);
}


/** \brief Initialise the VADC to run with the expected frequency and calibration
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create module config
 *     IfxVadc_Adc_Config adcModuleConfig;
 *     IfxVadc_Adc_initModuleConfig(&adcModuleConfig, &vadc);
 *
 *    // change disablePostCalibrationGroup (default is FALSE, change to TRUE)
 *    for (i = 0; i < IFXVADC_NUM_CAL_ADC; i++)
 *    {
 *        adcModuleConfig.disablePostCalibrationGroup[i] = TRUE;
 *    }
 *
 *    IfxVadc_Adc_initModule(&vadc, &adcModuleConfig);
 * \endcode
 *
 * \param vadc pointer to the VADC module
 * \param config pointer to the VADC configuration
 * \return IfxVadc_Status
 */
IFX_EXTERN IfxVadc_Status IfxVadc_Adc_initModule(IfxVadc_Adc *vadc, const IfxVadc_Adc_Config *config);

/** \brief Initialise buffer with default VADC configuration
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create module config
 *     IfxVadc_Adc_Config adcModuleConfig;
 *     IfxVadc_Adc_initModuleConfig(&adcModuleConfig, &vadc);
 * \endcode
 *
 * \param config pointer to the VADC module configuration
 * \param vadc   pointer to the VADC
 * \return void
 */
IFX_EXTERN void IfxVadc_Adc_initModuleConfig(IfxVadc_Adc_Config *config, Ifx_VADC *vadc);

/** \} */
/** \addtogroup IfxLld_Vadc_Interface_Group
 * \{ */

/** \brief Reset the VADC group
 * \param group pointer to the VADC group
 * \return void
 */
IFX_EXTERN void IfxVadc_Adc_deInitGroup(IfxVadc_Adc_Group *group);

/** \brief Get the current group configuration (e.g. vadc frequency)
 * \param group pointer to the VADC group
 * \param config pointer to the VADC group configuration
 * \return void
 */
IFX_EXTERN void IfxVadc_Adc_getGroupConfig(IfxVadc_Adc_Group *group, IfxVadc_Adc_GroupConfig *config);

/** \brief Get conversion result for the group
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *    adcGroupConfig.scanRequest.autoscanEnable = TRUE;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig2;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig2, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig2.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel2;
 *    IfxVadc_Adc_initChannel(&adcChannel2, &adcChannelConfig2);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig5;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig5, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId5)
 *    adcChannelConfig5.channelId = IfxVadc_ChannelId5;
 *    // change channel result register (default is ChannelResult0, change to ChannelResult1)
 *    adcChannelConfig5.resultRegister = IfxVadc_ChannelResult1;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel5;
 *    IfxVadc_Adc_initChannel(&adcChannel5, &adcChannelConfig5);
 *
 *    uint32 channels = (1 << 5) | (1 << 2); // enable channel #5 and #2
 *    uint32 mask = (1 << 7) | (1 << 5) | (1 << 2); // modify the selection for channel #7, #5 and #2; channel #7 will be disabled
 *    boolean continuous = TRUE; //  continuous autoscan
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel2, TRUE);
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel5, TRUE);
 *
 *    // configure autoscan
 *    IfxVadc_Adc_setScan(&adcGroup, channels, mask, continuous);
 *
 *     // start the autoscan
 *     IfxVadc_Adc_startScan(&adcGroup);
 *
 *    // wait for conversion to finish
 *    IfxVadc_Status scanStatus;
 *    do
 *    {
 *        scanStatus = IfxVadc_Adc_getScanStatus(&adcGroup);
 *    } while(scanStatus==IfxVadc_Status_ChannelsStillPending);
 *
 *    // fetch the 2 results of conversion for group 0
 *   Ifx_VADC_RES results[10];
 *   IfxVadc_Adc_getGroupResult(&adcGroup, results, 0, 2);
 * \endcode
 *
 * \param group pointer to the VADC group
 * \param results pointer to scaled conversion results
 * \param resultOffset offset for the first result
 * \param numResults number of results
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_getGroupResult(IfxVadc_Adc_Group *group, Ifx_VADC_RES *results, uint32 resultOffset, uint32 numResults)
{
    IfxVadc_getGroupResult(group->group, results, resultOffset, numResults);
}


/** \brief Initialise the VADC group (also autoscan and queue modes) Slave Groups must initialize first.
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 * \endcode
 *
 * \param group pointer to the VADC group
 * \param config pointer to the VADC group configuration
 * \return IfxVadc_Status
 */
IFX_EXTERN IfxVadc_Status IfxVadc_Adc_initGroup(IfxVadc_Adc_Group *group, const IfxVadc_Adc_GroupConfig *config);

/** \brief Initialise buffer with default VADC configuration
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 * \endcode
 *
 * \param config pointer to the VADC group configuration
 * \param vadc pointer to the VADC module
 * \return void
 */
IFX_EXTERN void IfxVadc_Adc_initGroupConfig(IfxVadc_Adc_GroupConfig *config, const IfxVadc_Adc *vadc);

/** brief access function to enable/disable wait for read mode for result registers
 *
 * \param channel pointer to the VADC channel
 * \param resultIdx result register index
 * \param waitForRead wait for read mode enabled/disable
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_configureWaitForReadMode(IfxVadc_Adc_Channel *channel, boolean waitForRead)
{
    IfxVadc_configureWaitForReadMode(channel->group->group, channel->resultreg, waitForRead);
}


/** \} */
/** \addtogroup IfxLld_Vadc_Interface_Channel
 * \{ */

/** \brief Get the current channel configuration (e.g. sample settings)
 * \param channel pointer to the VADC channel
 * \param config pointer to the VADC channel configuration
 * \return void
 */
IFX_EXTERN void IfxVadc_Adc_getChannelConfig(IfxVadc_Adc_Channel *channel, IfxVadc_Adc_ChannelConfig *config);

/** \brief Get conversion result (Function does not care about the alignment)
 * value = raw * gain + offset.
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *    adcGroupConfig.scanRequest.autoscanEnable = TRUE;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel;
 *    IfxVadc_Adc_initChannel(&adcChannel, &adcChannelConfig);
 *
 *     uint32 channels = (1 << 2); // enable channel #2
 *     uint32 mask = (1 << 7) | (1 << 2); // modify the selection for channel #7 and #2; channel #7 will be disabled
 *     boolean continuous = TRUE; // use continuous autoscan
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel, TRUE);
 *
 *    // configure autoscan
 *    IfxVadc_Adc_setScan(&adcGroup, channels, mask, continuous);
 *
 *     // start the autoscan
 *     IfxVadc_Adc_startScan(&adcGroup);
 *
 *    // wait for valid result
 *    Ifx_VADC_RES resultChannel;
 *    do {
 *         resultChannel = IfxVadc_Adc_getResult(&adcChannel);
 *    } while( !resultChannel.B.VF );
 * \endcode
 *
 * \param channel pointer to the VADC channel
 * \param Index of Result register
 * \return scaled Conversion result
 */
IFX_INLINE Ifx_VADC_RES IfxVadc_Adc_getResult(IfxVadc_Adc_Channel *channel)
{
    return IfxVadc_getResult(channel->group->group, channel->resultreg);
}


/** \brief Initialise one channel with given configuration
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel;
 *    IfxVadc_Adc_initChannel(&adcChannel, &adcChannelConfig);
 * \endcode
 *
 * \param channel pointer to the VADC channel
 * \param config pointer to the VADC channel configuration
 * \return IfxVadc_Status
 */
IFX_EXTERN IfxVadc_Status IfxVadc_Adc_initChannel(IfxVadc_Adc_Channel *channel, const IfxVadc_Adc_ChannelConfig *config);

/** \brief Initialise buffer with default channel configuration
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig, &adcGroup);
 * \endcode
 *
 * \param config pointer to the VADC channel configuration
 * \param group pointer to the VADC group
 * \return void
 */
IFX_EXTERN void IfxVadc_Adc_initChannelConfig(IfxVadc_Adc_ChannelConfig *config, const IfxVadc_Adc_Group *group);

/** \} */
/** \addtogroup IfxLld_Vadc_Interface_BackGround_AutoScan
 * \{ */

/** \brief Gives the background scan status for a group
 * \param vadc pointer to the VADC module
 * \return IfxVadc_Status
 */
IFX_INLINE IfxVadc_Status IfxVadc_Adc_getBackgroundScanStatus(IfxVadc_Adc *vadc)
{
    return IfxVadc_getBackgroundScanStatus(vadc->vadc);
}


/** \brief returns result stored in global result register
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *    adcGroupConfig.master  = adcGroupConfig.groupId ;
 *    adcGroupConfig.backgroundScanReq.autoscanEnable = TRUE;
 *    adcGroupConfig.arbiter.usedReqSlots[2] = TRUE;
 *    adcGroupConfig.backgroundScanReq.triggerConfig.gatemode = IfxVadc_GatingMode_always;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig2;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig2, &adcGroup);
 *    adcChannelConfig2.backgroundChannel   = TRUE;
 *    adcChannelConfig2.globalResultUsage   = TRUE;
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig2.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel2;
 *    IfxVadc_Adc_initChannel(&adcChannel2, &adcChannelConfig2);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig5;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig5, &adcGroup);
 *    adcChannelConfig5.backgroundChannel   = TRUE;
 *    adcChannelConfig5.globalResultUsage   = TRUE;
 *    // change channel (default is ChannelId0, change to ChannelId5)
 *    adcChannelConfig5.channelId = IfxVadc_ChannelId5;
 *
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel5;
 *    IfxVadc_Adc_initChannel(&adcChannel5, &adcChannelConfig5);
 *
 *     uint32 channels = (1 << 5) | (1 << 2); // enable channel #5 and #2
 *     uint32 mask = (1 << 7) | (1 << 5) | (1 << 2); // modify the selection for channel #7, #5 and #2; channel #7 will be disabled
 *     boolean continuous = TRUE; // disable continuous background scan
 *     //configure wait for read mode
 *     IfxVadc_Adc_configureWaitForReadModeForGlobalResultRegister(&vadc, TRUE);
 *
 *     // configure background scan
 *     IfxVadc_Adc_setBackgroundScan(&vadc, &adcGroupConfig, channels, mask);
 *
 *     // start the background scan
 *     IfxVadc_Adc_startBackgroundScan(&vadc, continuous);
 *
 *     // wait for valid result for channel 2
 *     Ifx_VADC_GLOBRES resultChannel2;
 *     do {
 *         resultChannel2 = IfxVadc_Adc_getGlobalResult(&vadc);
 *     } while( !resultChannel2.B.VF );
 *
 *     // wait for valid result for channel 5
 *     Ifx_VADC_GLOBRES resultChannel5;
 *     do {
 *         resultChannel5 = IfxVadc_Adc_getGlobalResult(&vadc);
 *     } while( !resultChannel5.B.VF );
 * \endcode
 *
 * \param vadc pointer to the VADC module
 * \return  global result register
 */
IFX_INLINE Ifx_VADC_GLOBRES IfxVadc_Adc_getGlobalResult(IfxVadc_Adc *vadc)
{
    return IfxVadc_getGlobalResult(vadc->vadc);
}


/** \brief configures a background scan; can also stop autoscan if all channels are 0
 * Background scan can be enabled/disabled for the given channels which are selected with the mask
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *    adcGroupConfig.master  = adcGroupConfig.groupId ;
 *    adcGroupConfig.backgroundScanReq.autoscanEnable = TRUE;
 *    adcGroupConfig.arbiter.usedReqSlots[2] = TRUE;
 *    adcGroupConfig.backgroundScanReq.triggerConfig.gatemode = IfxVadc_GatingMode_always;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig2;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig2, &adcGroup);
 *    adcChannelConfig2.backgroundChannel   = TRUE;
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig2.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel2;
 *    IfxVadc_Adc_initChannel(&adcChannel2, &adcChannelConfig2);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig5;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig5, &adcGroup);
 *    adcChannelConfig5.backgroundChannel   = TRUE;
 *
 *    // change channel (default is ChannelId0, change to ChannelId5)
 *    adcChannelConfig5.channelId = IfxVadc_ChannelId5;
 *    // change channel result register (default is ChannelResult0, change to ChannelResult1)
 *    adcChannelConfig5.resultRegister = IfxVadc_ChannelResult1;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel5;
 *    IfxVadc_Adc_initChannel(&adcChannel5, &adcChannelConfig5);
 *
 *     uint32 channels = (1 << 5) | (1 << 2); // enable channel #5 and #2
 *     uint32 mask = (1 << 7) | (1 << 5) | (1 << 2); // modify the selection for channel #7, #5 and #2; channel #7 will be disabled
 *     boolean continuous = TRUE; // disable continuous background scan
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel2, TRUE);
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel5, TRUE);
 *
 *     // configure background scan
 *     IfxVadc_Adc_setBackgroundScan(&vadc, &adcGroupConfig, channels, mask);
 *
 *     // start the background scan
 *     IfxVadc_Adc_startBackgroundScan(&vadc, continuous);
 *
 *     // wait for valid result for channel 2
 *     Ifx_VADC_RES resultChannel2;
 *     do {
 *         resultChannel2 = IfxVadc_Adc_getResult(&adcChannel2);
 *     } while( !resultChannel2.B.VF );
 *
 *     // wait for valid result for channel 5
 *     Ifx_VADC_RES resultChannel5;
 *     do {
 *         resultChannel5 = IfxVadc_Adc_getResult(&adcChannel5);
 *     } while( !resultChannel5.B.VF );
 * \endcode
 *
 * \param vadc pointer to the VADC module
 * \param group pointer to the VADC group
 * \param channels specifies the channels which should be enabled/disabled
 * \param mask specifies the channels which should be modified
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_setBackgroundScan(IfxVadc_Adc *vadc, IfxVadc_Adc_Group *group, uint32 channels, uint32 mask)
{
    IfxVadc_setBackgroundScan(vadc->vadc, group->groupId, channels, mask);
}


/** \brief Starts a background scan in either continuous or single shot conversion mode as per the parameter
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *    adcGroupConfig.backgroundScanReq.autoscanEnable = TRUE;
 *    adcGroupConfig.arbiter.usedReqSlots[2] = TRUE;
 *    adcGroupConfig.backgroundScanReq.triggerConfig.gatemode = IfxVadc_GatingMode_always;
 *
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig2;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig2, &adcGroup);
 *    adcChannelConfig2.backgroundChannel   = TRUE;
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig2.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel2;
 *    IfxVadc_Adc_initChannel(&adcChannel2, &adcChannelConfig2);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig5;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig5, &adcGroup);
 *    adcChannelConfig5.backgroundChannel   = TRUE;
 *
 *    // change channel (default is ChannelId0, change to ChannelId5)
 *    adcChannelConfig5.channelId = IfxVadc_ChannelId5;
 *    // change channel result register (default is ChannelResult0, change to ChannelResult1)
 *    adcChannelConfig5.resultRegister = IfxVadc_ChannelResult1;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel5;
 *    IfxVadc_Adc_initChannel(&adcChannel5, &adcChannelConfig5);
 *
 *    uint32 channels = (1 << 5) | (1 << 2); // enable channel #5 and #2
 *    uint32 mask = (1 << 7) | (1 << 5) | (1 << 2); // modify the selection for channel #7, #5 and #2; channel #7 will be disabled
 *    boolean continuous = TRUE; //  continuous background scan
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel2, TRUE);
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel5, TRUE);
 *
 *    // configure background scan
 *    IfxVadc_Adc_setBackgroundScan(&vadc, &adcGroupConfig, channels, mask);
 *
 *     // start the background scan
 *     IfxVadc_Adc_startBackgroundScan(&vadc, continuous);
 *
 *    // wait for valid result for channel 2
 *    Ifx_VADC_RES resultChannel2;
 *    do {
 *         resultChannel2 = IfxVadc_Adc_getResult(&adcChannel2);
 *    } while( !resultChannel2.B.VF );
 *
 *     // wait for valid result for channel 5
 *     Ifx_VADC_RES resultChannel5;
 *     do {
 *         resultChannel5 = IfxVadc_Adc_getResult(&adcChannel5);
 *     } while( !resultChannel5.B.VF );
 * \endcode
 *
 * \param vadc pointer to the VADC module
 * \param continuous continuous or single shot conversion
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_startBackgroundScan(IfxVadc_Adc *vadc, boolean continuous)
{
    IfxVadc_startBackgroundScan(vadc->vadc, continuous);
}


/** brief access function to enable/disable  wait for read mode for global result register
 * \param vadc pointer to the VADC module
 * \param waitForRead wait for read mode enabled/disabled
 * \ return void
 */
IFX_INLINE void IfxVadc_Adc_configureWaitForReadModeForGlobalResultRegister(IfxVadc_Adc *vadc, boolean waitForRead)
{
    IfxVadc_configureWaitForReadModeForGlobalResultRegister(vadc->vadc, waitForRead);
}


/** \} */
/** \addtogroup IfxLld_Vadc_Interface_ChannelScan
 * \{ */

/** \brief returns Gives the scan status for a group
 *
 * \code
 *     IfxVadc_Adc_Group adcGroup;
 *     adcGroup.group = &MODULE_VADC.G[0];
 *
 *     // get auto scan status for group 0
 *     boolean channelsStillPending;
 *     channelsStillPending = (IfxVadc_Adc_getScanStatus(&adcGroup)==IfxVadc_Status_ChannelsStillPending)?TRUE:FALSE;
 * \endcode
 *
 * \param group pointer to the VADC group
 * \return IfxVadc_Status
 */
IFX_INLINE IfxVadc_Status IfxVadc_Adc_getScanStatus(IfxVadc_Adc_Group *group)
{
    return IfxVadc_getScanStatus(group->group);
}


/** \brief Configures an autoscan
 * Autoscan can be enabled/disabled for the given channels which are selected with the mask
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *    adcGroupConfig.scanRequest.autoscanEnable  = TRUE;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig2;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig2, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel2;
 *    IfxVadc_Adc_initChannel(&adcChannel2, &adcChannelConfig2);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig5;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig5, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId5)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId5;
 *    // change channel result register (default is ChannelResult0, change to ChannelResult1)
 *    adcChannelConfig.resultRegister = IfxVadc_ChannelResult1;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel5;
 *    IfxVadc_Adc_initChannel(&adcChannel5, &adcChannelConfig5);
 *
 *     uint32 channels = (1 << 5) | (1 << 2); // enable channel #5 and #2
 *     uint32 mask = (1 << 7) | (1 << 5) | (1 << 2); // modify the selection for channel #7, #5 and #2; channel #7 will be disabled
 *     boolean continuous = TRUE; // use continuous autoscan
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel2, TRUE);
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel5, TRUE);
 *
 *    // configure autoscan
 *    IfxVadc_Adc_setScan(&adcGroup, channels, mask, continuous);
 *
 *     // start the autoscan
 *     IfxVadc_Adc_startScan(&adcGroup);
 *
 *    // wait for valid result for channel 2
 *    Ifx_VADC_RES resultChannel2;
 *    do {
 *         resultChannel2 = IfxVadc_Adc_getResult(&adcChannel2);
 *    } while( !resultChannel2.B.VF );
 *
 *    // wait for valid result for channel 5
 *    Ifx_VADC_RES resultChannel5;
 *    do {
 *         resultChannel5 = IfxVadc_Adc_getResult(&adcChannel5);
 *    } while( !resultChannel5.B.VF );
 * \endcode
 *
 * \param group pointer to the VADC group
 * \param channels specifies the channels which should be enabled/disabled
 * \param mask specifies the channels which should be modified
 * \param continuous specifies if single triggered or continuous autoscan
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_setScan(IfxVadc_Adc_Group *group, uint32 channels, uint32 mask, boolean continuous)
{
    IfxVadc_setScan(group->group, channels, mask, continuous);
}


/** \brief Starts an autoscan on the specified group
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *    adcGroupConfig.scanRequest.autoscanEnable  = TRUE;
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig2;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig2, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel2;
 *    IfxVadc_Adc_initChannel(&adcChannel2, &adcChannelConfig2);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig5;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig5, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId5)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId5;
 *    // change channel result register (default is ChannelResult0, change to ChannelResult1)
 *    adcChannelConfig.resultRegister = IfxVadc_ChannelResult1;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel5;
 *    IfxVadc_Adc_initChannel(&adcChannel5, &adcChannelConfig5);
 *
 *     uint32 channels = (1 << 5) | (1 << 2); // enable channel #5 and #2
 *     uint32 mask = (1 << 7) | (1 << 5) | (1 << 2); // modify the selection for channel #7, #5 and #2; channel #7 will be disabled
 *     boolean continuous = TRUE; // use continuous autoscan
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel2, TRUE);
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel5, TRUE);
 *
 *     // configure autoscan
 *     IfxVadc_Adc_setScan(&adcGroup, channels, mask, continuous);
 *
 *     // start the autoscan
 *     IfxVadc_Adc_startScan(&adcGroup);
 *
 *     // wait for valid result for channel 2
 *     Ifx_VADC_RES resultChannel2;
 *     do {
 *         resultChannel2 = IfxVadc_Adc_getResult(&adcChannel2);
 *     } while( !resultChannel2.B.VF );
 *
 *     // wait for valid result for channel 5
 *     Ifx_VADC_RES resultChannel5;
 *     do {
 *         resultChannel5 = IfxVadc_Adc_getResult(&adcChannel5);
 *     } while( !resultChannel5.B.VF );
 * \endcode
 *
 * \param group pointer to the VADC group
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_startScan(IfxVadc_Adc_Group *group)
{
    IfxVadc_startScan(group->group);
}


/** \} */
/** \addtogroup IfxLld_Vadc_Interface_Queue
 * \{ */

/** \brief Add an entry to the queue of a group for the specified channel with the following options set:
 *  refill
 *  source interrupt enable/disable
 * external trigger control
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel;
 *    IfxVadc_Adc_initChannel(&adcChannel, &adcChannelConfig);
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel, TRUE);
 *
 *    // Add channel 2 to queue of group 2 with the refill turned on
 *     IfxVadc_Adc_addToQueue(&adcChannel, IFXVADC_QUEUE_REFILL);
 *
 *     // start the Queue
 *     IfxVadc_Adc_startQueue(&adcGroup);
 *
 *     // wait for valid result
 *     Ifx_VADC_RES result;
 *     do {
 *         result = IfxVadc_Adc_getResult(&adcChannel);
 *     } while( !result.B.VF );
 * \endcode
 *
 * \param channel pointer to the VADC channel
 * \param options
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_addToQueue(IfxVadc_Adc_Channel *channel, uint32 options)
{
    IfxVadc_addToQueue(channel->group->group, channel->channel, options);
}


/** \brief Flush the contents of the queue of a group
 * \param group pointer to the VADC group
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_clearQueue(IfxVadc_Adc_Group *group)
{
    IfxVadc_clearQueue(group->group);
}


/** \brief Gives the status of the Queue of a group by returning non zero value if the Queue is full
 * \param group pointer to the VADC group
 * \return IfxVadc_Status
 */
IFX_INLINE IfxVadc_Status IfxVadc_Adc_getQueueStatus(IfxVadc_Adc_Group *group)
{
    return IfxVadc_getQueueStatus(group->group);
}


/** \brief Starts a queue of a group by generating a trigger event through software
 *
 * \code
 *    // initialize module pointer
 *    IfxVadc_Adc vadc;
 *    vadc.vadc = &MODULE_VADC;
 *
 *    // create group config
 *    IfxVadc_Adc_GroupConfig adcGroupConfig;
 *    IfxVadc_Adc_initGroupConfig(&adcGroupConfig, &vadc);
 *
 *    // change group (default is GroupId0, change to GroupId2)
 *    adcGroupConfig.groupId = IfxVadc_GroupId2;
 *
 *    // initialize the group
 *    IfxVadc_Adc_Group adcGroup;
 *    IfxVadc_Adc_initGroup(&adcGroup, &adcGroupConfig);
 *
 *    // create channel config
 *    IfxVadc_Adc_ChannelConfig adcChannelConfig;
 *    IfxVadc_Adc_initChannelConfig(&adcChannelConfig, &adcGroup);
 *
 *    // change channel (default is ChannelId0, change to ChannelId2)
 *    adcChannelConfig.channelId = IfxVadc_ChannelId2;
 *
 *    // initialize the channel
 *    IfxVadc_Adc_Channel adcChannel;
 *    IfxVadc_Adc_initChannel(&adcChannel, &adcChannelConfig);
 *
 *    // configure wait for read mode
 *    IfxVadc_Adc_configureWaitForReadMode(&adcChannel, TRUE);
 *
 *    // Add channel 2 to queue of group 2 with the refill turned on
 *     IfxVadc_Adc_addToQueue(&adcChannel, (1<<IFX_VADC_G_QBUR_QINR0_QBUR0_RF_OFF) );
 *
 *     // start the Queue
 *     IfxVadc_Adc_startQueue(&adcGroup);
 *
 *     // wait for valid result
 *     Ifx_VADC_RES result;
 *     do {
 *         result = IfxVadc_Adc_getResult(&adcChannel);
 *     } while( !result.B.VF );
 * \endcode
 *
 * \param group pointer to the VADC group
 * \return void
 */
IFX_INLINE void IfxVadc_Adc_startQueue(IfxVadc_Adc_Group *group)
{
    IfxVadc_startQueue(group->group);
}


/** \} */

#endif /* IFXADC_H_ */
